////////////////////////////////////////////////////////////////////////////////
//                               FastMines project
//                                       Sergey Krivulya (KSerg, aka SeregaLBN)
// file name: "Assistant.h"
//
//   CAssistant
// Copyright (C) 2006-2007 Sergey Krivulya
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
////////////////////////////////////////////////////////////////////////////////

#ifndef __FILE__ASSISTANT__
#define __FILE__ASSISTANT__

#include <Windows.h>
#include <vector>
#include "./Control/GradientCursor.h"
#include "./Mosaic/Base.h"

#define WORK_RESULT bool

class CMosaic;

class CAssistant {
private:
   /////////////////////
   // ---- TYPES -----
   /////////////////////
   typedef std::vector<int> VEC_Int;

   struct CClickData {
      COORD m_CoordCell;
      bool  m_bIsLeft;
      CCell::EClose m_Close; // only for right click
      float m_fProbability;
      COORD m_PrbltCell;
      bool  m_bSuggestClick;
      CClickData():
         m_CoordCell(INCORRECT_COORD),
         m_bIsLeft(true),
         m_Close(CCell::_Flag),
         m_fProbability(0.0),
         m_PrbltCell(INCORRECT_COORD),
         m_bSuggestClick(false) {}
   };

   class CSequentialMove {    // ,     ,      
      /////////////////////// //      .   !
      // ---- variables -----
      ///////////////////////
   private:                
      CAssistant &m_Assistant;
      bool        m_bSequentialMove;
      VEC_Int     m_VecSM; //       
      BYTE        m_iRow, m_iCol;
      int         m_iMineNoFlag;

      /////////////////////
      // ---- methods -----
      /////////////////////
   private:
      WORK_RESULT AddNextVectors(const int, const int iBitMask4SM);
      WORK_RESULT AnalyseVector(const int iBitMask4SM);
   public:
      CSequentialMove(CAssistant &Assistant) : m_Assistant(Assistant), m_bSequentialMove(false) {}
      bool        IsProcessed() {return m_bSequentialMove;}
      WORK_RESULT CanBegin();
      void        Work(CClickData&);
      void        Reset() { m_bSequentialMove = false; m_VecSM.clear(); m_iRow=m_iCol=0; m_iMineNoFlag=0; }
   };
   friend CSequentialMove;

   class CSync {
      friend CSequentialMove;
      ///////////////////////
      // ---- variables -----
      ///////////////////////
   private:
      CAssistant &m_Assistant;
      HANDLE m_hThread;                //      
      HANDLE m_hEventDestroyThread;    //     
      HANDLE m_hEventCanWork;          //     
      HANDLE m_hEventWorkAbort;        //      
      HANDLE m_hEventWorkAborted;      // ,    
      HANDLE m_hEventSuggestLeftClick; // ,      
      HANDLE m_hEventHaveClickData;    // ,      

      /////////////////////
      // ---- methods -----
      /////////////////////
   private:
      friend CAssistant;
      CSync(CAssistant &Assistant);
     ~CSync();

   public:
      // ,   
      void SetSuggest() { ::SetEvent(m_hEventSuggestLeftClick); }
      void StartWorkStage();
      void AbortAndWaitCurrentWorkStage(DWORD dwWaitTime = INFINITE); //        
      void AbortThread();
      bool HaveClickData();

   private:
      // ,    
      bool IsNeedAbortWorkStage(); // ,       
   };
   friend CSync;

   ///////////////////////
   // ---- variables -----
   ///////////////////////
private:
   CSync            m_Sync;
   CMosaic         &m_Mosaic;
   CSequentialMove  m_SM;
   CGradientCursor  m_Cursor;

   //         
   //      
   CCell::SET_cpBase m_SetForOpen;     //            
   CCell::SET_cpBase m_SetForFlag;     //            
   CCell::SET_cpBase m_SetForAnalyse;  //            (   )
   CCell::SET_cpBase m_SetCloseNoFlag; //     

   CClickData m_click;

   /////////////////////
   // ---- methods -----
   /////////////////////
private:
   inline bool TestCombination(const int, const CCell*, const CCell*, const CCell::SET_cpBase&) const;
   WORK_RESULT Analyse2Cell(const CCell*, const CCell*, OUT CClickData&);
   WORK_RESULT Analyse(OUT CClickData&);
   WORK_RESULT FindCell(OUT CClickData&);

   void Work(bool bSuggestClick);

   static DWORD WINAPI ThreadStatic(PVOID pvParam) { return ((CAssistant*)pvParam)->Thread(); }
   DWORD Thread();

public:
   void InitForNewGame(); //     
   void ClickEnd(const CCell::CClickReportContext &ClickReportContext);

   CAssistant(CMosaic& mosaic);
  ~CAssistant();

   CSync& Sync() { return m_Sync; }
   void OnClick();
};

#endif // __FILE__ASSISTANT__
