////////////////////////////////////////////////////////////////////////////////
//                               FastMines project
//                                       Sergey Krivulya (KSerg, aka SeregaLBN)
// file name: "EditGradient.cpp"
//   edit   
// Copyright (C) 2006-2007 Sergey Krivulya
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
////////////////////////////////////////////////////////////////////////////////

#include "StdAfx.h"
#include "EditGradient.h"
#include "../DynLoadLib.h"

inline COLORREF ChangeBrightness(COLORREF clrIn, double coeff) {
   CShlWAPILib::PF_ColorRGBToHLS funcColorRGBToHLS = CShlWAPILib::ColorRGBToHLS();
   CShlWAPILib::PF_ColorHLSToRGB funcColorHLSToRGB = CShlWAPILib::ColorHLSToRGB();
   if (funcColorRGBToHLS && funcColorHLSToRGB) {
      WORD wH=0, wL=0, wS=0;
      funcColorRGBToHLS(clrIn, &wH, &wL, &wS);
      return funcColorHLSToRGB(wH, wL*coeff, wS);
   } else {
      ::SetLastError(ERROR_CALL_NOT_IMPLEMENTED); // ERROR_OLD_WIN_VERSION ERROR_FUNCTION_NOT_CALLED ERROR_INVALID_FUNCTION
      return clrIn;
   }
}

COLORREF CEditGradient::GetBkColorDisable2 () const { COLORREF clr = ChangeBrightness(GetBkColorDisable (), 1.25); return clr; } // 
COLORREF CEditGradient::GetBkColorReadOnly2() const { COLORREF clr = ChangeBrightness(GetBkColorReadOnly(), 0.85); return clr; } // 
COLORREF CEditGradient::GetBkColorEnable2  () const { if (GetBkColorEnable() == 0xFFFFFF) return RGB(218,248,235);
                                                      COLORREF clr = ChangeBrightness(GetBkColorEnable  (), 0.75); return clr; }

#ifndef _WTL_VER
BOOL CEditGradient::DrawBkgrnd(HDC  hDC, const RECT &rc)
#else
BOOL CEditGradient::DrawBkgrnd(CDC &hDC, const RECT &rc)
#endif // _WTL_VER
{
   bool bFocus = (m_hWnd == ::GetFocus());
   COLORREF clr1 = bFocus ? GetBkColor2() : GetBkColor();
   COLORREF clr2 = bFocus ? GetBkColor()  : GetBkColor2();
   TRIVERTEX vert[] = {
      {
         rc.left,            // LONG    x;
         rc.top,             // LONG    y;
         GetRValue(clr1)<<8, // COLOR16 Red;   0x0000..0xff00
         GetGValue(clr1)<<8, // COLOR16 Green;
         GetBValue(clr1)<<8, // COLOR16 Blue;
         0x0000              // COLOR16 Alpha;
      }, {
         rc.right,           // LONG    x;
         rc.bottom,          // LONG    y;   
         GetRValue(clr2)<<8, // COLOR16 Red;   0x0000..0xff00
         GetGValue(clr2)<<8, // COLOR16 Green;
         GetBValue(clr2)<<8, // COLOR16 Blue;
         0x0000              // COLOR16 Alpha;
      }
   };
   GRADIENT_RECT gRc[] = {{0, 1}};
   ::GradientFill(hDC, vert, chDIMOF(vert), &gRc, 1, GRADIENT_FILL_RECT_V);
   return TRUE;
}
