////////////////////////////////////////////////////////////////////////////////
//                               FastMines project
//                                       Sergey Krivulya (KSerg, aka SeregaLBN)
// file name: "SelectPlayer.cpp"
//    "Players Administration" -  
// Copyright (C) 2004-2007 Sergey Krivulya
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
////////////////////////////////////////////////////////////////////////////////

#include "StdAfx.h"
#include "Administration.h"
#include "../FastMines2.h"
#include "../EraseBk.h"
#include "./DialogMessageHook.h"
#include "./Statistics.h"
#include "./InputText.h"
#include "./TemplateSelectImage.h"

////////////////////////////////////////////////////////////////////////////////
//                      global variables, types , defines, etc...
////////////////////////////////////////////////////////////////////////////////
extern CFastMines2WinMain& GetFM2WinMain();

WNDPROC_BUTTON(IDC_ADMINISTRATION_BUTTONCHECK_AUTOSTART, GetFM2WinMain().GetSkin())

////////////////////////////////////////////////////////////////////////////////
//                              implementation
////////////////////////////////////////////////////////////////////////////////
#ifndef _WTL_VER

// modal dialogs
int CDialogAdministration::DoModal(HWND hWndParent) {
   m_bModal = true;
   return ::DialogBoxParam(::GetModuleHandle(NULL), CDialogAdministration::IDD(), hWndParent, (DLGPROC)CDialogAdministration::DialogProcThis, (LPARAM)this);
}

// modeless dialogs
HWND CDialogAdministration::Create(HWND hWndParent) {
   m_bModal = false;
	return ::CreateDialogParam(::GetModuleHandle(NULL), CDialogAdministration::IDD(), hWndParent, (DLGPROC)CDialogAdministration::DialogProcThis, (LPARAM)this);
}

/* static */ INT_PTR CALLBACK CDialogAdministration::DialogProcThis(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
   CDialogAdministration *This = NULL;
   if (uMsg == WM_INITDIALOG) {
      This = (CDialogAdministration*)lParam;
      if (This) {
         This->m_hWnd = hWnd;
         ::SetWindowUserData(hWnd, (LONG_PTR)This);
      }
   } else {
      This = (CDialogAdministration*)::GetWindowUserData(hWnd);
   }
   if (This) {
      return This->DialogProc(hWnd, uMsg, wParam, lParam);
   }
   return FALSE;
}

BOOL CDialogAdministration::OnInitDialog(HWND, HWND hWndFocus, LPARAM lParam) {
   m_BtnOk         .SubclassWindow(::GetDlgItem(m_hWnd, IDOK));
   m_BtnCancel     .SubclassWindow(::GetDlgItem(m_hWnd, IDCANCEL));
   m_BtnNewPlayer  .SubclassWindow(::GetDlgItem(m_hWnd, IDC_ADMINISTRATION_BUTTON_NEWPLAYER));
   m_BtnNewPassword.SubclassWindow(::GetDlgItem(m_hWnd, IDC_ADMINISTRATION_BUTTON_NEWPASSWORD));
   m_BtnAvatar     .SubclassWindow(::GetDlgItem(m_hWnd, IDC_ADMINISTRATION_BUTTON_AVATAR));
   m_BtnRename     .SubclassWindow(::GetDlgItem(m_hWnd, IDC_ADMINISTRATION_BUTTON_RENAME));
   m_BtnRemove     .SubclassWindow(::GetDlgItem(m_hWnd, IDC_ADMINISTRATION_BUTTON_REMOVE));
   m_Table         .SubclassWindow(::GetDlgItem(m_hWnd, IDC_ADMINISTRATION_TABLE));
   Button_SetCheck(::GetDlgItem(m_hWnd, IDC_ADMINISTRATION_BUTTONCHECK_AUTOSTART), GetFM2WinMain().GetAutoloadAdmin() ? BST_CHECKED : BST_UNCHECKED);

   ::SetWindowText(                 m_hWnd                                           , CLang::GetString(IDS__DIALOG_ADMINISTRATION             ));
   ::SetWindowText(m_BtnOk         .m_hWnd                                           , CLang::GetString(IDS__DIALOG_ADMINISTRATION__SELECT     ));
   ::SetWindowText(m_BtnCancel     .m_hWnd                                           , CLang::GetString(IDS__CANCEL                            ));
   ::SetWindowText(m_BtnNewPlayer  .m_hWnd                                           , CLang::GetString(IDS__DIALOG_ADMINISTRATION__NEWPLAYER  ));
   ::SetWindowText(m_BtnNewPassword.m_hWnd                                           , CLang::GetString(IDS__DIALOG_ADMINISTRATION__NEWPASSWORD));
   ::SetWindowText(m_BtnAvatar     .m_hWnd                                           , CLang::GetString(IDS__DIALOG_ADMINISTRATION__AVATAR     ));
   ::SetWindowText(m_BtnRename     .m_hWnd                                           , CLang::GetString(IDS__DIALOG_ADMINISTRATION__RENAME     ));
   ::SetWindowText(m_BtnRemove     .m_hWnd                                           , CLang::GetString(IDS__DIALOG_ADMINISTRATION__REMOVE     ));
   ::SetWindowText(::GetDlgItem(    m_hWnd, IDC_ADMINISTRATION_BUTTONCHECK_AUTOSTART), CLang::GetString(IDS__DIALOG_ADMINISTRATION__AUTOSTART  ));

   ReloadTable();

   m_CloseWnd = !m_bModal;
   if (m_bModal) {
      ::SetWindowLong(m_hWnd, GWL_STYLE, WS_SYSMENU ^ ::GetWindowLong(m_hWnd, GWL_STYLE));
      ::EnableWindow(m_BtnCancel.m_hWnd, FALSE);
   }

   SETNEWWNDPROC(m_hWnd, IDC_ADMINISTRATION_BUTTONCHECK_AUTOSTART);

   if (!m_bModal) {
      CDialogMessageHook::InstallHook(m_hWnd);
   }

   return TRUE;
}

void CDialogAdministration::OnCommand(HWND, int idControl, HWND hWndCtl, UINT codeNotify) {
   switch (idControl) {
   case IDC_ADMINISTRATION_TABLE:
      switch (codeNotify) {
      case CTable::NC_SELECT_CELL: FORWARD_WM_COMMAND(m_hWnd, IDOK                            , m_Table.m_hWnd, BN_CLICKED, ::PostMessage); break;
      case CTable::NC_DELETE_CELL: FORWARD_WM_COMMAND(m_hWnd, IDC_ADMINISTRATION_BUTTON_REMOVE, m_Table.m_hWnd, BN_CLICKED, ::PostMessage); break;
      }
      return;
   case IDC_ADMINISTRATION_BUTTON_REMOVE:
      {
         CString strCurrUser = m_Table.GetCurrentText();
         if (strCurrUser.IsEmpty()) return;
         if (VerifyPassword()) {
            if (strCurrUser == GetFM2WinMain().GetPlayerName()) {
               //          . 
               m_CloseWnd = false;
               ::SetWindowLong(m_hWnd, GWL_STYLE, WS_SYSMENU ^ ::GetWindowLong(m_hWnd, GWL_STYLE));
               ::EnableWindow(m_BtnCancel.m_hWnd, FALSE);
               ::DrawMenuBar(m_hWnd);
            }
            GetFM2WinMain().GetPlayers().RemovePlayer(strCurrUser);
            ReloadTable();
         }
      }
      return;
   case IDC_ADMINISTRATION_BUTTON_AVATAR:
      {
         CString strCurrUser = m_Table.GetCurrentText();
         if (strCurrUser.IsEmpty()) return;
         if (VerifyPassword()) {
            CString strFileAvatar = GetFM2WinMain().GetPlayers().GetAvatar(strCurrUser);
            CDialogTemplateSelectImage dlgTmplSI(
               m_hWnd,
               NULL,
               strFileAvatar.IsEmpty() ? ::GetCurrentDirectory() : strFileAvatar, 
               CLang::GetString(IDS__DIALOG_CHANGESKIN__OPENFILENAME_TITLE_OPEN_IMAGE),
               NULL);
            if (IDOK == dlgTmplSI.DoModal()) {
               strFileAvatar = dlgTmplSI.m_strFile+dlgTmplSI.m_strResTypeAndName;
               if (!strFileAvatar.IsEmpty()) {
                  GetFM2WinMain().GetPlayers().SetAvatar(strCurrUser, strFileAvatar);
                  ReloadTable();
               }
            }
         }
      }
      return;
   case IDC_ADMINISTRATION_BUTTON_RENAME:
      {
         CString strCurrUser = m_Table.GetCurrentText();
         if (strCurrUser.IsEmpty()) return;
         if (VerifyPassword()) {
            CDialogInputText dlgInputText(CDialogInputText::ePlayerRename, strCurrUser);
            if (IDOK == dlgInputText.DoModal(m_hWnd)) {
               if (strCurrUser == GetFM2WinMain().GetPlayerName())
                  GetFM2WinMain().SetPlayerName(dlgInputText.m_strName);
               GetFM2WinMain().GetPlayers().RenamePlayer(strCurrUser, dlgInputText.m_strName);
               m_Table.SetText(0, m_Table.GetCurrentCell().Y, dlgInputText.m_strName);
            }
         }
      }
      return;
   case IDC_ADMINISTRATION_BUTTON_NEWPASSWORD:
      {
         CString strCurrUser = m_Table.GetCurrentText();
         if (strCurrUser.IsEmpty()) return;
         if (VerifyPassword()) {
            CDialogInputText dlgInputText(CDialogInputText::ePlayerNewPassword);
            if (IDOK == dlgInputText.DoModal(m_hWnd)) {
               GetFM2WinMain().GetPlayers().SetPassword(strCurrUser, dlgInputText.m_strPassword);
               m_Table.SetCurrentCell(0, 1+GetFM2WinMain().GetPlayers().FindName(strCurrUser), true);
            }
         }
      }
      return;
   case IDC_ADMINISTRATION_BUTTON_NEWPLAYER:
      {
         CDialogInputText dlgInputText(CDialogInputText::ePlayerNew);
         if (IDOK == dlgInputText.DoModal(m_hWnd)) {
            GetFM2WinMain().GetPlayers().AddNewRecord(dlgInputText.m_strName, dlgInputText.m_strPassword);
            ReloadTable();
            m_Table.SetCurrentCell(0, 1+GetFM2WinMain().GetPlayers().FindName(dlgInputText.m_strName), true);
         }
      }
      return;
   case IDOK:
      if (!VerifyPassword()) return;
      GetFM2WinMain().SetPlayerName(m_Table.GetCurrentText());
      m_CloseWnd = true;
   case IDCANCEL:
      ::SendMessage(m_hWnd, WM_CLOSE, 0L, 0L);
      return;
   }
}

void CDialogAdministration::OnClose(HWND) {
   if (!m_CloseWnd) return;


   GetFM2WinMain().SetAutoloadAdmin(BST_CHECKED == Button_GetCheck(::GetDlgItem(m_hWnd, IDC_ADMINISTRATION_BUTTONCHECK_AUTOSTART)));

   if (m_bModal) {
      ::EndDialog(m_hWnd, 0);
   } else {
      ::DestroyWindow(m_hWnd);
   }
}

void CDialogAdministration::OnDestroy(HWND) {
   if (!m_bModal) {
      CDialogMessageHook::UninstallHook(m_hWnd);
   }
   m_hWnd = NULL;
}

BOOL CDialogAdministration::OnEraseBkgnd(HWND, HDC hDC) {
   if (!GetFM2WinMain().GetSkin().m_bToAll)
   {
      return FALSE; // DefWindowProc(m_hWnd, WM_ERASEBKGND, (WPARAM)hDC, 0L);
   }
   return nsEraseBk::OnEraseBkgnd(m_hWnd, hDC, GetFM2WinMain().GetSkin().GetBkColor());
}

void CDialogAdministration::ReloadTable() {
   m_Table.SetColNumber(1);
   m_Table.SetRowNumber(max(7+1,GetFM2WinMain().GetPlayers().NumberPlayers()+1));
   m_Table.SetText(0,0, CLang::GetString(IDS__DIALOG_ADMINISTRATION__TABLE));
   {
      SHORT j=1;
      for (; j<m_Table.GetRowNumber(); j++) {
         m_Table.SetText (0, j, _T(""));
         m_Table.SetImage(0, j, NULL);
      }
      for (j=0; j<GetFM2WinMain().GetPlayers().NumberPlayers(); j++) {
         m_Table.SetText      (0, j+1, GetFM2WinMain().GetPlayers().GetPlayerName(j));
         m_Table.SetFormatText(0, j+1, DT_LEFT | DT_VCENTER | DT_SINGLELINE | DT_NOPREFIX);
         m_Table.SetImage     (0, j+1, &CImageSerialize(GetFM2WinMain().GetPlayers().GetAvatar(j)));
      }
      int index = GetFM2WinMain().GetPlayers().FindName(GetFM2WinMain().GetPlayerName());
      if (index>=0)
         m_Table.SetCurrentCell(0, index+1, true);
      else
         m_Table.SetCurrentCell(0,1, false);
   }
   RECT rcClient; ::GetClientRect(m_Table.m_hWnd, &rcClient);
   m_Table.SetColWidth(0, rcClient.right-rcClient.left);
}

bool CDialogAdministration::VerifyPassword() {
   if (!m_Table.GetCurrentText()[0]) {
      FORWARD_WM_COMMAND(m_hWnd, IDC_ADMINISTRATION_BUTTON_NEWPLAYER, m_hWnd, BN_CLICKED, ::PostMessage);
      return false;
   }
   CString strCurrUser = m_Table.GetCurrentText();
   if (strCurrUser == CDialogAdministration::DEFAULT_PLAYER_NAME()) {
      ::MessageBox(m_hWnd,
         CLang::GetString(IDS__DIALOG_ADMINISTRATION__THIS_VIRTUAL_PLAYER),
         CLang::GetString(IDS__ERROR),
         MB_OK | MB_ICONERROR);
      return false;
   }
   CString m_strPassword = GetFM2WinMain().GetPlayers().GetPassword(strCurrUser);
   if (!m_strPassword.IsEmpty()) {
      CDialogInputText dlgInputText(CDialogInputText::ePlayerSelect);
      if (IDOK == dlgInputText.DoModal(m_hWnd)) {
         if (dlgInputText.m_strPassword != m_strPassword) {
            ::MessageBox(m_hWnd,
               CLang::GetString(IDS__DIALOG_ADMINISTRATION__INCORRECT_PASSWORD),
               CLang::GetString(IDS__ERROR),
               MB_OK | MB_ICONERROR);
            return false;
         }
         return true;
      } else
         return false;
   }
   return true;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////
#else // _WTL_VER
////////////////////////////////////////////////////////////////////////////////////////////////////////

// modal dialogs
int CDialogAdministration::DoModal(HWND hWndParent, LPARAM dwInitParam) {
   ATLASSERT(m_hWnd == NULL);
   _Module.AddCreateWndData(&m_thunk.cd, (CDialogImpl<CDialogInputText>*)this);

//#ifdef _DEBUG
   m_bModal = true;
//#endif //_DEBUG
   return ::DialogBoxParam(_Module.GetResourceInstance(), CDialogAdministration::IDD(), hWndParent, (DLGPROC)CDialogInputText::StartDialogProc, dwInitParam);
}

// modeless dialogs
HWND CDialogAdministration::Create(HWND hWndParent, LPARAM dwInitParam)
{
	ATLASSERT(m_hWnd == NULL);
	_Module.AddCreateWndData(&m_thunk.cd, (CDialogImpl<CDialogInputText>*)this);
//#ifdef _DEBUG
	m_bModal = false;
//#endif //_DEBUG
	HWND hWnd = ::CreateDialogParam(_Module.GetResourceInstance(), CDialogAdministration::IDD(), hWndParent, (DLGPROC)CDialogAdministration::StartDialogProc, dwInitParam);
	ATLASSERT(m_hWnd == hWnd);
	return hWnd;
}

BOOL CDialogAdministration::OnInitDialog(HWND hwndFocus, LPARAM lParam) {
   m_bAutoloadAdmin = GetFM2WinMain().GetAutoloadAdmin();
   DoDataExchange();

   SetWindowText(                                                      CLang::GetString(IDS__DIALOG_ADMINISTRATION             ));
   m_BtnOk           .SetWindowText(                                   CLang::GetString(IDS__DIALOG_ADMINISTRATION__SELECT     ));
   m_BtnCancel       .SetWindowText(                                   CLang::GetString(IDS__CANCEL                            ));
   m_BtnNewPlayer    .SetWindowText(                                   CLang::GetString(IDS__DIALOG_ADMINISTRATION__NEWPLAYER  ));
   m_BtnNewPassword  .SetWindowText(                                   CLang::GetString(IDS__DIALOG_ADMINISTRATION__NEWPASSWORD));
   m_BtnAvatar       .SetWindowText(                                   CLang::GetString(IDS__DIALOG_ADMINISTRATION__AVATAR     ));
   m_BtnRename       .SetWindowText(                                   CLang::GetString(IDS__DIALOG_ADMINISTRATION__RENAME     ));
   m_BtnRemove       .SetWindowText(                                   CLang::GetString(IDS__DIALOG_ADMINISTRATION__REMOVE     ));
   ::SetWindowText(GetDlgItem(IDC_ADMINISTRATION_BUTTONCHECK_AUTOSTART), CLang::GetString(IDS__DIALOG_ADMINISTRATION__AUTOSTART));

   ReloadTable();

   m_CloseWnd = !m_bModal;
   if (m_bModal) {
      SetWindowLong(GWL_STYLE, WS_SYSMENU ^ GetWindowLong(GWL_STYLE));
      m_BtnCancel.EnableWindow(FALSE);
   }

   SETNEWWNDPROC(m_hWnd, IDC_ADMINISTRATION_BUTTONCHECK_AUTOSTART);

   if (!m_bModal) {
      CDialogMessageHook::InstallHook(m_hWnd);
   }

   return TRUE;
}

void CDialogAdministration::OnCommand(UINT codeNotify, int idControl, HWND /*hwndCtl*/) {
   switch (idControl) {
   case IDC_ADMINISTRATION_TABLE:
      switch (codeNotify) {
      case CTable::NC_SELECT_CELL: PostMessage(WM_COMMAND, MAKEWPARAM(IDOK                            , BN_CLICKED), (LPARAM)m_Table.m_hWnd); break;
      case CTable::NC_DELETE_CELL: PostMessage(WM_COMMAND, MAKEWPARAM(IDC_ADMINISTRATION_BUTTON_REMOVE, BN_CLICKED), (LPARAM)m_Table.m_hWnd); break;
      }
      return;
   case IDC_ADMINISTRATION_BUTTON_REMOVE:
      {
         CString strCurrUser = m_Table.GetCurrentText();
         if (strCurrUser.IsEmpty()) return;
         if (VerifyPassword()) {
            if (strCurrUser == GetFM2WinMain().GetPlayerName()) {
               //          . 
               m_CloseWnd = false;
               SetWindowLong(GWL_STYLE, WS_SYSMENU ^ GetWindowLong(GWL_STYLE));
               m_BtnCancel.EnableWindow(FALSE);
               DrawMenuBar();
            }
            GetFM2WinMain().GetPlayers().RemovePlayer(strCurrUser);
            ReloadTable();
         }
      }
      return;
   case IDC_ADMINISTRATION_BUTTON_AVATAR:
      {
         CString strCurrUser = m_Table.GetCurrentText();
         if (strCurrUser.IsEmpty()) return;
         if (VerifyPassword()) {
            CString strFileAvatar = GetFM2WinMain().GetPlayers().GetAvatar(strCurrUser);
            CDialogTemplateSelectImage dlgTmplSI(
               m_hWnd,
               NULL,
               strFileAvatar.IsEmpty() ? ::GetCurrentDirectory() : strFileAvatar, 
               CLang::GetString(IDS__DIALOG_CHANGESKIN__OPENFILENAME_TITLE_OPEN_IMAGE),
               NULL);
            if (IDOK == dlgTmplSI.DoModal()) {
               strFileAvatar = dlgTmplSI.m_strFile+dlgTmplSI.m_strResTypeAndName;
               if (!strFileAvatar.IsEmpty()) {
                  GetFM2WinMain().GetPlayers().SetAvatar(strCurrUser, strFileAvatar);
                  ReloadTable();
               }
            }
         }
      }
      return;
   case IDC_ADMINISTRATION_BUTTON_RENAME:
      {
         CString strCurrUser = m_Table.GetCurrentText();
         if (strCurrUser.IsEmpty()) return;
         if (VerifyPassword()) {
            CDialogInputText dlgInputText(CDialogInputText::ePlayerRename, strCurrUser);
            if (IDOK == dlgInputText.DoModal(m_hWnd)) {
               if (strCurrUser == GetFM2WinMain().GetPlayerName())
                  GetFM2WinMain().SetPlayerName(dlgInputText.m_strName);
               GetFM2WinMain().GetPlayers().RenamePlayer(strCurrUser, dlgInputText.m_strName);
               m_Table.SetText(0, m_Table.GetCurrentCell().Y, dlgInputText.m_strName);
            }
         }
      }
      return;
   case IDC_ADMINISTRATION_BUTTON_NEWPASSWORD:
      {
         CString strCurrUser = m_Table.GetCurrentText();
         if (strCurrUser.IsEmpty()) return;
         if (VerifyPassword()) {
            CDialogInputText dlgInputText(CDialogInputText::ePlayerNewPassword);
            if (IDOK == dlgInputText.DoModal(m_hWnd)) {
               GetFM2WinMain().GetPlayers().SetPassword(strCurrUser, dlgInputText.m_strPassword);
               m_Table.SetCurrentCell(0, 1+GetFM2WinMain().GetPlayers().FindName(strCurrUser), true);
            }
         }
      }
      return;
   case IDC_ADMINISTRATION_BUTTON_NEWPLAYER:
      {
         CDialogInputText dlgInputText(CDialogInputText::ePlayerNew);
         if (IDOK == dlgInputText.DoModal(m_hWnd)) {
            GetFM2WinMain().GetPlayers().AddNewRecord(dlgInputText.m_strName, dlgInputText.m_strPassword);
            ReloadTable();
            m_Table.SetCurrentCell(0, 1+GetFM2WinMain().GetPlayers().FindName(dlgInputText.m_strName), true);
         }
      }
      return;
   case IDOK:
      if (!VerifyPassword()) return;
      GetFM2WinMain().SetPlayerName(m_Table.GetCurrentText());
      m_CloseWnd = true;
   case IDCANCEL:
      SendMessage(WM_CLOSE, 0L, 0L);
      return;
   }
}

void CDialogAdministration::OnClose() {
   if (!m_CloseWnd) return;

   DoDataExchange(TRUE, IDC_ADMINISTRATION_BUTTONCHECK_AUTOSTART);
   GetFM2WinMain().SetAutoloadAdmin(m_bAutoloadAdmin);

   if (m_bModal) {
      EndDialog(0);
   } else {
      DestroyWindow();
   }
}

void CDialogAdministration::OnDestroy() {
   if (!m_bModal) {
      CDialogMessageHook::UninstallHook(m_hWnd);
   }

}

BOOL CDialogAdministration::OnEraseBkgnd(HDC hDC) {
   if (!GetFM2WinMain().GetSkin().m_bToAll) {
      SetMsgHandled(FALSE);
      return FALSE; // DefWindowProc(m_hWnd, WM_ERASEBKGND, (WPARAM)hDC, 0L);
   }
   return nsEraseBk::OnEraseBkgnd(m_hWnd, hDC, GetFM2WinMain().GetSkin().GetBkColor());
}

void CDialogAdministration::ReloadTable() {
   m_Table.SetColNumber(1);
   m_Table.SetRowNumber(max(7+1,GetFM2WinMain().GetPlayers().NumberPlayers()+1));
   m_Table.SetText(0,0, CLang::GetString(IDS__DIALOG_ADMINISTRATION__TABLE));
   {
      SHORT j=1;
      for (; j<m_Table.GetRowNumber(); j++) {
         m_Table.SetText (0, j, _T(""));
         m_Table.SetImage(0, j, NULL);
      }
      for (j=0; j<GetFM2WinMain().GetPlayers().NumberPlayers(); j++) {
         m_Table.SetText      (0, j+1, GetFM2WinMain().GetPlayers().GetPlayerName(j));
         m_Table.SetFormatText(0, j+1, DT_LEFT | DT_VCENTER | DT_SINGLELINE | DT_NOPREFIX);
         m_Table.SetImage     (0, j+1, &CImageSerialize(GetFM2WinMain().GetPlayers().GetAvatar(j)));
      }
      int index = GetFM2WinMain().GetPlayers().FindName(GetFM2WinMain().GetPlayerName());
      if (index>=0)
         m_Table.SetCurrentCell(0, index+1, true);
      else
         m_Table.SetCurrentCell(0,1, false);
   }
   RECT rcClient; m_Table.GetClientRect(&rcClient);
   m_Table.SetColWidth(0, rcClient.right-rcClient.left);
}

bool CDialogAdministration::VerifyPassword() {
   if (!m_Table.GetCurrentText()[0]) {
      PostMessage(WM_COMMAND, MAKEWPARAM(IDC_ADMINISTRATION_BUTTON_NEWPLAYER, BN_CLICKED), (LPARAM)m_hWnd);
      return false;
   }
   CString strCurrUser = m_Table.GetCurrentText();
   if (strCurrUser == CDialogAdministration::DEFAULT_PLAYER_NAME()) {
      MessageBox(
         CLang::GetString(IDS__DIALOG_ADMINISTRATION__THIS_VIRTUAL_PLAYER),
         CLang::GetString(IDS__ERROR),
         MB_OK | MB_ICONERROR);
      return false;
   }
   CString m_strPassword = GetFM2WinMain().GetPlayers().GetPassword(strCurrUser);
   if (!m_strPassword.IsEmpty()) {
      CDialogInputText dlgInputText(CDialogInputText::ePlayerSelect);
      if (IDOK == dlgInputText.DoModal(m_hWnd)) {
         if (dlgInputText.m_strPassword != m_strPassword) {
            MessageBox(
               CLang::GetString(IDS__DIALOG_ADMINISTRATION__INCORRECT_PASSWORD),
               CLang::GetString(IDS__ERROR),
               MB_OK | MB_ICONERROR);
            return false;
         }
         return true;
      } else
         return false;
   }
   return true;
}

#endif // _WTL_VER
