////////////////////////////////////////////////////////////////////////////////
//                               FastMines project
//                                       Sergey Krivulya (KSerg, aka SeregaLBN)
// file name: "Champions.cpp"
//    "Champions"
// Copyright (C) 2002-2007 Sergey Krivulya
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
////////////////////////////////////////////////////////////////////////////////

#include "StdAfx.h"
#include "Champions.h"
#include "../FastMines2.h"
#include "../EraseBk.h"
#include "./DialogMessageHook.h"
#include "./Administration.h"
#include "../OldVersion.h"
#include "../SerializeImpl.h"

////////////////////////////////////////////////////////////////////////////////
//                      global variables, types , defines, etc...
////////////////////////////////////////////////////////////////////////////////

extern CFastMines2WinMain& GetFM2WinMain();

////////////////////////////////////////////////////////////////////////////////
//                          implementation CChampions
////////////////////////////////////////////////////////////////////////////////

namespace nsChampions {
   struct CRecord : public ISerialize {
      CString m_strPlayerName;
      int     m_iTime;
      static int MAX_TIME() { return MAX_INTEGER; }
      CRecord() : m_iTime(MAX_TIME()) {};
      BOOL SerializeRead(HANDLE hFile) {
         DWORD dwNOBR = 0;
         return ::SerializeRead(hFile, m_strPlayerName) &&
            (::ReadFile(hFile, &m_iTime, sizeof(m_iTime), &dwNOBR, NULL) && (sizeof(m_iTime) == dwNOBR));
      }
      BOOL SerializeWrite(HANDLE hFile) const {
         DWORD dwNOBW = 0;
         return ::SerializeWrite(hFile, m_strPlayerName) &&
            (::WriteFile(hFile, &m_iTime, sizeof(m_iTime), &dwNOBW, NULL) && (sizeof(m_iTime) == dwNOBW));
      }
   };

   struct CBody : public IFileBody { // serialize body
      CRecord m_Data[eMosaicNil][eSkillLevelCustom][10];
      BOOL SerializeRead(HANDLE hFile) {
         BOOL bRes = TRUE;
         for (int i=0; bRes && i<eMosaicNil; i++)
            for (int j=0; bRes && j<eSkillLevelCustom; j++) 
               for (int k=0; bRes && k<10; k++) {
                  bRes &= m_Data[i][j][k].SerializeRead(hFile);
                  if (!bRes) {
                     m_Data[i][j][k] = CRecord();
                  }
               }
         return bRes;
      }
      BOOL SerializeWrite(HANDLE hFile) const {
         BOOL bRes = TRUE;
         for (int i=0; bRes && i<eMosaicNil; i++)
            for (int j=0; bRes && j<eSkillLevelCustom; j++) 
               for (int k=0; bRes && k<10; k++)
                  bRes &= m_Data[i][j][k].SerializeWrite(hFile);
         return bRes;
      }
   };
} // namespace nsChampions

//     
class CChampions : public CCommonSerializeEx<IFileHeaderImpl<0x9EE1B48C>, nsChampions::CBody,
                                             IDS__CHAMPIONS__ERROR_CREATE,
                                             IDS__CHAMPIONS__ERROR_OPEN,
                                             IDS__CHAMPIONS__ERROR_READ,
                                             IDS__CHAMPIONS__ERROR_WRITE,
                                             IDS__CHAMPIONS__ERROR_VERSION>
{
public:
   static int MAX_TIME() { return nsChampions::CRecord::MAX_TIME(); }
   int GetTime(EMosaic mosaic, ESkillLevel skill, UINT index) const {
      if ((mosaic == eMosaicNil) ||
          (skill  == eSkillLevelCustom) ||
          (index > 9)) return MAX_TIME();
      return m_Body.m_Data[mosaic][skill][index].m_iTime;
   }
   LPCTSTR GetPlayerName(EMosaic mosaic, ESkillLevel skill, UINT index) const {
      if ((mosaic == eMosaicNil) ||
          (skill  == eSkillLevelCustom) ||
          (index > 9)) return NULL;
      return m_Body.m_Data[mosaic][skill][index].m_strPlayerName;
   }

private:
   CString GetFileName() const { return _T("Mines.bst"); }
   BOOL IgnoreFileNotFound() const { return TRUE; }
   BOOL Convert(IN HANDLE hFile) {
      BOOL bRes = (::SetFilePointer(hFile, 0, NULL, FILE_BEGIN) != 0xFFFFFFFF);
      if (!bRes) return FALSE;

      *this = CChampions();

      DWORD dwNOBR = 0;
      const DWORD dwSizeFile = ::GetFileSize(hFile, NULL);
      switch (dwSizeFile) {
      default: bRes = FALSE; break;
      case sizeof(nsVer210::CFileChmpn):
         {
            nsVer210::CFileChmpn FileChmpn_v210;
            bRes = (
               ::ReadFile(hFile, &FileChmpn_v210, dwSizeFile, &dwNOBR, NULL) &&
               (dwSizeFile == dwNOBR) &&
               (0 == strcmp(FileChmpn_v210.m_szVersion, ID_VERSIONINFO_VERSION3_v210))
            );
            if (bRes) {
               bRes &= (
                  eSkillLevelBeginner     == nsVer210::eSkillLevelBeginner     &&
                  eSkillLevelAmateur      == nsVer210::eSkillLevelAmateur      &&
                  eSkillLevelProfessional == nsVer210::eSkillLevelProfessional &&
                  eSkillLevelCrazy        == nsVer210::eSkillLevelCrazy        &&

                  //CImg::_ePlaceUniformly | CImg::_ePlaceCenter | CImg::_ePlaceMiddle == nsVer210::ePlaceCenter  &&
                  //CImg::_ePlaceStretch   | CImg::_ePlaceCenter | CImg::_ePlaceMiddle == nsVer210::ePlaceStretch &&
                  //CImg::_ePlaceTile      | CImg::_ePlaceCenter | CImg::_ePlaceMiddle == nsVer210::ePlaceTile    &&

                  CCell::_Nil  == nsVer210::_Nil  &&
                  CCell::_1    == nsVer210::_1  &&
                  CCell::_2    == nsVer210::_2  &&
                  CCell::_3    == nsVer210::_3  &&
                  CCell::_4    == nsVer210::_4  &&
                  CCell::_5    == nsVer210::_5  &&
                  CCell::_6    == nsVer210::_6  &&
                  CCell::_7    == nsVer210::_7  &&
                  CCell::_8    == nsVer210::_8  &&
                  CCell::_9    == nsVer210::_9  &&
                  CCell::_10   == nsVer210::_10 &&
                  CCell::_11   == nsVer210::_11 &&
                  CCell::_12   == nsVer210::_12 &&
                  CCell::_13   == nsVer210::_13 &&
                  CCell::_14   == nsVer210::_14 &&
                  CCell::_15   == nsVer210::_15 &&
                  CCell::_16   == nsVer210::_16 &&
                  CCell::_17   == nsVer210::_17 &&
                  CCell::_18   == nsVer210::_18 &&
                  CCell::_19   == nsVer210::_19 &&
                  CCell::_20   == nsVer210::_20 &&
                  CCell::_21   == nsVer210::_21 &&
                  CCell::_Mine == nsVer210::_Mine &&

                  CCell::_Unknown == nsVer210::_Unknown &&
                  CCell::_Clear   == nsVer210::_Clear &&
                  CCell::_Flag    == nsVer210::_Flag
               );
            #ifdef _DEBUG
               if (!bRes) {
                  ::MessageBox(NULL, _T("  -"), _T("Error"), MB_OK | MB_ICONERROR);
               }
            #endif // _DEBUG
            }
            if (bRes) {
               // all Ok - convert from v210
               int j, k;
            #define CONVERT_CHAMP(name) \
               for (j=0; j<eSkillLevelCustom; j++) \
                  for (k=0; k<10; k++) { \
                     m_Body.m_Data[eMosaic##name][j][k].m_strPlayerName = FileChmpn_v210.m_Data[nsVer210::eMosaic##name][j][k].m_szPlayerName; \
                     m_Body.m_Data[eMosaic##name][j][k].m_iTime         = FileChmpn_v210.m_Data[nsVer210::eMosaic##name][j][k].m_iTime; \
                  }

               CONVERT_CHAMP(Triangle1)
               CONVERT_CHAMP(Triangle2)
               CONVERT_CHAMP(Triangle3)
               CONVERT_CHAMP(Triangle4)
               CONVERT_CHAMP(Square1)
               CONVERT_CHAMP(Square2)
               CONVERT_CHAMP(Parquet1)
               CONVERT_CHAMP(Parquet2)
               CONVERT_CHAMP(Trapezoid1)
               CONVERT_CHAMP(Trapezoid2)
               CONVERT_CHAMP(Trapezoid3)
               CONVERT_CHAMP(Rhombus1)
               CONVERT_CHAMP(Quadrangle1)
               CONVERT_CHAMP(PentagonT24)
               CONVERT_CHAMP(PentagonT5)
               CONVERT_CHAMP(PentagonT10)
               CONVERT_CHAMP(Hexagon1)
               CONVERT_CHAMP(TrSq1)
               CONVERT_CHAMP(TrSq2)
               CONVERT_CHAMP(SqTrHex)
            #undef CONVERT_CHAMP
            }
         }
         break;
      case sizeof(nsVer200::CFileChmpn):
         {
            nsVer200::CFileChmpn FileChmpn_v200;
            bRes = ReadFile(hFile, &FileChmpn_v200, dwSizeFile, &dwNOBR, NULL) && (dwSizeFile == dwNOBR);
            if (bRes) {
               // all Ok - convert from v200
               int j, k;
            #define CONVERT_CHAMP(name) \
               for (j=0; j<eSkillLevelCustom; j++) \
                  for (k=0; k<10; k++) { \
                     m_Body.m_Data[eMosaic##name][j][k].m_strPlayerName = FileChmpn_v200.m_Data[nsVer200::eMosaic##name][j][k].m_szPlayerName; \
                     m_Body.m_Data[eMosaic##name][j][k].m_iTime         = FileChmpn_v200.m_Data[nsVer200::eMosaic##name][j][k].m_iTime; \
                  }

               CONVERT_CHAMP(Triangle1)
               CONVERT_CHAMP(Triangle2)
               CONVERT_CHAMP(Square1)
               CONVERT_CHAMP(Square2)
               CONVERT_CHAMP(Parquet1)
               CONVERT_CHAMP(Parquet2)
               CONVERT_CHAMP(Trapezoid1)
               CONVERT_CHAMP(Rhombus1)
               CONVERT_CHAMP(PentagonT24)
               CONVERT_CHAMP(PentagonT10)
               CONVERT_CHAMP(Hexagon1)
               CONVERT_CHAMP(TrSq1)
               CONVERT_CHAMP(SqTrHex)
            #undef CONVERT_CHAMP
            }
         }
         break;
      } // end switch
      return bRes;
   }

public:
   int InsertRecord(EMosaic mosaic, ESkillLevel skill, int iTime, LPCTSTR szPlayerName) {
      int j = 9;
      do {
         if (m_Body.m_Data[mosaic][skill][j-1].m_iTime > iTime) {
             m_Body.m_Data[mosaic][skill][j] = m_Body.m_Data[mosaic][skill][j-1];
         } else {
            m_Body.m_Data[mosaic][skill][j].m_iTime = iTime;
            m_Body.m_Data[mosaic][skill][j].m_strPlayerName = szPlayerName;
            break;
         }
         j--;
      } while(j != 0);
      if (j == 0) {
         m_Body.m_Data[mosaic][skill][j].m_iTime = iTime;
         m_Body.m_Data[mosaic][skill][j].m_strPlayerName = szPlayerName;
      }
      return j;
   }
};

////////////////////////////////////////////////////////////////////////////////
//                          implementation CDialogChampions
////////////////////////////////////////////////////////////////////////////////

/* static */ bool CDialogChampions::SaveResult(EMosaic mosaic, ESkillLevel skill, int iTime, LPCTSTR szPlayerName) {
   if (skill == eSkillLevelCustom) return false;
   CChampions Champions;
   Champions.LoadFile();
   if (Champions.GetTime(mosaic, skill, 9) <= iTime) return false;
   m_iLastInsertPlayerIndex() = Champions.InsertRecord(mosaic, skill, iTime, szPlayerName);
   Champions.SaveFile();
   if (lstrcmpi(szPlayerName, CDialogAdministration::DEFAULT_PLAYER_NAME())) {
      FORWARD_WM_COMMAND(GetFM2WinMain().m_hWnd, IDM_HELP_CHAMPIONS, NULL, 1, ::PostMessage);
   }
   return true;
}

#ifndef _WTL_VER

void CDialogChampions::OnSize(HWND, UINT state, int cx, int cy) {
   int hB = 30;    //  
   if (cx < 400) {
      int wB = cx>>1; //  
      ::MoveWindow(::GetDlgItem(m_hWnd, IDC_STATISTICSorCHAMPIONS_TABCONTROL), 0,0,cx,cy-(hB<<1), TRUE);
      ::MoveWindow(m_BtnBeginner.m_hWnd, 0*wB,cy-(hB<<1),wB,hB, TRUE);
      ::MoveWindow(m_BtnAmateur .m_hWnd, 0*wB,cy-(hB<<0),wB,hB, TRUE);
      ::MoveWindow(m_BtnProfi   .m_hWnd, 1*wB,cy-(hB<<1),wB,hB, TRUE);
      ::MoveWindow(m_BtnCrazy   .m_hWnd, 1*wB,cy-(hB<<0),wB,hB, TRUE);
   } else {
      int wB = cx>>2; //  
      ::MoveWindow(::GetDlgItem(m_hWnd, IDC_STATISTICSorCHAMPIONS_TABCONTROL), 0,0,cx,cy-hB, TRUE);
      ::MoveWindow(m_BtnBeginner.m_hWnd, 0*wB,cy-hB,wB,hB, TRUE);
      ::MoveWindow(m_BtnAmateur .m_hWnd, 1*wB,cy-hB,wB,hB, TRUE);
      ::MoveWindow(m_BtnProfi   .m_hWnd, 2*wB,cy-hB,wB,hB, TRUE);
      ::MoveWindow(m_BtnCrazy   .m_hWnd, 3*wB,cy-hB,wB,hB, TRUE);
   }

   RECT rcTab = ::GetClientRect(m_hTabCtrl);
   RECTEX rcItem; TabCtrl_GetItemRect(m_hTabCtrl, 0, &rcItem);
   RECT rcTbl = {2, rcItem.height()+4, rcTab.right-2, rcTab.bottom-3};
   ::MoveWindow(m_Table.m_hWnd, rcTbl);

   float arrW[] = {0.75f,0.25f}; //   
   m_Table.SetColWidth(0, toINT(cx*arrW[0])-4);
   m_Table.SetColWidth(1, toINT(cx*arrW[1])-4);
}

////////////////////////////////////////////////////////////////////////////////////////////////////////
#else // _WTL_VER
////////////////////////////////////////////////////////////////////////////////////////////////////////

void CDialogChampions::OnSize(UINT state, CSize &size) {
   int hB = 30;    //  
   if (size.cx < 400) {
      int wB = size.cx>>1; //  
      m_TabCtrl    .MoveWindow(0,0,size.cx,size.cy-(hB<<1), TRUE);
      m_BtnBeginner.MoveWindow(0*wB,size.cy-(hB<<1),wB,hB, TRUE);
      m_BtnAmateur .MoveWindow(0*wB,size.cy-(hB<<0),wB,hB, TRUE);
      m_BtnProfi   .MoveWindow(1*wB,size.cy-(hB<<1),wB,hB, TRUE);
      m_BtnCrazy   .MoveWindow(1*wB,size.cy-(hB<<0),wB,hB, TRUE);
   } else {
      int wB = size.cx>>2; //  
      m_TabCtrl    .MoveWindow(0,0,size.cx,size.cy-hB, TRUE);
      m_BtnBeginner.MoveWindow(0*wB,size.cy-hB,wB,hB, TRUE);
      m_BtnAmateur .MoveWindow(1*wB,size.cy-hB,wB,hB, TRUE);
      m_BtnProfi   .MoveWindow(2*wB,size.cy-hB,wB,hB, TRUE);
      m_BtnCrazy   .MoveWindow(3*wB,size.cy-hB,wB,hB, TRUE);
   }

   RECT rcTab = ::GetClientRect(m_TabCtrl.m_hWnd);
   RECTEX rcItem; m_TabCtrl.GetItemRect(0, &rcItem);
   RECT rcTbl = {2, rcItem.height()+4, rcTab.right-2, rcTab.bottom-3};
   m_Table.MoveWindow(&rcTbl, TRUE);

   float arrW[] = {0.75f,0.25f}; //   
   m_Table.SetColWidth(0, toINT(size.cx*arrW[0])-4);
   m_Table.SetColWidth(1, toINT(size.cx*arrW[1])-4);
}

#endif // _WTL_VER

void CDialogChampions::OnInitTable() {
   m_Table.SetColNumber(2);
   m_Table.SetRowNumber(11);
   m_Table.SetText(0,0, CLang::GetString(IDS__CHAMPIONS__PLAYER_NAME));
   m_Table.SetText(1,0, CLang::GetString(IDS__CHAMPIONS__GAME_TIME  ));
   m_Table.SetColWidth(0, 200);
   m_Table.SetColWidth(1, 100);
   for (int i=1; i<11; i++)
      m_Table.SetFormatText(0, i, DT_LEFT | DT_VCENTER | DT_SINGLELINE | DT_NOPREFIX | DT_NOPREFIX);
}

void CDialogChampions::OnChangeTabPage() {
   int indexPlayer = m_iLastInsertPlayerIndex();
   if (indexPlayer < 0)
      m_Table.UnselectCell();
   else {
      m_Table.SetCurrentCell(0, 1+indexPlayer, true);
      m_iLastInsertPlayerIndex() = -1;
   }
   int i=0;
   for (; i<10; i++) {
      m_Table.SetText (0, i+1, _T(""));
      m_Table.SetText (1, i+1, _T(""));
      m_Table.SetImage(0, i+1, NULL);
   }
   CChampions Champions;
   Champions.LoadFile();
   for (i=0; i<10; i++) {
      int iTime = Champions.GetTime(m_Mosaic, m_SkillLevel, i);
      if (iTime == CChampions::MAX_TIME()) continue;
      CString strPlayerName = Champions.GetPlayerName(m_Mosaic, m_SkillLevel, i);
      if (strPlayerName.IsEmpty()) continue;
      m_Table.SetText(0, i+1, strPlayerName);
      int iPos = GetFM2WinMain().GetPlayers().FindName(strPlayerName);
      if (iPos >= 0) {
         m_Table.SetImage(0, i+1, &CImageSerialize(GetFM2WinMain().GetPlayers().GetAvatar(iPos)));
      }
      CString strTimeGame;
      strTimeGame.Format(_T("%d %s"), iTime, (LPCTSTR)CLang::GetString(IDS__SEC));
      m_Table.SetText(1, i+1, strTimeGame);
   }
}
