////////////////////////////////////////////////////////////////////////////////
//                               FastMines project
//                                       Sergey Krivulya (KSerg, aka SeregaLBN)
// file name: "CustomSkill.cpp"
//    "CustomSkillLevel"
// Copyright (C) 2002-2007 Sergey Krivulya
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
////////////////////////////////////////////////////////////////////////////////

#include "StdAfx.h"
#include "CustomSkill.h"
#include "../FastMines2.h"
#include "../EraseBk.h"
#include "../Lang.h"
#include "./DialogMessageHook.h"

//#pragma comment(lib, "MsImg32") // GradientFill

////////////////////////////////////////////////////////////////////////////////
//                      global variables, types , defines, etc...
////////////////////////////////////////////////////////////////////////////////
extern CFastMines2WinMain& GetFM2WinMain();

inline BOOL CreateMenuItem(HMENU hMenu, LPCTSTR str, UINT uIns, UINT uCom, HMENU hSubMenu, BOOL flag, UINT fType) {
   MENUITEMINFO mii;
   mii.cbSize = sizeof(MENUITEMINFO);
   mii.fMask = MIIM_STATE | MIIM_TYPE | MIIM_SUBMENU | MIIM_ID;
   mii.fType = fType;
   mii.fState = MFS_ENABLED;
   mii.dwTypeData = (LPTSTR)str;
   mii.cch = lstrlen(str);
   mii.wID = uCom;
   mii.hSubMenu = hSubMenu;
   return ::InsertMenuItem(hMenu, uIns, flag, &mii);
}

SIZE GetMaxSize(bool bCurrentArea) {
   int iArea = bCurrentArea ? GetFM2WinMain().GetArea() : CMosaic::AreaMinimum();
   SIZE size = GetFM2WinMain().GetSizeMosaic();

   const SIZE sizeScreen = ::GetScreenSize();

   do
      size.cx++;
   while(GetFM2WinMain().GetSize(size, iArea).cx <= sizeScreen.cx);
   size.cx--;

   do
      size.cy++;
   while (GetFM2WinMain().GetSize(size, iArea).cy <= sizeScreen.cy);
   size.cy--;

   return size;
}

HBITMAP CreateBitmapDownArrow() {
   //return ::LoadBitmap(NULL, MAKEINTRESOURCE(OBM_DNARROW));
   HBITMAP hBMP = ::CreateBitmap(13,15);
   HDC hCDC = ::CreateCompatibleDC(NULL);
   HGDIOBJ hBMPOld = ::SelectObject(hCDC, hBMP);
   HBRUSH hBrush = 
      GetFM2WinMain().GetSkin().m_bToAll ?
         ::CreateSolidBrush(GetFM2WinMain().GetSkin().GetBkColor()) :
         ::GetSysColorBrush(COLOR_BTNFACE);
   HGDIOBJ hBrushOld = ::SelectObject(hCDC, hBrush);
   ::PatBlt(hCDC, 0,0,13,15, PATCOPY);
   TRIVERTEX vert[] = {
      {
         0,  // LONG    x;
         4,  // LONG    y;
         0x00ff,  // COLOR16 Red;   0x0000..0xff00
         0x0f0f,  // COLOR16 Green;
         0x0ff0,  // COLOR16 Blue;
         0x0000   // COLOR16 Alpha;
      }, {
         13,  // LONG    x;
         4,   // LONG    y;   
         0x0f0f,  // COLOR16 Red;   0x0000..0xff00
         0x0ff0,  // COLOR16 Green;
         0x00ff,  // COLOR16 Blue;
         0x0000   // COLOR16 Alpha;
      }, {
         6,  // LONG    x;
         10, // LONG    y;   
         0x0ff0,  // COLOR16 Red;   0x0000..0xff00
         0x00ff,  // COLOR16 Green;
         0x0f0f,  // COLOR16 Blue;
         0x0000   // COLOR16 Alpha;
      }
   };
   GRADIENT_TRIANGLE gTri = {0, 1, 2};
   ::GradientFill(hCDC, vert, chDIMOF(vert), &gTri, 1, GRADIENT_FILL_TRIANGLE);
   ::SelectObject(hCDC, hBMPOld);
   ::SelectObject(hCDC, hBrushOld);
   if (GetFM2WinMain().GetSkin().m_bToAll)
      ::DeleteObject(hBrush);
   ::DeleteDC(hCDC);
   return hBMP;
}

   WNDPROC_STATIC(IDC_CUSTOMSKILL_GROUPBOX   , GetFM2WinMain().GetSkin())
   WNDPROC_STATIC(IDC_CUSTOMSKILL_STATIC_X   , GetFM2WinMain().GetSkin())
   WNDPROC_STATIC(IDC_CUSTOMSKILL_STATIC_Y   , GetFM2WinMain().GetSkin())
   WNDPROC_STATIC(IDC_CUSTOMSKILL_STATIC_MINE, GetFM2WinMain().GetSkin())
   WNDPROC_STATIC(IDC_CUSTOMSKILL_SPIN_X     , GetFM2WinMain().GetSkin())
   WNDPROC_STATIC(IDC_CUSTOMSKILL_SPIN_Y     , GetFM2WinMain().GetSkin())
   WNDPROC_STATIC(IDC_CUSTOMSKILL_SPIN_MINE  , GetFM2WinMain().GetSkin())
   WNDPROC_BUTTON(IDC_CUSTOMSKILL_FULLSCREEN1, GetFM2WinMain().GetSkin())
   WNDPROC_BUTTON(IDC_CUSTOMSKILL_FULLSCREEN2, GetFM2WinMain().GetSkin())

////////////////////////////////////////////////////////////////////////////////
//                        implementation CDialogCustomSkill
////////////////////////////////////////////////////////////////////////////////

#ifndef _WTL_VER

HWND CDialogCustomSkill::Create(HWND hWndParent) {
   return ::CreateDialogParam(::GetModuleHandle(NULL), CDialogCustomSkill::IDD(), hWndParent, (DLGPROC)CDialogCustomSkill::DialogProcThis, (LPARAM)this);
}

/*static*/ INT_PTR CALLBACK CDialogCustomSkill::DialogProcThis(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
   //LOG_PUT(g_pLog, Put, LL_DEBUG) _T("  CDialogCustomSkill::DialogProcThis: %s "), (LPCTSTR)CLogger::WindowMessage(uMsg));
   CDialogCustomSkill *This = NULL;
   if (uMsg == WM_INITDIALOG) {
      This = (CDialogCustomSkill*)lParam;
      if (This) {
         This->m_hWnd = hWnd;
         ::SetWindowUserData(hWnd, (LONG_PTR)This);
      }
   } else {
      This = (CDialogCustomSkill*)::GetWindowUserData(hWnd);
   }
   if (This) {
      return This->DialogProc(hWnd, uMsg, wParam, lParam);
   }
   return FALSE;
}

BOOL CDialogCustomSkill::OnInitDialog(HWND, HWND hWndFocus, LPARAM lParam) {
   m_BtnOk    .SubclassWindow(::GetDlgItem(m_hWnd, IDOK));
   m_BtnCancel.SubclassWindow(::GetDlgItem(m_hWnd, IDCANCEL));
   m_BtnPopup .SubclassWindow(::GetDlgItem(m_hWnd, IDC_CUSTOMSKILL_BUTTON_POPUP_MENU));
   m_EditX    .SubclassWindow(::GetDlgItem(m_hWnd, IDC_CUSTOMSKILL_EDIT_X   ));
   m_EditY    .SubclassWindow(::GetDlgItem(m_hWnd, IDC_CUSTOMSKILL_EDIT_Y   ));
   m_EditMine .SubclassWindow(::GetDlgItem(m_hWnd, IDC_CUSTOMSKILL_EDIT_MINE));

   ::SetWindowText(m_hWnd                                           , CLang::GetString(IDS__DIALOG_CUSTOM_SKILL                           ));
   ::SetWindowText(m_BtnOk.m_hWnd                                   , CLang::GetString(IDS__OK                                            ));
   ::SetWindowText(m_BtnCancel.m_hWnd                               , CLang::GetString(IDS__CANCEL                                        ));
   ::SetWindowText(::GetDlgItem(m_hWnd, IDC_CUSTOMSKILL_STATIC_X   ), CLang::GetString(IDS__DIALOG_CUSTOM_SKILL__X_WIDTH                  ));
   ::SetWindowText(::GetDlgItem(m_hWnd, IDC_CUSTOMSKILL_STATIC_Y   ), CLang::GetString(IDS__DIALOG_CUSTOM_SKILL__Y_HEIGHT                 ));
   ::SetWindowText(::GetDlgItem(m_hWnd, IDC_CUSTOMSKILL_STATIC_MINE), CLang::GetString(IDS__DIALOG_CUSTOM_SKILL__NUMBER_MINES             ));
   ::SetWindowText(::GetDlgItem(m_hWnd, IDC_CUSTOMSKILL_FULLSCREEN1), CLang::GetString(IDS__DIALOG_CUSTOM_SKILL__FULL_SCREEN__CURRENT_SIZE));
   ::SetWindowText(::GetDlgItem(m_hWnd, IDC_CUSTOMSKILL_FULLSCREEN2), CLang::GetString(IDS__DIALOG_CUSTOM_SKILL__FULL_SCREEN__MINIMAL_SIZE));

   m_iNewMines     = GetFM2WinMain().GetMines();
   m_sizeNewMosaic = GetFM2WinMain().GetSizeMosaic();




   ::SendDlgItemMessage(m_hWnd, IDC_CUSTOMSKILL_SPIN_X   , UDM_SETRANGE, 0L, MAKELPARAM(UD_MAXVAL, 5));
   ::SendDlgItemMessage(m_hWnd, IDC_CUSTOMSKILL_SPIN_Y   , UDM_SETRANGE, 0L, MAKELPARAM(UD_MAXVAL, 5));
   ::SendDlgItemMessage(m_hWnd, IDC_CUSTOMSKILL_SPIN_MINE, UDM_SETRANGE, 0L, MAKELPARAM(UD_MAXVAL, 1));
   ::SendDlgItemMessage(m_hWnd, IDC_CUSTOMSKILL_SPIN_X   , UDM_SETPOS  , 0L, m_sizeNewMosaic.cx);
   ::SendDlgItemMessage(m_hWnd, IDC_CUSTOMSKILL_SPIN_Y   , UDM_SETPOS  , 0L, m_sizeNewMosaic.cy);
   ::SendDlgItemMessage(m_hWnd, IDC_CUSTOMSKILL_SPIN_MINE, UDM_SETPOS  , 0L, m_iNewMines);

   m_hMenu = ::CreatePopupMenu();
   ::CreateMenuItem(m_hMenu, (LPCTSTR)CLang::GetString(IDS__MENU_GAME__BEGINNER    ), 0, IDM_GAME_BEGINNER    , NULL, FALSE, MFT_STRING);
   ::CreateMenuItem(m_hMenu, (LPCTSTR)CLang::GetString(IDS__MENU_GAME__AMATEUR     ), 0, IDM_GAME_AMATEUR     , NULL, FALSE, MFT_STRING);
   ::CreateMenuItem(m_hMenu, (LPCTSTR)CLang::GetString(IDS__MENU_GAME__PROFESSIONAL), 0, IDM_GAME_PROFESSIONAL, NULL, FALSE, MFT_STRING);
   ::CreateMenuItem(m_hMenu, (LPCTSTR)CLang::GetString(IDS__MENU_GAME__CRAZY       ), 0, IDM_GAME_CRAZY       , NULL, FALSE, MFT_STRING);
   GetFM2WinMain().SetOwnerDrawMenu(m_hMenu);

   ::SendMessage(m_BtnPopup.m_hWnd, BM_SETIMAGE, IMAGE_BITMAP, (LPARAM)::CreateBitmapDownArrow());

   SETNEWWNDPROC(m_hWnd, IDC_CUSTOMSKILL_GROUPBOX   );
   SETNEWWNDPROC(m_hWnd, IDC_CUSTOMSKILL_STATIC_X   );
   SETNEWWNDPROC(m_hWnd, IDC_CUSTOMSKILL_STATIC_Y   );
   SETNEWWNDPROC(m_hWnd, IDC_CUSTOMSKILL_STATIC_MINE);
   SETNEWWNDPROC(m_hWnd, IDC_CUSTOMSKILL_SPIN_X     );
   SETNEWWNDPROC(m_hWnd, IDC_CUSTOMSKILL_SPIN_Y     );
   SETNEWWNDPROC(m_hWnd, IDC_CUSTOMSKILL_SPIN_MINE  );
   SETNEWWNDPROC(m_hWnd, IDC_CUSTOMSKILL_FULLSCREEN1);
   SETNEWWNDPROC(m_hWnd, IDC_CUSTOMSKILL_FULLSCREEN2);

   CDialogMessageHook::InstallHook(m_hWnd);

   return TRUE;
}

void CDialogCustomSkill::OnMeasureItem(HWND, MEASUREITEMSTRUCT *lpMeasureItem) {
   if (lpMeasureItem) {
      if (lpMeasureItem->CtlType == ODT_MENU) {
         CMenuItem::OnMeasureItem(lpMeasureItem);
      }
   }
}

void CDialogCustomSkill::OnDrawItem(HWND, const DRAWITEMSTRUCT *lpDrawItem) {
   if (lpDrawItem) {
      if (lpDrawItem->CtlType == ODT_MENU) {
         CMenuItem::OnDrawItem(lpDrawItem);
      }
   }
}
#define CDialogCustomSkill__OnSpinChange(iVal, idControl, iMax) { \
   LRESULT lRes = ::SendDlgItemMessage(m_hWnd, (idControl), UDM_GETPOS, 0L, 0L); \
   (iVal) = min(LOWORD(lRes), (iMax)); \
   ::SendDlgItemMessage(m_hWnd, (idControl), UDM_SETPOS, 0L, (iVal)); \
}

void CDialogCustomSkill::OnCommand(HWND, int idControl, HWND hWndCtl, UINT codeNotify) {
   switch (idControl) {
   case IDC_CUSTOMSKILL_EDIT_X:
      if (codeNotify == EN_CHANGE) {
         CDialogCustomSkill__OnSpinChange(m_sizeNewMosaic.cx, IDC_CUSTOMSKILL_SPIN_X, GetMaxSize(false).cx);
         ::PostMessage(m_hWnd, WM_COMMAND, MAKEWPARAM(IDC_CUSTOMSKILL_EDIT_MINE, EN_CHANGE), 0); //      ,  -   

         ::SendDlgItemMessage(m_hWnd, IDC_CUSTOMSKILL_FULLSCREEN1, BM_SETCHECK, BST_UNCHECKED, 0L);
         ::SendDlgItemMessage(m_hWnd, IDC_CUSTOMSKILL_FULLSCREEN2, BM_SETCHECK, BST_UNCHECKED, 0L);
      }
      return;
   case IDC_CUSTOMSKILL_EDIT_Y:
      if (codeNotify == EN_CHANGE) {
         CDialogCustomSkill__OnSpinChange(m_sizeNewMosaic.cy, IDC_CUSTOMSKILL_SPIN_Y, GetMaxSize(false).cy);
         ::PostMessage(m_hWnd, WM_COMMAND, MAKEWPARAM(IDC_CUSTOMSKILL_EDIT_MINE, EN_CHANGE), 0); //      ,  -   

         ::SendDlgItemMessage(m_hWnd, IDC_CUSTOMSKILL_FULLSCREEN1, BM_SETCHECK, BST_UNCHECKED, 0L);
         ::SendDlgItemMessage(m_hWnd, IDC_CUSTOMSKILL_FULLSCREEN2, BM_SETCHECK, BST_UNCHECKED, 0L);
      }
      return;
   case IDC_CUSTOMSKILL_EDIT_MINE:
      if (codeNotify == EN_CHANGE) {
         CDialogCustomSkill__OnSpinChange(m_iNewMines, IDC_CUSTOMSKILL_SPIN_MINE, GetFM2WinMain().GetMaxMines(m_sizeNewMosaic));
      }
      return;
   case IDC_CUSTOMSKILL_FULLSCREEN1:
   case IDC_CUSTOMSKILL_FULLSCREEN2:
      if ((codeNotify == BN_CLICKED) &&
          (BST_CHECKED == ::SendMessage(hWndCtl, BM_GETCHECK, 0L, 0L)))
      {
         SIZE s = GetMaxSize(idControl == IDC_CUSTOMSKILL_FULLSCREEN1);
         ::SendDlgItemMessage(m_hWnd, IDC_CUSTOMSKILL_SPIN_X, UDM_SETPOS, 0L, s.cx);
         ::SendDlgItemMessage(m_hWnd, IDC_CUSTOMSKILL_SPIN_Y, UDM_SETPOS, 0L, s.cy);
         ::SendDlgItemMessage(m_hWnd, (idControl == IDC_CUSTOMSKILL_FULLSCREEN1) ?
                                                   IDC_CUSTOMSKILL_FULLSCREEN2 :
                                                   IDC_CUSTOMSKILL_FULLSCREEN1, BM_SETCHECK, BST_UNCHECKED, 0L);
         ::SendMessage(hWndCtl, BM_SETCHECK, BST_CHECKED, 0);
      }
      return;
   case IDC_CUSTOMSKILL_BUTTON_POPUP_MENU:
      if (codeNotify == BN_CLICKED) {
         RECTEX rc = ::GetWindowRect(hWndCtl);
         OnContextMenu(m_hWnd, hWndCtl, rc.pointLB().x, rc.pointLB().y);
      }
      return;
   case IDOK:
      if (codeNotify == BN_CLICKED) {
         FORWARD_WM_COMMAND(m_hWnd, IDC_CUSTOMSKILL_EDIT_X   , NULL, EN_CHANGE, ::SendMessage);
         FORWARD_WM_COMMAND(m_hWnd, IDC_CUSTOMSKILL_EDIT_Y   , NULL, EN_CHANGE, ::SendMessage);
         FORWARD_WM_COMMAND(m_hWnd, IDC_CUSTOMSKILL_EDIT_MINE, NULL, EN_CHANGE, ::SendMessage);

         GetFM2WinMain().SetSkillLevelCustom(m_sizeNewMosaic, m_iNewMines);
      }
   case IDCANCEL:
      if (codeNotify == BN_CLICKED) {
         ::SendMessage(m_hWnd, WM_CLOSE, 0L, 0L);
      }
      return;
   }
}

void CDialogCustomSkill::OnClose(HWND){
   ::DestroyMenu(m_hMenu); m_hMenu = NULL;
   ::DestroyWindow(m_hWnd);
}

void CDialogCustomSkill::OnDestroy(HWND) {
   ::DeleteObject((HBITMAP)::SendMessage(m_BtnPopup.m_hWnd, BM_GETIMAGE, IMAGE_BITMAP, 0));
   CDialogMessageHook::UninstallHook(m_hWnd);
   m_hWnd = NULL;
}

BOOL CDialogCustomSkill::OnEraseBkgnd(HWND, HDC hdc) {
   if (!GetFM2WinMain().GetSkin().m_bToAll)
   {
      return FALSE; // DefWindowProc(m_hWnd, WM_ERASEBKGND, (WPARAM)hdc, 0L);
   }
   return nsEraseBk::OnEraseBkgnd(m_hWnd, hdc, GetFM2WinMain().GetSkin().GetBkColor());
}

void CDialogCustomSkill::OnContextMenu(HWND, HWND hWndContext, UINT xPos, UINT yPos) {
 //DWORD xyPos = GetMessagePos();
   int id = TrackPopupMenu(
      m_hMenu,
      TPM_LEFTALIGN | TPM_TOPALIGN | TPM_RETURNCMD | TPM_LEFTBUTTON | TPM_RIGHTBUTTON,
      xPos, yPos, 0,
    //LOWORD(xyPos), HIWORD(xyPos), 0,
      m_hWnd,
      NULL);
   ESkillLevel skill;
   switch (id) {
   case IDM_GAME_BEGINNER    :
   case IDM_GAME_AMATEUR     :
   case IDM_GAME_PROFESSIONAL:
   case IDM_GAME_CRAZY       : skill = (ESkillLevel)(id-IDM_GAME_BEGINNER); break;
   default: return;
   }
   m_iNewMines = GetFM2WinMain().DefineNumberMines(skill, m_sizeNewMosaic);
   ::SendDlgItemMessage(m_hWnd, IDC_CUSTOMSKILL_SPIN_MINE, UDM_SETPOS, 0L, m_iNewMines);
}

////////////////////////////////////////////////////////////////////////////////////////////////////////
#else // _WTL_VER
////////////////////////////////////////////////////////////////////////////////////////////////////////

// modeless dialogs
HWND CDialogCustomSkill::Create(HWND hWndParent, LPARAM dwInitParam)
{
	ATLASSERT(m_hWnd == NULL);
	_Module.AddCreateWndData(&m_thunk.cd, this);
#ifdef _DEBUG
	m_bModal = false;
#endif //_DEBUG
	HWND hWnd = ::CreateDialogParam(_Module.GetResourceInstance(), CDialogCustomSkill::IDD(), hWndParent, (DLGPROC)CDialogCustomSkill::StartDialogProc, dwInitParam);
	ATLASSERT(hWnd == m_hWnd);
	return m_hWnd;
}

BOOL CDialogCustomSkill::OnInitDialog(HWND hwndFocus, LPARAM lParam) {

   DoDataExchange();


   SetWindowText(                                           CLang::GetString(IDS__DIALOG_CUSTOM_SKILL                           ));
   m_BtnOk    .SetWindowText(                               CLang::GetString(IDS__OK                                            ));
   m_BtnCancel.SetWindowText(                               CLang::GetString(IDS__CANCEL                                        ));
   ::SetWindowText(GetDlgItem(IDC_CUSTOMSKILL_STATIC_X   ), CLang::GetString(IDS__DIALOG_CUSTOM_SKILL__X_WIDTH                  ));
   ::SetWindowText(GetDlgItem(IDC_CUSTOMSKILL_STATIC_Y   ), CLang::GetString(IDS__DIALOG_CUSTOM_SKILL__Y_HEIGHT                 ));
   ::SetWindowText(GetDlgItem(IDC_CUSTOMSKILL_STATIC_MINE), CLang::GetString(IDS__DIALOG_CUSTOM_SKILL__NUMBER_MINES             ));
   ::SetWindowText(GetDlgItem(IDC_CUSTOMSKILL_FULLSCREEN1), CLang::GetString(IDS__DIALOG_CUSTOM_SKILL__FULL_SCREEN__CURRENT_SIZE));
   ::SetWindowText(GetDlgItem(IDC_CUSTOMSKILL_FULLSCREEN2), CLang::GetString(IDS__DIALOG_CUSTOM_SKILL__FULL_SCREEN__MINIMAL_SIZE));

   m_iNewMines     = GetFM2WinMain().GetMines();
   m_sizeNewMosaic = GetFM2WinMain().GetSizeMosaic();

   m_SpinX   .SetRange(5, UD_MAXVAL);
   m_SpinY   .SetRange(5, UD_MAXVAL);
   m_SpinMine.SetRange(1, UD_MAXVAL);
   m_SpinX   .SetPos(m_sizeNewMosaic.cx);
   m_SpinY   .SetPos(m_sizeNewMosaic.cy);
   m_SpinMine.SetPos(m_iNewMines);

   m_hMenu.CreatePopupMenu();
   ::CreateMenuItem(m_hMenu, (LPCTSTR)CLang::GetString(IDS__MENU_GAME__BEGINNER    ), 0, IDM_GAME_BEGINNER    , NULL, FALSE, MFT_STRING);
   ::CreateMenuItem(m_hMenu, (LPCTSTR)CLang::GetString(IDS__MENU_GAME__AMATEUR     ), 0, IDM_GAME_AMATEUR     , NULL, FALSE, MFT_STRING);
   ::CreateMenuItem(m_hMenu, (LPCTSTR)CLang::GetString(IDS__MENU_GAME__PROFESSIONAL), 0, IDM_GAME_PROFESSIONAL, NULL, FALSE, MFT_STRING);
   ::CreateMenuItem(m_hMenu, (LPCTSTR)CLang::GetString(IDS__MENU_GAME__CRAZY       ), 0, IDM_GAME_CRAZY       , NULL, FALSE, MFT_STRING);
   GetFM2WinMain().SetOwnerDrawMenu(m_hMenu);

   m_BtnPopup.SetBitmap(::CreateBitmapDownArrow());

   SETNEWWNDPROC(m_hWnd, IDC_CUSTOMSKILL_GROUPBOX   );
   SETNEWWNDPROC(m_hWnd, IDC_CUSTOMSKILL_STATIC_X   );
   SETNEWWNDPROC(m_hWnd, IDC_CUSTOMSKILL_STATIC_Y   );
   SETNEWWNDPROC(m_hWnd, IDC_CUSTOMSKILL_STATIC_MINE);
   SETNEWWNDPROC(m_hWnd, IDC_CUSTOMSKILL_SPIN_X     );
   SETNEWWNDPROC(m_hWnd, IDC_CUSTOMSKILL_SPIN_Y     );
   SETNEWWNDPROC(m_hWnd, IDC_CUSTOMSKILL_SPIN_MINE  );
   SETNEWWNDPROC(m_hWnd, IDC_CUSTOMSKILL_FULLSCREEN1);
   SETNEWWNDPROC(m_hWnd, IDC_CUSTOMSKILL_FULLSCREEN2);

   CDialogMessageHook::InstallHook(m_hWnd);

   return TRUE;
}

void CDialogCustomSkill::OnMeasureItem(UINT idControl, LPMEASUREITEMSTRUCT lpMeasureItem) {
   if (lpMeasureItem) {
      if (lpMeasureItem->CtlType == ODT_MENU) {
         CMenuItem::OnMeasureItem(lpMeasureItem);
      }
   }
}

void CDialogCustomSkill::OnDrawItem(UINT idControl, LPDRAWITEMSTRUCT lpDrawItem) {
   if (lpDrawItem) {
      if (lpDrawItem->CtlType == ODT_MENU) {
         CMenuItem::OnDrawItem(lpDrawItem);
      }
   }
}

#define CDialogCustomSkill__OnSpinChange(iVal, spin, iMax) { \
   int iRes = spin.GetPos(); \
   iVal = min(LOWORD(iRes), iMax); \
   spin.SetPos(iVal); \
}

void CDialogCustomSkill::OnSpinChangeX(UINT /*wNotifyCode*/, int /*idControl*/, HWND /*hWndCtl*/) {
   if (m_SpinX.m_hWnd) {
      CDialogCustomSkill__OnSpinChange(m_sizeNewMosaic.cx, m_SpinX, GetMaxSize(false).cx);
      PostMessage(WM_COMMAND, MAKEWPARAM(IDC_CUSTOMSKILL_EDIT_MINE, EN_CHANGE), 0);

      SendDlgItemMessage(IDC_CUSTOMSKILL_FULLSCREEN1, BM_SETCHECK, BST_UNCHECKED, 0L);
      SendDlgItemMessage(IDC_CUSTOMSKILL_FULLSCREEN2, BM_SETCHECK, BST_UNCHECKED, 0L);
   }
}

void CDialogCustomSkill::OnSpinChangeY(UINT /*wNotifyCode*/, int /*idControl*/, HWND /*hWndCtl*/) {
   if (m_SpinY.m_hWnd) {
      CDialogCustomSkill__OnSpinChange(m_sizeNewMosaic.cy, m_SpinY, GetMaxSize(false).cy);
      PostMessage(WM_COMMAND, MAKEWPARAM(IDC_CUSTOMSKILL_EDIT_MINE, EN_CHANGE), 0);

      SendDlgItemMessage(IDC_CUSTOMSKILL_FULLSCREEN1, BM_SETCHECK, BST_UNCHECKED, 0L);
      SendDlgItemMessage(IDC_CUSTOMSKILL_FULLSCREEN2, BM_SETCHECK, BST_UNCHECKED, 0L);
   }
}

void CDialogCustomSkill::OnSpinChangeMine(UINT /*wNotifyCode*/, int /*idControl*/, HWND /*hWndCtl*/) {
   if (m_SpinMine.m_hWnd) {
      CDialogCustomSkill__OnSpinChange(m_iNewMines, m_SpinMine, GetFM2WinMain().GetMaxMines(m_sizeNewMosaic));
   }
}

void CDialogCustomSkill::OnClickFullScreen(UINT /*wNotifyCode*/, int idControl, HWND hWndCtl) {
   //if (BST_CHECKED && Button_GetState(hWndCtl)) {
   if (BST_CHECKED == ::SendMessage(hWndCtl, BM_GETCHECK, 0L, 0L)) {
      SIZE s = GetMaxSize(idControl == IDC_CUSTOMSKILL_FULLSCREEN1);
      m_SpinX.SetPos(s.cx);
      m_SpinY.SetPos(s.cy);
      SendDlgItemMessage((idControl == IDC_CUSTOMSKILL_FULLSCREEN1) ?
                                      IDC_CUSTOMSKILL_FULLSCREEN2 :
                                      IDC_CUSTOMSKILL_FULLSCREEN1, BM_SETCHECK, BST_UNCHECKED, 0L);
      ::SendMessage(hWndCtl, BM_SETCHECK, BST_CHECKED, 0);
   }
}

void CDialogCustomSkill::OnClickPopupMenu(UINT /*wNotifyCode*/, int /*idControl*/, HWND hWndCtl) {
   RECTEX rc = ::GetWindowRect(hWndCtl);
   OnContextMenu(hWndCtl, rc.pointLB());
}

void CDialogCustomSkill::OnClickOkCancel(UINT /*wNotifyCode*/, int idControl, HWND /*hWndCtl*/) {
   if (idControl == IDOK) {
      //DoDataExchange(TRUE);

      SendMessage(WM_COMMAND, MAKEWPARAM(IDC_CUSTOMSKILL_EDIT_X   , EN_CHANGE), 0);
      SendMessage(WM_COMMAND, MAKEWPARAM(IDC_CUSTOMSKILL_EDIT_Y   , EN_CHANGE), 0);
      SendMessage(WM_COMMAND, MAKEWPARAM(IDC_CUSTOMSKILL_EDIT_MINE, EN_CHANGE), 0);

      GetFM2WinMain().SetSkillLevelCustom(m_sizeNewMosaic, m_iNewMines);
   }
   SendMessage(WM_CLOSE, 0L, 0L);
}

void CDialogCustomSkill::OnClose() {
   m_hMenu.DestroyMenu();
   DestroyWindow();
}

void CDialogCustomSkill::OnDestroy() {
   ::DeleteObject(m_BtnPopup.GetBitmap());
   CDialogMessageHook::UninstallHook(m_hWnd);
}

BOOL CDialogCustomSkill::OnEraseBkgnd(HDC hdc) {
   if (!GetFM2WinMain().GetSkin().m_bToAll) {
      SetMsgHandled(FALSE);
      return FALSE;
   }
   return nsEraseBk::OnEraseBkgnd(m_hWnd, hdc, GetFM2WinMain().GetSkin().GetBkColor());
}

void CDialogCustomSkill::OnContextMenu(HWND hWndContext, const CPoint &pos) {
   int id = m_hMenu.TrackPopupMenu(
      TPM_LEFTALIGN | TPM_TOPALIGN | TPM_RETURNCMD | TPM_LEFTBUTTON | TPM_RIGHTBUTTON,
      pos.x, pos.y,
      m_hWnd,
      NULL);
   ESkillLevel skill;
   switch (id) {
   case IDM_GAME_BEGINNER    :
   case IDM_GAME_AMATEUR     :
   case IDM_GAME_PROFESSIONAL:
   case IDM_GAME_CRAZY       : skill = (ESkillLevel)(id-IDM_GAME_BEGINNER); break;
   default: return;
   }
   m_iNewMines = GetFM2WinMain().DefineNumberMines(skill, m_sizeNewMosaic);
   m_SpinMine.SetPos(m_iNewMines);
}

#endif // _WTL_VER
