////////////////////////////////////////////////////////////////////////////////
//                               FastMines project
//                                       Sergey Krivulya (KSerg, aka SeregaLBN)
// file name: "Skin.h"
//
//
//
// Copyright (C) 2003-2007 Sergey Krivulya
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
////////////////////////////////////////////////////////////////////////////////

#ifndef __FILE__SKIN__
#define __FILE__SKIN__

#include "../Control/ButtonFM.h"
#include "../Control/EditFM.h"
#include "../FastMines2.h"

#ifndef _WTL_VER

class CDialogChangeSkin {
private:
   enum ESkinPart {
      eSkinImageMine,
      eSkinImageFlag,
      eSkinImagePause,
      eSkinImageBtnPause,
      eSkinImageBtnNew,
      eSkinImageBckgrnd,
      eSkinColorBckgrnd,
      eSkinBorder,
      eSkinColorText,
      eSkinTypeFont
   };
   class CWinSkinPreView {
   public:
      HWND m_hWnd;
   private:
      CDialogChangeSkin &m_SkinDialog;
   private:
      static LPCTSTR GetWindowClass() {
         const static TCHAR szWndClass[] = _T("CWinSkinPreView");
         return szWndClass;
      }

      static LRESULT CALLBACK WndProcThis(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
         CWinSkinPreView *This = NULL;
         if (uMsg == WM_CREATE) {
            CREATESTRUCT *pCS = (CREATESTRUCT*)lParam;
            This = pCS ? (CWinSkinPreView*)pCS->lpCreateParams : NULL;
            if (This) {
               This->m_hWnd = hWnd;
               ::SetWindowUserData(hWnd, (LONG_PTR)This);
            }            
         } else {
            This = (CWinSkinPreView*)::GetWindowUserData(hWnd);
         }
         if (This) {
            return This->WndProc(hWnd, uMsg, wParam, lParam);
         }

         return ::DefWindowProc(hWnd, uMsg, wParam, lParam);
      }

      LRESULT WndProc(HWND, UINT uMsg, WPARAM wParam, LPARAM lParam) {
         switch(uMsg){
         HANDLE_MSG(m_hWnd, WM_SETTEXT   , OnSetText);
         HANDLE_MSG(m_hWnd, WM_PAINT     , OnPaint);
         HANDLE_MSG(m_hWnd, WM_ERASEBKGND, OnEraseBkgnd);
         }
         return ::DefWindowProc(m_hWnd, uMsg, wParam, lParam);
      }

   public:
      CWinSkinPreView(CDialogChangeSkin &SkinDialog);

      BOOL Create(HWND hWndParent, const RECT &rc);

   private:
      void OnSetText   (HWND, LPCTSTR);
      void OnPaint     (HWND);
      BOOL OnEraseBkgnd(HWND, HDC);
   };
   friend CWinSkinPreView;
public:
   HWND m_hWnd;
private:
   CButtonExFM     m_BtnOk, m_BtnCancel;
   CButtonExFM     m_BtnImgMine, m_BtnImgFlag, m_BtnImgPause,
                   m_BtnImgBtnPause, m_BtnImgBtnNew,
                   m_BtnImgBkgrnd, m_BtnColorBkgrnd, m_BtnImgBorder,
                   m_BtnImgColorText, m_BtnImgTypeFont;
   CButtonExFM     m_BtnSelect, m_BtnSelect2, m_BtnReset;
   CEditGradientFM m_EditWidth;
   CSkin           m_Skin;
   ESkinPart       m_skinPart;
   HWND            m_hWndSpinPreView;
   CWinSkinPreView m_PreView;

private:
   static LPCTSTR IDD() { static TCHAR szResourceName[] = _T("ChangeSkin"); return szResourceName; }
   static INT_PTR CALLBACK DialogProcThis(HWND, UINT, WPARAM, LPARAM);
   INT_PTR DialogProc(HWND, UINT uMsg, WPARAM wParam, LPARAM lParam)  {
      switch (uMsg){
      HANDLE_MSG(m_hWnd, WM_INITDIALOG , OnInitDialog);
      HANDLE_MSG(m_hWnd, WM_COMMAND    , OnCommand);
      HANDLE_MSG(m_hWnd, WM_CLOSE      , OnClose);
      HANDLE_MSG(m_hWnd, WM_DESTROY    , OnDestroy);
      HANDLE_MSG(m_hWnd, WM_ERASEBKGND , OnEraseBkgnd);
      }
      return FALSE;
   }
public:
   CDialogChangeSkin();
   HWND Create(HWND hWndParent);
private:
   void ReInitDialog();
   BOOL OnInitDialog(HWND, HWND, LPARAM);
   void OnCommand   (HWND, int, HWND, UINT);
   void OnClose     (HWND);
   void OnDestroy   (HWND);
   BOOL OnEraseBkgnd(HWND, HDC);

   CImageSerialize  GetDefaultImage();
   CImageSerialize& GetSkinImage();
   COLORREF         GetSkinColor(bool bBtnSelect2);
   void             SetSkinColor(COLORREF, bool bBtnSelect2);
   void             AcceptImage();

   void InvalidateFrame();
};

////////////////////////////////////////////////////////////////////////////////////////////////////////
#else // _WTL_VER
////////////////////////////////////////////////////////////////////////////////////////////////////////

class CDialogChangeSkin : public CDialogImpl<CDialogChangeSkin>, public CWinDataExchange<CDialogChangeSkin> {
private:
   class CButtonImpl: public CWindowImpl<CButtonImpl, CButton    > { DECLARE_EMPTY_MSG_MAP() };
   class CStaticImpl: public CWindowImpl<CStaticImpl, CStatic    > { DECLARE_EMPTY_MSG_MAP() };
   class CSpinImpl  : public CWindowImpl<CSpinImpl  , CUpDownCtrl> { DECLARE_EMPTY_MSG_MAP() };
private:
   enum ESkinPart {
      eSkinImageMine,
      eSkinImageFlag,
      eSkinImagePause,
      eSkinImageBtnPause,
      eSkinImageBtnNew,
      eSkinImageBckgrnd,
      eSkinColorBckgrnd,
      eSkinBorder,
      eSkinColorText,
      eSkinTypeFont
   };
   class CWinSkinPreView: public CWindowImpl<CWinSkinPreView, CWindow, CWinTraits<WS_CHILD | WS_VISIBLE | WS_DLGFRAME> > {
   private:
      CDialogChangeSkin &m_SkinDialog;
   public:
      DECLARE_WND_CLASS(_T("CWinSkinPreView"))
   private:

      BEGIN_MSG_MAP_EX(CWinSkinPreView)
         MSG_WM_SETTEXT   (OnSetText)
         MSG_WM_PAINT     (OnPaint)
         MSG_WM_ERASEBKGND(OnEraseBkgnd)
      END_MSG_MAP()

   public:
      CWinSkinPreView(CDialogChangeSkin &SkinDialog);

   private:
      BOOL OnSetText   (LPCTSTR);
      void OnPaint     (HDC);
      BOOL OnEraseBkgnd(HDC);
   };
   friend CWinSkinPreView;
private:
   CButtonExFM     m_BtnOk, m_BtnCancel;
   CButtonExFM     m_BtnImgMine, m_BtnImgFlag, m_BtnImgPause,
                   m_BtnImgBtnPause, m_BtnImgBtnNew,
                   m_BtnImgBkgrnd, m_BtnColorBkgrnd, m_BtnImgBorder,
                   m_BtnImgColorText, m_BtnImgTypeFont;
   CButtonExFM     m_BtnSelect, m_BtnSelect2, m_BtnReset;
   CButtonImpl     m_BtnCheckTransparentOrSetAll;
   CButtonImpl     m_BtnRadioPlaceTopLeft   , m_BtnRadioPlaceTopCenter   , m_BtnRadioPlaceTopRight,
                   m_BtnRadioPlaceMiddleLeft, m_BtnRadioPlaceMiddleCenter, m_BtnRadioPlaceMiddleRight,
                   m_BtnRadioPlaceBottomLeft, m_BtnRadioPlaceBottomCenter, m_BtnRadioPlaceBottomRight,
                   m_BtnRadioPlaceAsIs, m_BtnRadioPlaceUniformly, m_BtnRadioPlaceStretch, m_BtnRadioPlaceTile;
   CStaticImpl     m_StcPlace, m_StcAlign, m_StcFrame, m_StcWidth;
   CEditGradientFM m_EditWidth;
   CSkin           m_Skin;
   ESkinPart       m_skinPart;
   CSpinImpl       m_SpinPreView, m_SpinWidth;
   CWinSkinPreView m_PreView;
   
public:
   static LPCTSTR IDD() { static TCHAR szResourceName[] = _T("ChangeSkin"); return szResourceName; }//enum { IDD = 0 };

   CDialogChangeSkin();
	// modeless dialogs
	HWND Create(HWND hWndParent, LPARAM dwInitParam = NULL);

private:
   BEGIN_MSG_MAP_EX(CDialogChangeSkin)
      MSG_WM_INITDIALOG (OnInitDialog)
      COMMAND_HANDLER_EX(IDC_CHANGESKIN_BUTTON_IMAGE_MINE         , BN_CLICKED, OnClickSkinPart)
      COMMAND_HANDLER_EX(IDC_CHANGESKIN_BUTTON_IMAGE_FLAG         , BN_CLICKED, OnClickSkinPart)
      COMMAND_HANDLER_EX(IDC_CHANGESKIN_BUTTON_IMAGE_PAUSE        , BN_CLICKED, OnClickSkinPart)
      COMMAND_HANDLER_EX(IDC_CHANGESKIN_BUTTON_IMAGE_BTNPAUSE     , BN_CLICKED, OnClickSkinPart)
      COMMAND_HANDLER_EX(IDC_CHANGESKIN_BUTTON_IMAGE_BTNNEW       , BN_CLICKED, OnClickSkinPart)
      COMMAND_HANDLER_EX(IDC_CHANGESKIN_BUTTON_IMAGE_BCKGRND      , BN_CLICKED, OnClickSkinPart)
      COMMAND_HANDLER_EX(IDC_CHANGESKIN_BUTTON_COLOR_BCKGRND      , BN_CLICKED, OnClickSkinPart)
      COMMAND_HANDLER_EX(IDC_CHANGESKIN_BUTTON_BORDER             , BN_CLICKED, OnClickSkinPart)
      COMMAND_HANDLER_EX(IDC_CHANGESKIN_BUTTON_COLOR_TEXT         , BN_CLICKED, OnClickSkinPart)
      COMMAND_HANDLER_EX(IDC_CHANGESKIN_BUTTON_TYPE_FONT          , BN_CLICKED, OnClickSkinPart)
      COMMAND_HANDLER_EX(IDC_CHANGESKIN_BUTTON_SELECT             , BN_CLICKED, OnClickSelect)
      COMMAND_HANDLER_EX(IDC_CHANGESKIN_BUTTON_SELECT2            , BN_CLICKED, OnClickSelect)
      COMMAND_HANDLER_EX(IDC_CHANGESKIN_BUTTON_TRANSPARENTorSETALL, BN_CLICKED, OnClickTransparentOrSetAll)
      COMMAND_HANDLER_EX(IDC_CHANGESKIN_BUTTON_RESET              , BN_CLICKED, OnClickReset)
      COMMAND_HANDLER_EX(IDC_CHANGESKIN_EDIT_WIDHT                , EN_CHANGE , OnSpinWidth)
      COMMAND_HANDLER_EX(IDC_CHANGESKIN_RADIO_TOP_LEFT            , BN_CLICKED, OnClickPlace)
      COMMAND_HANDLER_EX(IDC_CHANGESKIN_RADIO_TOP_CENTER          , BN_CLICKED, OnClickPlace)
      COMMAND_HANDLER_EX(IDC_CHANGESKIN_RADIO_TOP_RIGHT           , BN_CLICKED, OnClickPlace)
      COMMAND_HANDLER_EX(IDC_CHANGESKIN_RADIO_MIDDLE_LEFT         , BN_CLICKED, OnClickPlace)
      COMMAND_HANDLER_EX(IDC_CHANGESKIN_RADIO_MIDDLE_CENTER       , BN_CLICKED, OnClickPlace)
      COMMAND_HANDLER_EX(IDC_CHANGESKIN_RADIO_MIDDLE_RIGHT        , BN_CLICKED, OnClickPlace)
      COMMAND_HANDLER_EX(IDC_CHANGESKIN_RADIO_BOTTOM_LEFT         , BN_CLICKED, OnClickPlace)
      COMMAND_HANDLER_EX(IDC_CHANGESKIN_RADIO_BOTTOM_CENTER       , BN_CLICKED, OnClickPlace)
      COMMAND_HANDLER_EX(IDC_CHANGESKIN_RADIO_BOTTOM_RIGHT        , BN_CLICKED, OnClickPlace)
      COMMAND_HANDLER_EX(IDC_CHANGESKIN_RADIO_AS_IS               , BN_CLICKED, OnClickPlace)
      COMMAND_HANDLER_EX(IDC_CHANGESKIN_RADIO_UNIFORMLY           , BN_CLICKED, OnClickPlace)
      COMMAND_HANDLER_EX(IDC_CHANGESKIN_RADIO_STRETCH             , BN_CLICKED, OnClickPlace)
      COMMAND_HANDLER_EX(IDC_CHANGESKIN_RADIO_TILE                , BN_CLICKED, OnClickPlace)
      COMMAND_HANDLER_EX(IDOK                                     , BN_CLICKED, OnClickOkCancel)
      COMMAND_HANDLER_EX(IDCANCEL                                 , BN_CLICKED, OnClickOkCancel)
      MSG_WM_CLOSE      (OnClose)
      MSG_WM_DESTROY    (OnDestroy)
      MSG_WM_ERASEBKGND (OnEraseBkgnd)
   END_MSG_MAP()
	BEGIN_DDX_MAP(CDialogChangeSkin)
      DDX_CONTROL(IDOK                                     , m_BtnOk)
      DDX_CONTROL(IDCANCEL                                 , m_BtnCancel)
      DDX_CONTROL(IDC_CHANGESKIN_BUTTON_IMAGE_MINE         , m_BtnImgMine)
      DDX_CONTROL(IDC_CHANGESKIN_BUTTON_IMAGE_FLAG         , m_BtnImgFlag)
      DDX_CONTROL(IDC_CHANGESKIN_BUTTON_IMAGE_PAUSE        , m_BtnImgPause)
      DDX_CONTROL(IDC_CHANGESKIN_BUTTON_IMAGE_BTNPAUSE     , m_BtnImgBtnPause)
      DDX_CONTROL(IDC_CHANGESKIN_BUTTON_IMAGE_BTNNEW       , m_BtnImgBtnNew)
      DDX_CONTROL(IDC_CHANGESKIN_BUTTON_IMAGE_BCKGRND      , m_BtnImgBkgrnd)
      DDX_CONTROL(IDC_CHANGESKIN_BUTTON_COLOR_BCKGRND      , m_BtnColorBkgrnd)
      DDX_CONTROL(IDC_CHANGESKIN_BUTTON_BORDER             , m_BtnImgBorder)
      DDX_CONTROL(IDC_CHANGESKIN_BUTTON_COLOR_TEXT         , m_BtnImgColorText)
      DDX_CONTROL(IDC_CHANGESKIN_BUTTON_TYPE_FONT          , m_BtnImgTypeFont)
      DDX_CONTROL(IDC_CHANGESKIN_BUTTON_SELECT             , m_BtnSelect)
      DDX_CONTROL(IDC_CHANGESKIN_BUTTON_SELECT2            , m_BtnSelect2)
      DDX_CONTROL(IDC_CHANGESKIN_BUTTON_TRANSPARENTorSETALL, m_BtnCheckTransparentOrSetAll)
      DDX_CONTROL(IDC_CHANGESKIN_BUTTON_RESET              , m_BtnReset)
      DDX_CONTROL(IDC_CHANGESKIN_RADIO_TOP_LEFT            , m_BtnRadioPlaceTopLeft     )
      DDX_CONTROL(IDC_CHANGESKIN_RADIO_TOP_CENTER          , m_BtnRadioPlaceTopCenter   )
      DDX_CONTROL(IDC_CHANGESKIN_RADIO_TOP_RIGHT           , m_BtnRadioPlaceTopRight    )
      DDX_CONTROL(IDC_CHANGESKIN_RADIO_MIDDLE_LEFT         , m_BtnRadioPlaceMiddleLeft  )
      DDX_CONTROL(IDC_CHANGESKIN_RADIO_MIDDLE_CENTER       , m_BtnRadioPlaceMiddleCenter)
      DDX_CONTROL(IDC_CHANGESKIN_RADIO_MIDDLE_RIGHT        , m_BtnRadioPlaceMiddleRight )
      DDX_CONTROL(IDC_CHANGESKIN_RADIO_BOTTOM_LEFT         , m_BtnRadioPlaceBottomLeft  )
      DDX_CONTROL(IDC_CHANGESKIN_RADIO_BOTTOM_CENTER       , m_BtnRadioPlaceBottomCenter)
      DDX_CONTROL(IDC_CHANGESKIN_RADIO_BOTTOM_RIGHT        , m_BtnRadioPlaceBottomRight )
      DDX_CONTROL(IDC_CHANGESKIN_RADIO_AS_IS               , m_BtnRadioPlaceAsIs        )
      DDX_CONTROL(IDC_CHANGESKIN_RADIO_UNIFORMLY           , m_BtnRadioPlaceUniformly   )
      DDX_CONTROL(IDC_CHANGESKIN_RADIO_STRETCH             , m_BtnRadioPlaceStretch     )
      DDX_CONTROL(IDC_CHANGESKIN_RADIO_TILE                , m_BtnRadioPlaceTile        )
      DDX_CONTROL(IDC_CHANGESKIN_STATIC_PLACE              , m_StcPlace)
      DDX_CONTROL(IDC_CHANGESKIN_STATIC_ALIGMENT           , m_StcAlign)
      DDX_CONTROL(IDC_CHANGESKIN_BUTTON_FRAME              , m_StcFrame)
    //DDX_CONTROL(                                         , m_SpinPreView)
      DDX_CONTROL(IDC_CHANGESKIN_EDIT_WIDHT                , m_EditWidth)
      DDX_CONTROL(IDC_CHANGESKIN_STATIC_WIDHT              , m_StcWidth)
      DDX_CONTROL(IDC_CHANGESKIN_SPIN_WIDHT                , m_SpinWidth)
    //DDX_CONTROL(IDC_CHANGESKIN_SPIN_PREVIEW              , m_PreView)
	END_DDX_MAP()

   void ReInitDialog();
   BOOL OnInitDialog(HWND, LPARAM);
   void OnClickSkinPart           (UINT, int, HWND);
   void OnClickSelect             (UINT, int, HWND);
   void OnSpinWidth               (UINT, int, HWND);
   void OnClickPlace              (UINT, int, HWND);
   void OnClickTransparentOrSetAll(UINT, int, HWND);
   void OnClickReset              (UINT, int, HWND);
   void OnClickOkCancel           (UINT, int, HWND);
   void OnClose();
   void OnDestroy();
   BOOL OnEraseBkgnd(HDC hDC);

   CImageSerialize  GetDefaultImage();
   CImageSerialize& GetSkinImage();
   COLORREF         GetSkinColor(bool bBtnSelect2);
   void             SetSkinColor(COLORREF, bool bBtnSelect2);
   void             AcceptImage();

   void InvalidateFrame();
};

#endif // _WTL_VER

class CSkinMenu {
public:
   static void  LoadMenuList  (HMENU);
   static void  ReloadMenuList(HMENU);
   static CSkin LoadSkin      (HMENU, UINT);
private:
   static BOOL CALLBACK FileEnumProc(IN LPCTSTR szDirName, IN LPCTSTR szFileName, IN UINT iPos, IN LPVOID lpUserData);
};

struct CFileSkin : public CCommonSerializeEx<IFileHeaderImpl<0x84C0501B>, CSkin,
                                             IDS__FILESKIN__ERROR_CREATE,
                                             IDS__FILESKIN__ERROR_OPEN,
                                             IDS__FILESKIN__ERROR_READ,
                                             IDS__FILESKIN__ERROR_WRITE,
                                             IDS__FILESKIN__ERROR_VERSION>
{
private:
   CString m_strSkinName;
protected:
   BOOL IgnoreFileNotFound() const { return FALSE; }
   BOOL Convert(IN HANDLE);
   void ChangePathAndCopy(CString &);
public:
   static const CString& GetSkinFileExtension() { static CString str = _T("skn"); return str; }
   static const CString& GetAllSkinDir() {
      static CString strDir = ::GetModuleDir(::GetModuleHandle(NULL)) + _T("Skin");
      return strDir;
   }
   CString GetThisSkinSubDir() const {
      CString strDir = GetAllSkinDir() + _T('\\') + m_strSkinName;
      return strDir;
   }
   void SetSkinName(const CString &strSkinName) {
      static BOOL bRes = ::CreateDirectory(GetAllSkinDir(), NULL); //    .    
      m_strSkinName = strSkinName;
      ::CreateDirectory(GetThisSkinSubDir(), NULL); //    
   }
   CString GetFileName() const { return GetThisSkinSubDir()+_T('.')+GetSkinFileExtension(); }

   void  SetSkin(const CSkin skin);
   CSkin GetSkin() const;
};

#endif // __FILE__SKIN__
