////////////////////////////////////////////////////////////////////////////////
//                               FastMines project
//                                       Sergey Krivulya (KSerg, aka SeregaLBN)
// file name: "FMToolbar.h"
// Copyright (C) 2006-2007 Sergey Krivulya
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
////////////////////////////////////////////////////////////////////////////////

#ifndef __FILE__FMTOOLBAR__
#define __FILE__FMTOOLBAR__

#include "../Control/ButtonFM.h"
#include "../Control/EditFM.h"
#include "../Resource.h"

#ifndef _WTL_VER

class CToolbarFM {
public:
   HWND            m_hWnd;
private:
   CButtonImageFM  m_BtnNew, m_BtnPause;
   CEditGradientFM m_EditMines, m_EditTime;
private:
   static LPCTSTR IDD() { static TCHAR szResourceName[] = _T("ToolbarFM"); return szResourceName; }
   static INT_PTR CALLBACK DialogProcThis(HWND, UINT, WPARAM, LPARAM);
   INT_PTR DialogProc(HWND, UINT uMsg, WPARAM wParam, LPARAM lParam) {
      switch (uMsg) {
      HANDLE_MSG(m_hWnd, WM_INITDIALOG, OnInitDialog);
      HANDLE_MSG(m_hWnd, WM_COMMAND   , OnCommand);
      HANDLE_MSG(m_hWnd, WM_SIZE      , OnSize);
      HANDLE_MSG(m_hWnd, WM_DESTROY   , OnDestroy);
      HANDLE_MSG(m_hWnd, WM_ERASEBKGND, OnEraseBkgnd);
      }
      return FALSE;
   }

public:
   CToolbarFM() : m_hWnd(NULL) {}
   HWND Create(HWND);
   static int GetHeightWindow() { return 40; }

   BOOL SetImageNew  (const CImageSerialize &img, WORD nImage) { return m_BtnNew  .SetImage(img, nImage); }
   BOOL SetImagePause(const CImageSerialize &img, WORD nImage) { return m_BtnPause.SetImage(img, nImage); }
   void SetTextMines(LPCTSTR szText) { ::SetWindowText(::GetDlgItem(m_hWnd, IDC_FMTOOBAR_EDIT_MINES), szText); }
   void SetTextTimer(LPCTSTR szText) { ::SetWindowText(::GetDlgItem(m_hWnd, IDC_FMTOOBAR_EDIT_TIME ), szText); }
   void BtnPauseCheck (BOOL bCheck ) { ::SendMessage(m_BtnPause.m_hWnd, BM_SETCHECK, bCheck ? BST_CHECKED : BST_UNCHECKED, 0L); }
   void BtnPauseEnable(BOOL bEnable) { ::EnableWindow(m_BtnPause.m_hWnd, bEnable); }
   void SetFont(const LOGFONT &font);

private:
   BOOL OnInitDialog(HWND, HWND, LPARAM);
   void OnCommand   (HWND, int, HWND, UINT);
   void OnSize      (HWND, UINT, int, int);
   void OnDestroy   (HWND);
   BOOL OnEraseBkgnd(HWND, HDC);
};

////////////////////////////////////////////////////////////////////////////////////////////////////////
#else // _WTL_VER
////////////////////////////////////////////////////////////////////////////////////////////////////////

class CToolbarFM : public CDialogImpl<CToolbarFM>, public CWinDataExchange<CToolbarFM>
{
private:
   CButtonImageFM  m_BtnNew, m_BtnPause;
   CEditGradientFM m_EditMines, m_EditTime;
public:
   static LPCTSTR IDD() { static TCHAR szResourceName[] = _T("ToolbarFM"); return szResourceName; }//enum { IDD = 0 };

	// modeless dialogs
	HWND Create(HWND hWndParent, LPARAM dwInitParam = NULL);
   static int GetHeightWindow() { return 40; }

   BOOL SetImageNew  (const CImageSerialize &img, WORD nImage) { return m_BtnNew  .SetImage(img, nImage); }
   BOOL SetImagePause(const CImageSerialize &img, WORD nImage) { return m_BtnPause.SetImage(img, nImage); }
   void SetTextMines(LPCTSTR szText) { m_EditMines.SetWindowText(szText); }
   void SetTextTimer(LPCTSTR szText) { m_EditTime .SetWindowText(szText); }
   void BtnPauseCheck (BOOL bCheck ) { m_BtnPause.SetCheck(bCheck ? BST_CHECKED : BST_UNCHECKED); }
   void BtnPauseEnable(BOOL bEnable) { m_BtnPause.EnableWindow(bEnable); }
   void SetFont(const LOGFONT &font);

private:
   BEGIN_MSG_MAP_EX(CToolbarFM)
      MSG_WM_INITDIALOG(OnInitDialog)
      COMMAND_HANDLER_EX(IDC_FMTOOBAR_BUTTON_NEW  , BN_CLICKED , OnClickBtnNew)
      COMMAND_HANDLER_EX(IDC_FMTOOBAR_BUTTON_NEW  , BN_SETFOCUS, OnFocusBtnNew)
      COMMAND_HANDLER_EX(IDC_FMTOOBAR_BUTTON_PAUSE, BN_CLICKED , OnClickBtnPause)
      COMMAND_HANDLER_EX(IDC_FMTOOBAR_BUTTON_PAUSE, BN_SETFOCUS, OnFocusBtnPause)
      COMMAND_HANDLER_EX(IDC_FMTOOBAR_EDIT_MINES  , EN_SETFOCUS, OnFocusEdit)
      COMMAND_HANDLER_EX(IDC_FMTOOBAR_EDIT_TIME   , EN_SETFOCUS, OnFocusEdit)
      MSG_WM_SIZE      (OnSize)
      MSG_WM_ERASEBKGND(OnEraseBkgnd)
   END_MSG_MAP()
	BEGIN_DDX_MAP(CDialogAbout)
      DDX_CONTROL(IDC_FMTOOBAR_EDIT_MINES  , m_EditMines)
      DDX_CONTROL(IDC_FMTOOBAR_EDIT_TIME   , m_EditTime)
      DDX_CONTROL(IDC_FMTOOBAR_BUTTON_NEW  , m_BtnNew)
      DDX_CONTROL(IDC_FMTOOBAR_BUTTON_PAUSE, m_BtnPause)
	END_DDX_MAP()

   BOOL OnInitDialog(HWND, LPARAM);
   void OnSize      (UINT, CSize&);
   BOOL OnEraseBkgnd(HDC hDC);

   void OnClickBtnNew  (UINT /*wNotifyCode*/, int /*idControl*/, HWND /*hWndCtl*/) { FORWARD_WM_COMMAND(GetParent(), IDA_NEW_GAME, m_BtnNew  .m_hWnd, 0, ::SendMessage); }
   void OnClickBtnPause(UINT /*wNotifyCode*/, int /*idControl*/, HWND /*hWndCtl*/) { FORWARD_WM_COMMAND(GetParent(), IDA_PAUSE   , m_BtnPause.m_hWnd, 0, ::SendMessage); }
   void OnFocusBtnNew  (UINT /*wNotifyCode*/, int /*idControl*/, HWND /*hWndCtl*/) { ::SetFocus(GetParent()); }
   void OnFocusBtnPause(UINT /*wNotifyCode*/, int /*idControl*/, HWND /*hWndCtl*/) { ::SetFocus(GetParent()); }
   void OnFocusEdit    (UINT /*wNotifyCode*/, int /*idControl*/, HWND   hWndCtl  ) {
      ::EnableWindow(hWndCtl, FALSE);
      ::EnableWindow(hWndCtl, TRUE );
      ::SetFocus(GetParent());
   }
};

#endif // _WTL_VER

#endif // __FILE__FMTOOLBAR__
