////////////////////////////////////////////////////////////////////////////////
//                               FastMines project
//                                       Sergey Krivulya (KSerg, aka SeregaLBN)
// file name: "EraseBk.h"
//
// -    
//
// Copyright (C) 2002-2007 Sergey Krivulya
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
////////////////////////////////////////////////////////////////////////////////

#ifndef __FILE__ERASEBK__
#define __FILE__ERASEBK__

#include "./FastMines2.h"

namespace nsEraseBk {
   HBRUSH OnCtlColor(HWND hWnd, HDC hDC, HWND hWndChild, int type, const CSkin &Skin);
   BOOL OnEraseBkgnd(HWND hWnd, HDC hDC, COLORREF colorBk);

   void FillWnd(HWND hWnd, COLORREF bkColor, bool client = true, const RECTEX& fillRect = RECTEX(-1,-1,-1,-1));
}

////////////////////////////////////////////////////////////////////////////////
//                   Overdetermination Windows Procedure
////////////////////////////////////////////////////////////////////////////////

#define WNDPROC_EDIT(name, Skin) WNDPROC& defWndProc_##name() { static WNDPROC wndProc;  return wndProc; } \
LRESULT CALLBACK newWndProc_##name(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam) {  \
   /** /LOG_PUT(g_pLog, Put, LL_DEBUG) _T(" msg: %s"), (LPCTSTR)CLogger::WindowMessage(msg)); /**/\
   if ((Skin).m_bToAll)  \
      switch (msg) { \
      case WM_SETFOCUS: \
      case WM_KILLFOCUS: \
      case WM_ENABLE:   \
      case WM_SETTEXT: \
      case WM_UPDATEUISTATE: \
      case WM_KEYDOWN: \
      case WM_KEYUP: \
      case WM_LBUTTONUP: \
      case WM_CAPTURECHANGED: \
      case WM_PAINT: \
         LRESULT lRes = ::CallWindowProc((WNDPROC)defWndProc_##name(), hWnd, msg, wParam, lParam); \
         nsEraseBk::FillWnd(hWnd, (Skin).GetBkColor()); \
         return lRes;   \
      }  \
   return ::CallWindowProc((WNDPROC)defWndProc_##name(), hWnd, msg, wParam, lParam);   \
}

#define WNDPROC_STATIC(name, Skin) WNDPROC& defWndProc_##name() { static WNDPROC wndProc;  return wndProc; } \
LRESULT CALLBACK newWndProc_##name(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam) {  \
   if ((Skin).m_bToAll)  \
      switch (msg) { \
      case WM_UPDATEUISTATE: \
      case WM_PAINT: \
         LRESULT lRes = ::CallWindowProc((WNDPROC)defWndProc_##name(), hWnd, msg, wParam, lParam); \
         nsEraseBk::FillWnd(hWnd, (Skin).GetBkColor()); \
         return lRes;   \
      }  \
   return ::CallWindowProc((WNDPROC)defWndProc_##name(), hWnd, msg, wParam, lParam);   \
}

#define WNDPROC_ONLY_ERASEBKGND(name, Skin) WNDPROC& defWndProc_##name() { static WNDPROC wndProc;  return wndProc; } \
LRESULT CALLBACK newWndProc_##name(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam) {  \
   if ((Skin).m_bToAll)  \
      switch (msg) { \
      case WM_ERASEBKGND: \
         if (!(Skin).m_bToAll) { \
            return FALSE; /* ::DefWindowProc(hWnd, WM_ERASEBKGND, (WPARAM)hDC, 0L); */ \
         } \
         return nsEraseBk::OnEraseBkgnd(hWnd, (HDC)wParam, (Skin).GetBkColor()); \
      }  \
   return ::CallWindowProc((WNDPROC)defWndProc_##name(), hWnd, msg, wParam, lParam);   \
}

#define WNDPROC_BUTTON(name, Skin) WNDPROC& defWndProc_##name() { static WNDPROC wndProc;  return wndProc; } \
LRESULT CALLBACK newWndProc_##name(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam) {   \
   /** /LOG_PUT(g_pLog, Put, LL_DEBUG) _T("  hWnd[0x%08X]; class[%s]; msg: %s"), hWnd, (LPCTSTR)::GetClassName(hWnd), (LPCTSTR)CLogger::WindowMessage(msg)); /**/\
   if ((Skin).m_bToAll)  \
      switch (msg) { \
      case WM_MOUSEMOVE: { \
         LRESULT lRes = ::CallWindowProc((WNDPROC)defWndProc_##name(), hWnd, msg, wParam, lParam);   \
         {  if (wParam & MK_LBUTTON) {  \
               /*POINT cursorPos = {LOWORD(lParam), HIWORD(lParam)}; */ \
               POINTEX cursorPos = ::GetCursorPos();   \
               RECTEX winRect = ::GetWindowRect(hWnd);   \
               BOOL inRegion = ::PointInRect(cursorPos, winRect);  \
               static BOOL oldInRegion = inRegion; \
               if (oldInRegion != inRegion)  \
                  nsEraseBk::FillWnd(hWnd, (Skin).GetBkColor()); \
               oldInRegion = inRegion; \
            }  \
         }  \
         return lRes; } \
      case WM_KEYDOWN:  \
      case WM_ENABLE:   \
      case WM_PAINT: \
      case WM_LBUTTONDBLCLK: \
      case WM_LBUTTONDOWN: \
      case WM_LBUTTONUP:   \
      case BM_SETCHECK: \
      case WM_SETTEXT: \
      case WM_UPDATEUISTATE: { \
         LRESULT lRes = ::CallWindowProc((WNDPROC)defWndProc_##name(), hWnd, msg, wParam, lParam);   \
         nsEraseBk::FillWnd(hWnd, (Skin).GetBkColor()); \
         return lRes; } \
      }  \
   return ::CallWindowProc((WNDPROC)defWndProc_##name(), hWnd, msg, wParam, lParam);  \
}

#define SETNEWWNDPROC(hWnd, name) defWndProc_##name() = (WNDPROC)SetWindowProc(::GetDlgItem(hWnd, name), newWndProc_##name)
//#define SETNEWWNDPROC_ID(hWnd, name, id) defWndProc_##name() = (WNDPROC)SetWindowProc(::GetDlgItem(hWnd, id), newWndProc_##name)
#define SETNEWWNDPROC_HWND(hWnd, name) defWndProc_##name() = (WNDPROC)SetWindowProc(hWnd, newWndProc_##name)

#endif // __FILE__ERASEBK__
