////////////////////////////////////////////////////////////////////////////////
//                               FastMines project
//                                       Sergey Krivulya (KSerg, aka SeregaLBN)
// file name: "FastMines2.cpp"
//
//    
// Copyright (C) 2002-2007 Sergey Krivulya
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
////////////////////////////////////////////////////////////////////////////////

#include "StdAfx.h"
#include "FastMines2.h"
#include <ShellAPI.h>
#include "./WinDefAdd.h"
#include "./EraseBk.h"
#include "./Dialog/Skin.h"
#include "./Dialog/CustomSkill.h"
#include "./Dialog/InputText.h"
#include "./Dialog/Champions.h"
#include "./Dialog/AssistantDlg.h"
#include "./Dialog/About.h"
#include "./Dialog/SelectMosaic.h"
#include "./Dialog/Administration.h"
#include "./ODItems/MenuItemFM.h"
#include "./SerializeImpl.h"

////////////////////////////////////////////////////////////////////////////////
//                        global variables in all project
////////////////////////////////////////////////////////////////////////////////
#define ID_EVENT_TIMER_INACTION 1 // id  ,          
#define ID_EVENT_TIMER_JOB      2 // id  ,     

#define WS_CAPTION_HIDE (WS_CLIPCHILDREN | WS_BORDER  | WS_SYSMENU | WS_MINIMIZEBOX | WS_MAXIMIZEBOX) //WS_POPUPWINDOW //
#define WS_CAPTION_SHOW (WS_CLIPCHILDREN | WS_CAPTION | WS_SYSMENU | WS_MINIMIZEBOX | WS_MAXIMIZEBOX)

#define bMainMenuOwnerDraw TRUE

////////////////////////////////////////////////////////////////////////////////
//                            types & constants
////////////////////////////////////////////////////////////////////////////////
#define WM_SELECT_PLAYER (WM_USER+20)

#define HSUBMENU_SKIN      ::GetSubMenu(::GetSubMenu(::GetSubMenu(m_hMenu, 2), 6), 0)
#define HSUBMENU_ASSISTANT              ::GetSubMenu(::GetSubMenu(m_hMenu, 3), 3)
#define HSUBMENU_LANGUAGE               ::GetSubMenu(::GetSubMenu(m_hMenu, 2), 8)

namespace nsFileGame {
   struct CBody : public IFileBody { // serialize body
      EMosaic m_eMosaic;
      SIZE    m_SizeMosaic;
      UINT    m_iMines;
      CStorageMines m_RepositoryMines;
      CBody():
         m_eMosaic(eMosaicSquare1),
         m_iMines(1)
      {
         m_SizeMosaic.cx = m_SizeMosaic.cy = 10;
         m_iMines = CMosaic::DefineNumberMines(eSkillLevelBeginner, m_eMosaic, m_SizeMosaic);
      }

      void SetData(EMosaic eMosaic, const SIZE &sizeMosaic, const CStorageMines &RepositoryMines) {
         m_eMosaic = eMosaic;
         m_SizeMosaic = sizeMosaic;
         m_RepositoryMines = RepositoryMines;
         m_iMines = m_RepositoryMines.GetSize();
      }

      BOOL SerializeRead(HANDLE hFile) {
         DWORD dwNOBR = 0;
         BOOL bRes = (::ReadFile(hFile, &m_eMosaic   , sizeof(m_eMosaic   ), &dwNOBR, NULL) && (sizeof(m_eMosaic   ) == dwNOBR)) &&
                     (::ReadFile(hFile, &m_SizeMosaic, sizeof(m_SizeMosaic), &dwNOBR, NULL) && (sizeof(m_SizeMosaic) == dwNOBR)) &&
                     (::ReadFile(hFile, &m_iMines    , sizeof(m_iMines    ), &dwNOBR, NULL) && (sizeof(m_iMines    ) == dwNOBR)) &&
                     (m_RepositoryMines.SerializeRead(hFile) && (m_RepositoryMines.GetSize() == m_iMines));
         return bRes;
      }
      BOOL SerializeWrite(HANDLE hFile) const {
         DWORD dwNOBW = 0;
         BOOL bRes = (::WriteFile(hFile, &m_eMosaic   , sizeof(m_eMosaic   ), &dwNOBW, NULL) && (sizeof(m_eMosaic   ) == dwNOBW)) &&
                     (::WriteFile(hFile, &m_SizeMosaic, sizeof(m_SizeMosaic), &dwNOBW, NULL) && (sizeof(m_SizeMosaic) == dwNOBW)) &&
                     (::WriteFile(hFile, &m_iMines    , sizeof(m_iMines    ), &dwNOBW, NULL) && (sizeof(m_iMines    ) == dwNOBW)) &&
                     m_RepositoryMines.SerializeWrite(hFile);
         return bRes;
      }
   };
} // namespace nsFileGame

class CFileGame : public CCommonSerializeEx<IFileHeaderImpl<0x4C0278AD>, nsFileGame::CBody,
                                            IDS__FMS_FILE__ERROR_CREATE,
                                            IDS__FMS_FILE__ERROR_OPEN,
                                            IDS__FMS_FILE__ERROR_READ,
                                            IDS__FMS_FILE__ERROR_WRITE,
                                            IDS__FMS_FILE__ERROR_VERSION>
{
private:
   CString m_strFileName;
protected:
   BOOL IgnoreFileNotFound() const { return FALSE; }
   BOOL Convert(IN HANDLE) { return FALSE; }
public:
   void SetFileName(const CString &strFileName) { m_strFileName = strFileName; }
   CString GetFileName() const { return m_strFileName; }
   void SetData(EMosaic eMosaic, const SIZE &sizeMosaic, const CStorageMines &RepositoryMines) { m_Body.SetData(eMosaic, sizeMosaic, RepositoryMines); }

   EMosaic              GetMosaic    () const { return m_Body.m_eMosaic;         }
   SIZE                 GetSizeMosaic() const { return m_Body.m_SizeMosaic;      }
   int                  GetMines     () const { return m_Body.m_iMines;          }
   CStorageMines const &GetStorage   () const { return m_Body.m_RepositoryMines; }
};

////////////////////////////////////////////////////////////////////////////////
//                              implementation
////////////////////////////////////////////////////////////////////////////////
/* static */ LRESULT CALLBACK CFastMines2WinMain::WndProcThis(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam) {
   /*
   switch (uMsg) {
   case WM_NCPAINT    :
      LOG_PUT(g_pLog, Put, LL_DEBUG) _T("  WndProcThis: msg=%s "), (LPCTSTR)CLogger::WindowMessage(uMsg));
      break;
   case WM_NCACTIVATE :
      LOG_PUT(g_pLog, Put, LL_DEBUG) _T("  WndProcThis: msg=%s wParam=%i "), (LPCTSTR)CLogger::WindowMessage(uMsg));
      break;
   case WM_NCCREATE   :
   case WM_NCDESTROY  :
   case WM_NCCALCSIZE :
   case WM_NCHITTEST  :
   case WM_NCMOUSEMOVE    :
   case WM_NCLBUTTONDOWN  :
   case WM_NCLBUTTONUP    :
   case WM_NCLBUTTONDBLCLK:
   case WM_NCRBUTTONDOWN  :
   case WM_NCRBUTTONUP    :
   case WM_NCRBUTTONDBLCLK:
   case WM_NCMBUTTONDOWN  :
   case WM_NCMBUTTONUP    :
   case WM_NCMBUTTONDBLCLK:
   case WM_NCXBUTTONDOWN  :
   case WM_NCXBUTTONUP    :
   case WM_NCXBUTTONDBLCLK:
      break;
   }
   /**/
   //LOG_PUT(g_pLog, Put, LL_DEBUG) _T("  WndProcThis: msg=%s "), (LPCTSTR)CLogger::WindowMessage(uMsg));
   CFastMines2WinMain *This = NULL;
   if (uMsg == WM_CREATE) {
      CREATESTRUCT *pCS = (CREATESTRUCT*)lParam;
      This = pCS ? (CFastMines2WinMain*)pCS->lpCreateParams : NULL;
      if (This) {
         This->m_hWnd = hWnd;
         ::SetWindowUserData(hWnd, (LONG_PTR)This);
      }            
   } else {
      This = (CFastMines2WinMain*)::GetWindowUserData(hWnd);
   }
   if (This) {
      return This->WndProc(hWnd, uMsg, wParam, lParam);
   }

   return ::DefWindowProc(hWnd, uMsg, wParam, lParam);
}

LRESULT CFastMines2WinMain::WndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam){
   switch(uMsg){
   HANDLE_MSG(hWnd, WM_ERASEBKGND       , OnEraseBkgnd);
   HANDLE_MSG(hWnd, WM_CREATE           , OnCreate);
   HANDLE_MSG(hWnd, WM_MOVE             , OnMove);
   HANDLE_MSG(hWnd, WM_SIZE             , OnSize);
 //HANDLE_MSG(hWnd, WM_WINDOWPOSCHANGING, OnWindowPosChanging);
 //HANDLE_MSG(hWnd, WM_WINDOWPOSCHANGED , OnWindowPosChanged);
   HANDLE_MSG(hWnd, WM_DESTROY          , OnDestroy);
 //HANDLE_MSG(hWnd, WM_GETMINMAXINFO    , OnGetMinMaxInfo);
   HANDLE_MSG(hWnd, WM_SYSCOMMAND       , OnSysCommand);
   HANDLE_MSG(hWnd, WM_COMMAND          , OnCommand);
   HANDLE_MSG(hWnd, WM_ACTIVATE         , OnActivate);
   HANDLE_MSG(hWnd, WM_MEASUREITEM      , OnMeasureItem);
   HANDLE_MSG(hWnd, WM_DRAWITEM         , OnDrawItem);
 //HANDLE_MSG(hWnd, WM_NCPAINT          , OnNCPaint);
   /**/
   case WM_NCACTIVATE: return CMenuNCDrawFM::OnNCActivate(m_hWnd, wParam);
   case WM_NCPAINT   : if    (CMenuNCDrawFM::OnNCPaint(m_hWnd, (HRGN)wParam)) {
         return 0;
      } else {
         return ::DefWindowProc(hWnd, WM_NCPAINT, wParam, 0);
      }
   /**/
   HANDLE_MSG(hWnd, WM_NOTIFYICON             , OnNotifyIcon            );
   HANDLE_MSG(hWnd, WM_MOSAIC_ADJUSTAREA      , OnMosaicAdjustArea      );
   HANDLE_MSG(hWnd, WM_MOSAIC_CLICK           , OnMosaicClick           );
   HANDLE_MSG(hWnd, WM_MOSAIC_CHANGECOUNTERS  , OnMosaicChangeCounters  );
   HANDLE_MSG(hWnd, WM_MOSAIC_CHANGEGAMESTATUS, OnMosaicChangeGameStatus);
   HANDLE_MSG(hWnd, WM_MOSAIC_PAUSE           , OnMosaicPause           );

   HANDLE_MSG(hWnd, WM_MOUSEWHEEL             , OnMouseWheel            );

   case WM_SELECT_PLAYER:                       OnSelectPlayer(hWnd, wParam); return 0L;
   }

   return ::DefWindowProc(hWnd, uMsg, wParam, lParam);
}

////////////////////////////////////////////////////////////////////////////////
//                               CFastMines2WinMain
////////////////////////////////////////////////////////////////////////////////
CFastMines2WinMain::CFastMines2WinMain() :
   m_hWnd(NULL)
{
   m_hIconProject = LoadIcon(::GetModuleHandle(NULL), _T("iconPROJECT"));
   static WNDCLASSEX wc = {
      sizeof(WNDCLASSEX),                    // UINT        cbSize;
      0,                                     // UINT        style;
      CFastMines2WinMain::WndProcThis,       // WNDPROC     lpfnWndProc;
      0,                                     // int         cbClsExtra;
      0,                                     // int         cbWndExtra;
      ::GetModuleHandle(NULL),               // HINSTANCE   hInstance;
      m_hIconProject,                        // HICON       hIcon;
      ::LoadCursor(NULL, IDC_ARROW),         // HCURSOR     hCursor;
      ::GetSysColorBrush(COLOR_BTNFACE),     // HBRUSH      hbrBackground;
      _T("FastMinesMenu"),                   // LPCTSTR     lpszMenuName;
      CFastMines2WinMain::GetWindowClass(),  // LPCTSTR     lpszClassName;
      NULL                                   // HICON       hIconSm;
   };
   static ATOM atom = ::RegisterClassEx(&wc);
}

CFastMines2WinMain::~CFastMines2WinMain() {
   DelOwnerDrawMenu();
}

// constructor
void CFastMines2WinMain::Create(LPCTSTR szFileName) {

   CreateWindow(
      CFastMines2WinMain::GetWindowClass(),
      CLang::GetString(IDS__LOGO_VERS),
      WS_CAPTION_SHOW,
      ::GetSystemMetrics(SM_CXSCREEN)/2-GetSystemMetrics(SM_CXMIN)/2,
      ::GetSystemMetrics(SM_CYSCREEN)/2-GetSystemMetrics(SM_CYMIN)/2,
      ::GetSystemMetrics(SM_CXMIN),
      ::GetSystemMetrics(SM_CYMIN),
      NULL, (HMENU)0, ::GetModuleHandle(NULL), this);
   /*
   WINDOWPLACEMENT winPlace = {sizeof(WINDOWPLACEMENT), WPF_SETMINPOSITION,
                               SW_SHOW, {10,10}, {0,0}, {0,0,0,0}};
   if (!SetWindowPlacement(m_hWnd, &winPlace)) {
      DWORD errCode = GetLastError();
   }/**/

   m_hMenu = GetMenu(m_hWnd);

   ReloadOwnerDrawMenu(); //      

   if (LoadFile()) {
      CLang::Load(m_Body.m_strLanguage);
   } else {
      PostMessage(m_hWnd, WM_COMMAND, MAKEWPARAM(IDM_HELP_ABOUT, 0), 0L);
   }
   Apply_ShowToolbar();
   Apply_ShowMenu();
   Apply_ShowCaption();
   Apply_AlwaysMaxSize();
   Apply_ToTray();
   Apply_UseAssistant();
   Apply_UseUnknown();

   CheckMenuRadioItem_Mosaic(GetMosaic());
   ::CheckMenuRadioItem(::GetSubMenu(m_hMenu, 0),
      IDM_GAME_BEGINNER, IDM_GAME_CUSTOM,
      IDM_GAME_BEGINNER + m_Body.m_Mosaic.GetSkillLevel(), MF_BYCOMMAND);/**/

   ApplySkin();

   //       
   CSkinMenu::LoadMenuList(HSUBMENU_SKIN); //       

   //        
   CLang::LoadMenuList(HSUBMENU_LANGUAGE); //       
   SetOwnerDrawMenu(HSUBMENU_LANGUAGE);
   CLang::LoadImages(HSUBMENU_LANGUAGE); //        (/) (  SetOwnerDrawMenu(HSUBMENU_LANGUAGE)!!!)
   
   Menu_ChangeLanguage(); // ..     MFT_OWNERDRAW,    -    ReloadOwnerDrawMenu()

   if (m_Body.m_bAutoloadAdmin || (GetPlayers().NumberPlayers()==0)) {
      ::PostMessage(m_hWnd, WM_SELECT_PLAYER, 1/* modal */, 0);
   }

   if (szFileName)
      GameLoad(szFileName);
}

// WM_CREATE
BOOL CFastMines2WinMain::OnCreate(HWND hWnd, LPCREATESTRUCT lpCreateStruct) {
   m_hWnd = hWnd;
   m_Toolbar.Create(m_hWnd);
   m_Body.m_Mosaic.Create(m_hWnd);
   return TRUE;
}

// WM_WINDOWPOSCHANGING
BOOL CFastMines2WinMain::OnWindowPosChanging(HWND, LPWINDOWPOS lpwpos) {
   //LOG_PUT(g_pLog, Put, LL_DEBUG) _T("flags = 0x08X"), lpwpos->flags);
   if ((lpwpos->flags != (SWP_NOOWNERZORDER | SWP_NOACTIVATE | SWP_NOZORDER)) &&
       (lpwpos->flags !=  SWP_NOSIZE))
      return FORWARD_WM_WINDOWPOSCHANGING(m_hWnd, lpwpos, ::DefWindowProc);
   if (!IsWindowEnabled(m_hWnd) || !IsWindowVisible(m_hWnd))
      return FORWARD_WM_WINDOWPOSCHANGING(m_hWnd, lpwpos, ::DefWindowProc);

   const SIZE sizeScreen = ::GetScreenSize();
   const RECT rectMin = {sizeScreen.cx/50, sizeScreen.cy/40, //0,0,           //
                         sizeScreen.cx  -  sizeScreen.cx/50, //sizeScreen.x,  //
                         sizeScreen.cy  -  sizeScreen.cy/40};//sizeScreen.y}; //
   if (lpwpos->x+lpwpos->cx > rectMin.right ) lpwpos->x = sizeScreen.cx-lpwpos->cx;
   if (lpwpos->y+lpwpos->cy > rectMin.bottom) lpwpos->y = sizeScreen.cy-lpwpos->cy;
   if (lpwpos->x < rectMin.left) lpwpos->x = 0;
   if (lpwpos->y < rectMin.top ) lpwpos->y = 0;

   return 0;
}

// WM_WINDOWPOSCHANGED
void CFastMines2WinMain::OnWindowPosChanged(HWND hwnd, const LPWINDOWPOS lpwpos) {
   FORWARD_WM_WINDOWPOSCHANGED(m_hWnd, lpwpos, ::DefWindowProc);
}

// WM_MOVE
void CFastMines2WinMain::OnMove(HWND, int x, int y) {
   if (!IsWindowEnabled(m_hWnd) || !IsWindowVisible(m_hWnd)) return;
   RECTEX rectWin(GetWindowRect(m_hWnd));
   m_Body.m_PointCenter = rectWin.center();//GetScreenSize()/2;//
}

// WM_SIZE
void CFastMines2WinMain::OnSize(HWND, UINT state, int cx, int cy){
   //FORWARD_WM_SIZE(m_hWnd, state, cx, cy, ::DefWindowProc);
   if (!IsWindowEnabled(m_hWnd) || !IsWindowVisible(m_hWnd)) return;
   //if (state != SIZE_RESTORED) return;
   const SIZE sizeMosaic = GetSizeMosaicWindow();
   const SIZE sizeWindow = GetSize(sizeMosaic);
   const SIZE sizeScreen = ::GetScreenSize();
   int iHeightToolbar = m_Body.m_bShowToolbar ? CToolbarFM::GetHeightWindow() : 0;

   {
      RECTEX rectWinNew(GetWindowRect(m_hWnd));
      rectWinNew.size(sizeWindow);
      POINTEX c1(rectWinNew.center());
      rectWinNew.center(m_Body.m_PointCenter);
      POINTEX c2(rectWinNew.center());
      {/**/
         if (rectWinNew.right  > sizeScreen.cx) rectWinNew.alignRight (sizeScreen.cx);
         if (rectWinNew.bottom > sizeScreen.cy) rectWinNew.alignBottom(sizeScreen.cy);
         if (rectWinNew.left   < 0            ) rectWinNew.alignLeft  (0);
         if (rectWinNew.top    < 0            ) rectWinNew.alignTop   (0);
         /**/
      }
      ::MoveWindow(m_hWnd, rectWinNew);
      //::SetWindowPos(m_hWnd, NULL, rectWinNew, SWP_NOZORDER | SWP_NOMOVE);
      //FORWARD_WM_SIZE(m_hWnd, state, cx, cy, ::DefWindowProc);
   }
   ::MoveWindow(m_Body.m_Mosaic.m_hWnd,
       0, iHeightToolbar,
       sizeMosaic.cx,
       sizeMosaic.cy,
      TRUE);
   ::MoveWindow(m_Toolbar.m_hWnd,
      0, 0,
      sizeMosaic.cx,
      iHeightToolbar,
      TRUE);
   ::InvalidateRect(m_Toolbar.m_hWnd, NULL, TRUE);
   {
      LONG style = ::GetWindowLong(m_hWnd, GWL_STYLE);
      if (GetMaximalArea() == m_Body.m_Mosaic.GetArea())
         ::SetWindowLong(m_hWnd, GWL_STYLE, style & (WS_MAXIMIZEBOX^0xFFFFFFFFL));
      else
         ::SetWindowLong(m_hWnd, GWL_STYLE, style | WS_MAXIMIZEBOX);
      //::SendMessage(m_hWnd, WM_NCACTIVATE, (WPARAM)TRUE, 0L);
   }
}

// WM_DESTROY
void CFastMines2WinMain::OnDestroy(HWND){
   SaveFile();
   PostQuitMessage(0);

   m_hWnd = NULL;
}

// WM_SYSCOMMAND
void CFastMines2WinMain::OnSysCommand(HWND, UINT cmd, int x, int y) {
#ifdef _DEBUG
#endif
   switch (cmd) {
   case SC_MAXIMIZE:
 //case 0xF012: // one    click in caption
   case 0xF032: // double click in caption
      FORWARD_WM_COMMAND(m_hWnd, ID_KEY_AREA_MAX, m_hWnd, 2, SendMessage);
      return;
   }
   FORWARD_WM_SYSCOMMAND(m_hWnd, cmd, x, y, ::DefWindowProc);
}

// WM_GETMINMAXINFO
void CFastMines2WinMain::OnGetMinMaxInfo(HWND, LPMINMAXINFO lpMinMaxInfo) {
   if (!IsWindowEnabled(m_hWnd) || !IsWindowVisible(m_hWnd)) return;
 //lpMinMaxInfo->ptMaxSize      = GetSize(GetMaximalArea());
 //lpMinMaxInfo->ptMaxPosition  = POINTEX(0,0);
 //lpMinMaxInfo->ptMinTrackSize = POINTEX(0,0);
 //lpMinMaxInfo->ptMaxTrackSize = POINTEX(0,0);
   FORWARD_WM_GETMINMAXINFO(m_hWnd, lpMinMaxInfo, ::DefWindowProc);
}

BOOL CALLBACK CFastMines2WinMain::CloseDialogs(HWND hWnd, LPARAM lParam) {
   //LOG_PUT(g_pLog, Put, LL_DEBUG) _T("  CloseDialogs: hWnd: text=[%s] class=[%s] style =[%s] styleEx=[%s]"), (LPCTSTR)GetWindowText(hWnd), (LPCTSTR)GetClassName(hWnd), (LPCTSTR)CLogger::WindowStyle(hWnd), (LPCTSTR)CLogger::WindowStyleEx(hWnd));
   if ((::GetWindowStyle(hWnd) & WS_POPUP) && ((HWND)lParam == ::GetParent(hWnd))) {
      //LOG_PUT(g_pLog, Put, LL_DEBUG) _T("  CloseDialogs:  : text=[%s] class=[%s] style =[%s] styleEx=[%s]"), (LPCTSTR)GetWindowText(hWnd), (LPCTSTR)GetClassName(hWnd), (LPCTSTR)CLogger::WindowStyle(hWnd), (LPCTSTR)CLogger::WindowStyleEx(hWnd));
      ::SendMessage(hWnd, WM_CLOSE  , 0,0);
      ::SendMessage(hWnd, WM_DESTROY, 0,0);
   }
   return TRUE;
}

BOOL CALLBACK CFastMines2WinMain::InvalidateDialogs(HWND hWnd, LPARAM lParam) {
   if ((::GetWindowStyle(hWnd) & WS_POPUP) && ((HWND)lParam == ::GetParent(hWnd))) {
      ::InvalidateRect(hWnd, NULL, TRUE);
      //::SendMessage(hWnd, WM_DESTROY, 0,0);
   }
   return TRUE;
}

// WM_COMMAND
void CFastMines2WinMain::OnCommand(HWND, int id, HWND hwndCtl, UINT codeNotify){
   //LOG_PUT(g_pLog, Put, LL_DEBUG) _T("OnCommand: id=%d; hwndCtl=0x%08X (class[%s]); codeNotify=%d"), id, hwndCtl, (LPCTSTR)GetClassName(hwndCtl), codeNotify);
   switch (id){
   // menu File
   case IDA_NEW_GAME:
   case IDM_GAME_NEW_GAME:
      m_Body.m_Mosaic.GameNew();
      return;
   case IDM_GAME_BEGINNER:
      ::CheckMenuRadioItem( ::GetSubMenu(m_hMenu, 0),
         IDM_GAME_BEGINNER, IDM_GAME_CUSTOM,
         IDM_GAME_BEGINNER + eSkillLevelBeginner,
         MF_BYCOMMAND);
      m_Body.m_Mosaic.SetSkillLevel(eSkillLevelBeginner);
      return;
   case IDM_GAME_AMATEUR:
      ::CheckMenuRadioItem( ::GetSubMenu(m_hMenu, 0),
         IDM_GAME_BEGINNER, IDM_GAME_CUSTOM,
         IDM_GAME_BEGINNER + eSkillLevelAmateur,
         MF_BYCOMMAND);
      m_Body.m_Mosaic.SetSkillLevel(eSkillLevelAmateur);
      return;
   case IDM_GAME_PROFESSIONAL:
      ::CheckMenuRadioItem( ::GetSubMenu(m_hMenu, 0),
         IDM_GAME_BEGINNER, IDM_GAME_CUSTOM,
         IDM_GAME_BEGINNER + eSkillLevelProfessional,
         MF_BYCOMMAND);
      m_Body.m_Mosaic.SetSkillLevel(eSkillLevelProfessional);
      return;
   case IDM_GAME_CRAZY:
      ::CheckMenuRadioItem( ::GetSubMenu(m_hMenu, 0),
         IDM_GAME_BEGINNER, IDM_GAME_CUSTOM,
         IDM_GAME_BEGINNER + eSkillLevelCrazy,
         MF_BYCOMMAND);
      m_Body.m_Mosaic.SetSkillLevel(eSkillLevelCrazy);
      return;
   case IDM_GAME_CUSTOM:
      {
         static CDialogCustomSkill dlg;
         if (dlg.m_hWnd == NULL) {
            dlg.Create(m_hWnd);
         } else {
            ::DestroyWindow(dlg.m_hWnd);
         }
      }
      return;
   case IDM_GAME_CREATE:
      m_Body.m_Mosaic.GameCreate();
      return;
   case IDM_GAME_SAVE:
      GameSave();
      return;
   case IDM_GAME_LOAD:
      GameLoad();
      return;
   case IDM_GAME_PLAYERS:
      ::SendMessage(m_hWnd, WM_SELECT_PLAYER, 0/* modless */, 0);
      return;
   case IDM_GAME_EXIT:
      ::EnumWindows(CFastMines2WinMain::CloseDialogs, (LPARAM)m_hWnd);
      //::EnumChildWindows(m_hWnd, CloseDialogs, (LPARAM)m_hWnd);
      ::SendMessage(m_hWnd, WM_DESTROY, 0L, 0L);
      return;
   // menu m_Body.m_Mosaic
   case IDM_MOSAIC_TRIANGLE1:
   case IDM_MOSAIC_TRIANGLE2:
   case IDM_MOSAIC_TRIANGLE3:
   case IDM_MOSAIC_TRIANGLE4:
   case IDM_MOSAIC_SQUARE1:
   case IDM_MOSAIC_SQUARE2:
   case IDM_MOSAIC_PARQUET1:
   case IDM_MOSAIC_PARQUET2:
   case IDM_MOSAIC_TRAPEZOID1:
   case IDM_MOSAIC_TRAPEZOID2:
   case IDM_MOSAIC_TRAPEZOID3:
   case IDM_MOSAIC_RHOMBUS1:
   case IDM_MOSAIC_QUADRANGLE1:
   case IDM_MOSAIC_PENTAGONT24:
   case IDM_MOSAIC_PENTAGONT5:
   case IDM_MOSAIC_PENTAGONT10:
   case IDM_MOSAIC_HEXAGON1:
   case IDM_MOSAIC_TRSQ1:
   case IDM_MOSAIC_TRSQ2:
   case IDM_MOSAIC_SQTRHEX:
      SetMosaic(EMosaic(id-IDM_MOSAIC_TRIANGLE1));
      return;
   // menu Options
   case IDM_OPTIONS_ALWAYSMAXSIZE:
      m_Body.m_bAlwaysMaxSize = !m_Body.m_bAlwaysMaxSize;
      Apply_AlwaysMaxSize();
      return;
   case IDM_OPTIONS_USE_UNKNOWN:
      m_Body.m_Mosaic.SetUseUnknown(!m_Body.m_Mosaic.GetUseUnknown());
      Apply_UseUnknown();
      return;
   case IDM_OPTIONS_SHOW_TOOLBAR:
      m_Body.m_bShowToolbar = !m_Body.m_bShowToolbar;
      Apply_ShowToolbar();
      return;
   case IDM_OPTIONS_SHOW_MENU:
      m_Body.m_bShowMenu = !m_Body.m_bShowMenu;
      Apply_ShowMenu();
      return;
   case IDM_OPTIONS_SHOW_CAPTION:
      m_Body.m_bShowCaption = !m_Body.m_bShowCaption;
      Apply_ShowCaption();
      return;
   case IDM_OPTIONS_TO_TRAY:
      m_Body.m_bToTray = !m_Body.m_bToTray;
      Apply_ToTray();
      return;
   case IDM_OPTIONS_SKIN_CHANGE:
      {
         static CDialogChangeSkin dlg;
         if (dlg.m_hWnd == NULL) {
            dlg.Create(m_hWnd);
         } else {
            ::DestroyWindow(dlg.m_hWnd);
         }
      }
      return;
   case IDM_OPTIONS_SKIN_WEB_SKIN_PAGE:
      ::ShellExecute(m_hWnd, _T("open"), CLang::GetString(IDS__WEB_PAGE__FASTMINES_SKIN), NULL, NULL, SW_RESTORE);
      return;
   case IDM_OPTIONS_SKIN_SAVE:
      {
         static CDialogInputText dlg(CDialogInputText::eSkinName);
         if (dlg.m_hWnd == NULL) {
            dlg.Create(m_hWnd);
         } else {
            ::DestroyWindow(dlg.m_hWnd);
         }
      }
      return;
   case IDM_OPTIONS_SKIN_LOAD:
      //FORWARD_WM_SYSKEYDOWN(m_hWnd, VK_MENU, 0x0001, 0x2038, SendMessage);
      //FORWARD_WM_SYSKEYDOWN(m_hWnd, 0x4F   , 0x0001, 0x2018, SendMessage);
      //FORWARD_WM_SYSKEYUP  (m_hWnd, VK_MENU, 0,0, SendMessage);
      //FORWARD_WM_SYSKEYUP  (m_hWnd, 0x4F   , 0,0, SendMessage);
      //FORWARD_WM_SYSCHAR   (m_hWnd, 0x6F   , 0x20180001,   SendMessage);
      //FORWARD_WM_SYSKEYUP  (m_hWnd, VK_MENU, 0,0, SendMessage);
      return;
   case IDM_OPTIONS_SKIN_LOAD0:
      SetSkin(CSkin());
      DrawMenuBar(m_hWnd);
      return;
   case IDM_OPTIONS_LANG_LOAD0:
      CLang::RestoreDefaultLanguage();
      m_Body.m_strLanguage = _T("English.lng");
      CLang::Load(m_Body.m_strLanguage);
      Menu_ChangeLanguage();
      /** /
      ::CheckMenuRadioItem(HSUBMENU_LANGUAGE,
         IDM_OPTIONS_LANG_LOAD0, IDM_OPTIONS_LANG_LOAD0+::GetMenuItemCount(HSUBMENU_LANGUAGE),
         IDM_OPTIONS_LANG_LOAD0,
         MF_BYCOMMAND);
      /**/
      return;
   default:
      if (codeNotify == 0) { //  from menu
         if (id > IDM_OPTIONS_LANG_LOAD0) {
            m_Body.m_strLanguage = ::GetMenuString(HSUBMENU_LANGUAGE, id, MF_BYCOMMAND) + _T(".lng");
            CLang::Load(m_Body.m_strLanguage);
            Menu_ChangeLanguage();
            /** /
            ::CheckMenuRadioItem(HSUBMENU_LANGUAGE,
               IDM_OPTIONS_LANG_LOAD0, IDM_OPTIONS_LANG_LOAD0+::GetMenuItemCount(HSUBMENU_LANGUAGE),
               id,
               MF_BYCOMMAND);
            /**/
         } else
         if (id > IDM_OPTIONS_SKIN_LOAD0) {
            SetSkin(CSkinMenu::LoadSkin(HSUBMENU_SKIN, id));
            DrawMenuBar(m_hWnd);
            ::EnumWindows(CFastMines2WinMain::InvalidateDialogs, (LPARAM)m_hWnd); //    
         }
      }
      return;
   // menu Help
   case IDM_HELP_CHAMPIONS:
      {
         static CDialogChampions dlg;
         if (dlg.m_hWnd == NULL) {
            dlg.Create(m_hWnd);
         } else {
            ::DestroyWindow(dlg.m_hWnd);
         }
      }
      return;
   case IDM_HELP_STATISTICS:
      {
         static CDialogStatistics dlg;
         if (dlg.m_hWnd == NULL) {
            dlg.Create(m_hWnd);
         } else {
            ::DestroyWindow(dlg.m_hWnd);
         }
      }
      return;
   case IDM_HELP_ASSISTANT_ONOFF:
      {
         CAssistantInfo AssistantInfo = GetAssistantInfo();
         AssistantInfo.m_bUse = !AssistantInfo.m_bUse;
         SetAssistantInfo(AssistantInfo);
         Apply_UseAssistant();
         m_Toolbar.SetImagePause(GetImageBtnPause(0), CButtonImage::_nImageNormal);
      }
      return;
   case IDM_HELP_ASSISTANT_SUGGEST:
      m_Body.m_Mosaic.AssistantSuggest();
      return;
   case IDM_HELP_ASSISTANT_OPTIONS:
      {
         static CDialogAssistant dlg;
         if (dlg.m_hWnd == NULL) {
            dlg.Create(m_hWnd);
         } else {
            ::DestroyWindow(dlg.m_hWnd);
         }
      }
      return;
   case IDM_HELP_ABOUT:
      {
         static CDialogAbout dlg;
         if (dlg.m_hWnd == NULL) {
            dlg.Create(m_hWnd);
         } else {
            ::DestroyWindow(dlg.m_hWnd);
         }
      }
      return;
   // Accelerators
   case IDA_MINIMIZE:
      m_Body.m_Mosaic.SetPause(true);
      ::ShowWindow(m_hWnd, SW_MINIMIZE);
      return;
   case IDA_PAUSE:
      m_Body.m_Mosaic.SetPause(!m_Body.m_Mosaic.GetPause());
      return;
   case ID_KEY_AREA_MAX:
   case ID_KEY_AREA_MIN:
   case ID_KEY_AREA_INCREMENT:
   case ID_KEY_AREA_DECREMENT:
      if (!m_Body.m_bAlwaysMaxSize) {
         bool noChanged;
         switch (id){
         case ID_KEY_AREA_MAX:       noChanged = AreaMax();       break;
         case ID_KEY_AREA_MIN:       noChanged = AreaMin();       break;
         case ID_KEY_AREA_INCREMENT: noChanged = AreaIncrement(); break;
         case ID_KEY_AREA_DECREMENT: noChanged = AreaDecrement(); break;
         }
//         if (noChanged) BeepSpeaker();
      }
      return;
 case ID_KEY_NUM0:
/**
      {
         RECTEX wndRect; ::GetClientRect(m_hWnd, &wndRect);
         RECTEX fillRect(wndRect.left+10, wndRect.top+10, wndRect.right-10, wndRect.bottom-10);
         nsEraseBk::FillWnd(m_hWnd, 0, false, fillRect);
         HDC hDC = ::GetDC(m_hWnd);
         //CFastMines2WinMain::GetImageMosaic((EMosaic)0)->Draw(hDC, &fillRect);
         BOOL res = DrawIconEx(hDC,
            fillRect.left, fillRect.top,
            CFastMines2WinMain::GetIconMosaic((EMosaic)0),
            fillRect.width(),
            fillRect.height(),
            0,
            NULL,
            DI_NORMAL);
         if (!res) {
            DWORD errCode = GetLastError();
         }
         ReleaseDC(m_hWnd, hDC);
      }
/**
       {
          LOG_PUT(g_pLog, Put, LL_DEBUG) _T("SM_CXHSCROLL = %i"), ::GetSystemMetrics(SM_CXHSCROLL));
          LOG_PUT(g_pLog, Put, LL_DEBUG) _T("SM_CYHSCROLL = %i"), ::GetSystemMetrics(SM_CYHSCROLL));
          LOG_PUT(g_pLog, Put, LL_DEBUG) _T("SM_CXHTHUMB  = %i"), ::GetSystemMetrics(SM_CXHTHUMB ));
          LOG_PUT(g_pLog, Put, LL_DEBUG) _T("SM_CXVSCROLL = %i"), ::GetSystemMetrics(SM_CXVSCROLL));
          LOG_PUT(g_pLog, Put, LL_DEBUG) _T("SM_CYVSCROLL = %i"), ::GetSystemMetrics(SM_CYVSCROLL));
          LOG_PUT(g_pLog, Put, LL_DEBUG) _T("SM_CYVTHUMB  = %i"), ::GetSystemMetrics(SM_CYVTHUMB ));
        }
/**/
       break;
 //case ID_KEY_NUM1:
 //case ID_KEY_NUM2:
   case ID_KEY_NUM3:
   case ID_KEY_NUM4:
   case ID_KEY_NUM5:
   case ID_KEY_NUM6:
   case ID_KEY_NUM7:
 //case ID_KEY_NUM8:
 //case ID_KEY_NUM9:
      {
         static CDialogSelectMosaic dlg;
         if (dlg.m_hWnd) {
            ::DestroyWindow(dlg.m_hWnd);
         }
         dlg.SetFirstKey(id-ID_KEY_NUM0);
         dlg.Create(m_hWnd);
      }
      return;
   }
}

// WM_SELECT_PLAYER
void CFastMines2WinMain::OnSelectPlayer(HWND hwnd, BOOL bModal) {
   if (bModal) {
      CDialogAdministration().DoModal(m_hWnd);
   } else {
      static CDialogAdministration dlg;
      if (dlg.m_hWnd == NULL) {
         dlg.Create(m_hWnd);
      } else {
         ::DestroyWindow(dlg.m_hWnd);
      }
   }
}

// WM_ACTIVATE
void CFastMines2WinMain::OnActivate(HWND, UINT state, HWND hWndActDeact, BOOL fMinimized){
   FORWARD_WM_ACTIVATE(m_hWnd, state, hWndActDeact, fMinimized, ::DefWindowProc);
#ifndef _DEBUG
   if (state == WA_INACTIVE) {
      ::SetWindowPos(m_hWnd, HWND_BOTTOM , 0,0,0,0, SWP_NOSIZE | SWP_NOMOVE);
   } else {
      ::SetWindowPos(m_hWnd, HWND_TOPMOST, 0,0,0,0, SWP_NOSIZE | SWP_NOMOVE);
   }
#endif
   if ((state == WA_INACTIVE) ||
       (fMinimized != false))
      m_Body.m_Mosaic.SetPause(true);
   if ((fMinimized != false) && m_Body.m_bToTray) {
      TrayMessage(NIM_ADD, CLang::GetString(IDS__LOGO_VERS));
   }
}

// WM_NOTIFYICON
void CFastMines2WinMain::OnNotifyIcon(HWND, UINT notifyMsg) {
   switch (notifyMsg) {
   case WM_LBUTTONDOWN:
   case WM_RBUTTONDOWN:
      TrayMessage(NIM_DELETE, NULL);
   }
}

// WM_MEASUREITEM
void CFastMines2WinMain::OnMeasureItem(HWND, MEASUREITEMSTRUCT *lpMeasureItem) {
   if (lpMeasureItem) {
      if (lpMeasureItem->CtlType == ODT_MENU) {
         CMenuItem::OnMeasureItem(lpMeasureItem);
      }
   }
}

// WM_DRAWITEM
void CFastMines2WinMain::OnDrawItem(HWND, const DRAWITEMSTRUCT *lpDrawItem) {
   if (lpDrawItem) {
      if (lpDrawItem->CtlType == ODT_MENU) {
         CMenuItem::OnDrawItem(lpDrawItem);
      }
   }
}

// WM_MOSAIC_ADJUSTAREA
void CFastMines2WinMain::OnMosaicAdjustArea(HWND) {
   int maxArea = GetMaximalArea();
   if (m_Body.m_Mosaic.GetArea() > maxArea)
      m_Body.m_Mosaic.SetArea(maxArea);
}

// WM_MOSAIC_CLICK
void CFastMines2WinMain::OnMosaicClick(HWND, BOOL leftClick, BOOL down) {
   m_Toolbar.SetImagePause(GetImageBtnPause(0), CButtonImage::_nImageNormal);
   if (leftClick) {
      if (down) {
         m_Toolbar.SetImageNew(GetImageBtnNew(1), CButtonImage::_nImageNormal);
      } else {
         m_Toolbar.SetImageNew(GetImageBtnNew(0), CButtonImage::_nImageNormal);
      }
   } else {
   }
}

// WM_MOSAIC_CHANGECOUNTERS
void CFastMines2WinMain::OnMosaicChangeCounters(HWND) {
   CString strCaption;
   if (m_Body.m_bShowToolbar) {
      strCaption.Format(_T("%i"), m_Body.m_Mosaic.GetCountTimer());
      m_Toolbar.SetTextTimer(strCaption);
      strCaption.Format(_T("%i"), m_Body.m_Mosaic.GetCountMines());
      m_Toolbar.SetTextMines(strCaption);
      strCaption = CLang::GetString(IDS__LOGO_VERS);
   } else {
      strCaption.Format(
         _T("%s  -  %s[%i]  %s[%i %s]"),
         (LPCTSTR)CLang::GetString(IDS__LOGO_VERS),
         (LPCTSTR)CLang::GetString(IDS__MINES    ), m_Body.m_Mosaic.GetCountMines(),
         (LPCTSTR)CLang::GetString(IDS__TIME     ), m_Body.m_Mosaic.GetCountTimer(),
         (LPCTSTR)CLang::GetString(IDS__SEC      )
      );
   }
   if (m_Body.m_Mosaic.GetGameStatus() == eGSEnd) {
      if (m_Body.m_Mosaic.GetVictory())
         strCaption += _T(" - ")+CLang::GetString(IDS__VICTORY);
      else
         strCaption += _T(" - ")+CLang::GetString(IDS__DEFEAT);
   }
   SetWindowText(m_hWnd, strCaption);

   //if (m_Body.m_Mosaic.GetGameStatusIsCreate())
   {
      ::CheckMenuRadioItem(::GetSubMenu(m_hMenu, 0),
         IDM_GAME_BEGINNER, IDM_GAME_CUSTOM,
         IDM_GAME_BEGINNER + GetSkillLevel(),
         MF_BYCOMMAND);
   }
}

// WM_MOSAIC_CHANGEGAMESTATUS
void CFastMines2WinMain::OnMosaicChangeGameStatus(HWND, EGameStatus eGS) {
   switch (eGS) {
   case eGSReady:
      m_Toolbar.SetImageNew(GetImageBtnNew(0), CButtonImage::_nImageNormal);
      break;
   case eGSEnd:
      if (m_Body.m_Mosaic.GetVictory()) {
         m_Toolbar.SetImageNew(GetImageBtnNew(2), CButtonImage::_nImageNormal);
      } else {
         m_Toolbar.SetImageNew(GetImageBtnNew(3), CButtonImage::_nImageNormal);
      }
      break;
   case -1:
      m_Toolbar.SetImagePause(GetImageBtnPause(3), CButtonImage::_nImageNormal);
      break;
   }
   switch (eGS) {
   case eGSReady:
      // game create
      if (m_Body.m_bAlwaysMaxSize) AreaMax();
      ::EnableMenuItem(::GetSubMenu(m_hMenu, 0), IDM_GAME_SAVE, MF_BYCOMMAND | MF_GRAYED);
      break;
   case eGSPlay:
      // game begin
      ::EnableMenuItem(::GetSubMenu(m_hMenu, 0), IDM_GAME_SAVE, MF_BYCOMMAND | MF_ENABLED);
      break;
   case eGSEnd:
      // game end
      if (m_Body.m_Mosaic.GetVictory()) {
         if (!m_Body.m_Mosaic.GetGameIsIgnorable() && m_Body.m_Mosaic.GetPlayerIsUser() && !m_Body.m_Mosaic.GetPlayerIsAssistant())
            CDialogChampions::SaveResult(GetMosaic(), GetSkillLevel(), m_Body.m_Mosaic.GetCountTimer(), m_Body.m_strPlayerName);
      }
      if ((m_Body.m_Mosaic.GetSkillLevel() != eSkillLevelCustom) &&
          !m_Body.m_Mosaic.GetGameIsIgnorable() &&
          (m_Body.m_Mosaic.GetPlayerIsUser () ||
           m_Body.m_Mosaic.GetPlayerIsAssistant()))
      {
         bool bVictory = m_Body.m_Mosaic.GetVictory();
         DWORD dwOpenField  = bVictory ? GetSizeMosaic().cx*GetSizeMosaic().cy-DefineNumberMines() : m_Body.m_Mosaic.GetCountOpen();
         DWORD dwPlayTime   = bVictory ? m_Body.m_Mosaic.GetCountTimer() : 0;
         DWORD dwClickCount = bVictory ? m_Body.m_Mosaic.GetCountClick() : 0;
         if (m_Body.m_Mosaic.GetPlayerIsUser() && !m_Body.m_Mosaic.GetPlayerIsAssistant()) //    
            GetPlayers().ModifyData(m_Body.m_strPlayerName                      , GetMosaic(), m_Body.m_Mosaic.GetSkillLevel(), bVictory, dwOpenField, dwPlayTime, dwClickCount);
         if (m_Body.m_Mosaic.GetPlayerIsAssistant() && !m_Body.m_Mosaic.GetPlayerIsUser()) //    
            GetPlayers().ModifyData(CDialogAdministration::DEFAULT_PLAYER_NAME(), GetMosaic(), m_Body.m_Mosaic.GetSkillLevel(), bVictory, dwOpenField, dwPlayTime, dwClickCount);
      }
      break;
   }
}

// WM_MOSAIC_PAUSE
void CFastMines2WinMain::OnMosaicPause(HWND) {
   if (m_Body.m_Mosaic.GetGameStatus() != eGSPlay) {
      m_Toolbar.BtnPauseCheck(FALSE);
      m_Toolbar.BtnPauseEnable(FALSE);
   } else {
      m_Toolbar.BtnPauseCheck(m_Body.m_Mosaic.GetPause());
      m_Toolbar.BtnPauseEnable(TRUE);
   }
}

#ifndef WHEEL_DELTA
   #define WHEEL_DELTA       120       /* Value for rolling one detent */
#endif

// WM_MOUSEWHEEL
void CFastMines2WinMain::OnMouseWheel(HWND hWnd, int xPos, int yPos, int zDelta, WORD fwKeys) {
   //LOG_PUT(g_pLog, Put, LL_DEBUG) _T("OnMouseWheel: fwKeys=0x%04X, zDelta=%d, xPos=%d, yPos=%d"),  fwKeys, zDelta, xPos, yPos);
   //const int k = abs(zDelta)/WHEEL_DELTA;
   //for (int i=0; i<k; i++)
   {
      FORWARD_WM_COMMAND(m_hWnd, (zDelta > 0) ? ID_KEY_AREA_INCREMENT : ID_KEY_AREA_DECREMENT, NULL, 0, PostMessage);
   }
}

// WM_ERASEBKGND
BOOL CFastMines2WinMain::OnEraseBkgnd(HWND, HDC hDC) {
   if (!m_Body.m_Skin.m_bToAll)
      return FALSE; // FORWARD_WM_ERASEBKGND(m_hWnd, hDC, ::DefWindowProc);
   return nsEraseBk::OnEraseBkgnd(m_hWnd, hDC, m_Body.m_Skin.GetBkColor());
}

////////////////////////////////////////////////////////////////////////////////
//                             other function
////////////////////////////////////////////////////////////////////////////////
void CFastMines2WinMain::TrayMessage(UINT msg, const TCHAR *pTip) {
   NOTIFYICONDATA nid = {
      sizeof(NOTIFYICONDATA),          //DWORD cbSize
      m_hWnd,                          //HWND  hWnd
      0,                               //UINT  uID
      NIF_ICON | NIF_MESSAGE | NIF_TIP,//UINT  uFlags
      WM_NOTIFYICON,                   //UINT  uCallbackMessage
      m_hIconProject,                  //HICON hIcon
      0                                //TCHAR szTip[64]
   };
   memcpy(
      nid.szTip,
      pTip ? pTip : CLang::GetString(IDS__LOGO_VERS),
      min(63<<(sizeof(TCHAR)-1), lstrlen(pTip ? pTip : CLang::GetString(IDS__LOGO_VERS)))
   );
   Shell_NotifyIcon(msg, &nid);
   if (msg == NIM_ADD)
      ::ShowWindow(m_hWnd, SW_HIDE);
   if (msg == NIM_DELETE) {
      ::ShowWindow(m_hWnd, SW_RESTORE);
      //FORWARD_WM_ACTIVATE(m_hWnd, WA_ACTIVE, NULL, FALSE, PostMessage);
      //FORWARD_WM_SETFOCUS(m_hWnd, NULL, PostMessage);
      //ShowWindow(m_hWnd, SW_SHOW);
      //EnableWindow(m_hWnd, TRUE);
      //SetFocus(m_hWnd);
      //SetActiveWindow(m_hWnd);
   }
}

SIZE CFastMines2WinMain::GetSize(const SIZE &sizeMosaicInPixel) const {
   //    
   //     
   RECTEX windowRect, clientRect;
   ::GetWindowRect(m_hWnd, &windowRect);
   ::GetClientRect(m_hWnd, &clientRect);
   SIZE result = {
      (windowRect.width()) -
      (clientRect.width()) + sizeMosaicInPixel.cx,
      (windowRect.height()) -
      (clientRect.height()) + sizeMosaicInPixel.cy + (m_Body.m_bShowToolbar ? CToolbarFM::GetHeightWindow() : 0)};
   return result;
}

int CFastMines2WinMain::GetMaximalArea() const {
   //      
   //         .
   int result;
   const SIZE sizeScreen = ::GetScreenSize();
   result = CMosaic::AreaMinimum();
   result++;
   SIZE sizeWnd = GetSize(result);
   while ((sizeWnd.cx <= sizeScreen.cx) &&
          (sizeWnd.cy <= sizeScreen.cy)) {
      result++;
      sizeWnd = GetSize(result);
   }
   result--;
   return result;
}

bool CFastMines2WinMain::AreaIncrement() {
   //  true   
   int maxArea = GetMaximalArea();
   if (maxArea == m_Body.m_Mosaic.GetArea()) return true;
   int newArea = min(maxArea, toINT(toFLOAT(m_Body.m_Mosaic.GetArea())*105.0f/100));
   m_Body.m_Mosaic.SetArea(newArea);
   return false;
}

bool CFastMines2WinMain::AreaDecrement() {
   //  true   
   if (m_Body.m_Mosaic.GetArea() == CMosaic::AreaMinimum()) return true;
   int newArea = max(CMosaic::AreaMinimum(), toINT(toFLOAT(m_Body.m_Mosaic.GetArea())*95.0f/100));
   m_Body.m_Mosaic.SetArea(newArea);
   return false;
}

bool CFastMines2WinMain::AreaMax() {
   if (!IsWindowEnabled(m_hWnd) || !IsWindowVisible(m_hWnd)) return (true || true); //   ...
   //  true   
   int maxArea = GetMaximalArea();
   if (maxArea == m_Body.m_Mosaic.GetArea()) return true;
   m_Body.m_Mosaic.SetArea(maxArea);
   {
      //    AreaMax()      ,  
      //   -     , ..,  
      // GetMaximalArea()     .
      //        .
      if (maxArea < GetMaximalArea())
         AreaMax(); //      
      else;         //     
   }
   return false;
}

bool CFastMines2WinMain::AreaMin() {
   //  true   
   if (m_Body.m_Mosaic.GetArea() == CMosaic::AreaMinimum()) return true;
   m_Body.m_Mosaic.SetArea(CMosaic::AreaMinimum());
   return false;
}

void CFastMines2WinMain::CheckMenuRadioItem_Mosaic(EMosaic eMosaic) const {
   BOOL bRes = ::CheckMenuRadioItem(::GetSubMenu(m_hMenu, 1), 0, 4,
      CDialogSelectMosaic::GetGroup(eMosaic), MF_BYPOSITION);
   bRes = ::CheckMenuRadioItem(::GetSubMenu(m_hMenu, 1),
      IDM_MOSAIC_TRIANGLE1, IDM_MOSAIC_TRIANGLE4,
      IDM_MOSAIC_TRIANGLE1 + eMosaic, MF_BYCOMMAND);
   bRes = ::CheckMenuRadioItem(::GetSubMenu(m_hMenu, 1),
      IDM_MOSAIC_SQUARE1, IDM_MOSAIC_QUADRANGLE1,
      IDM_MOSAIC_TRIANGLE1 + eMosaic, MF_BYCOMMAND);
   bRes = ::CheckMenuRadioItem(::GetSubMenu(m_hMenu, 1),
      IDM_MOSAIC_PENTAGONT24, IDM_MOSAIC_PENTAGONT10,
      IDM_MOSAIC_TRIANGLE1 + eMosaic, MF_BYCOMMAND);
   bRes = ::CheckMenuRadioItem(::GetSubMenu(m_hMenu, 1),
      IDM_MOSAIC_HEXAGON1, IDM_MOSAIC_HEXAGON1,
      IDM_MOSAIC_TRIANGLE1 + eMosaic, MF_BYCOMMAND);
   bRes = ::CheckMenuRadioItem(::GetSubMenu(m_hMenu, 1),
      IDM_MOSAIC_TRSQ1, IDM_MOSAIC_SQTRHEX,
      IDM_MOSAIC_TRIANGLE1 + eMosaic, MF_BYCOMMAND);
}

void CFastMines2WinMain::Apply_AlwaysMaxSize() {
   CheckMenuItem(::GetSubMenu(m_hMenu, 2), IDM_OPTIONS_ALWAYSMAXSIZE,
      m_Body.m_bAlwaysMaxSize ? MF_CHECKED : MF_UNCHECKED);
   if (m_Body.m_bAlwaysMaxSize) AreaMax();
   ::EnableMenuItem(::GetSubMenu(m_hMenu, 2), ID_KEY_AREA_INCREMENT, m_Body.m_bAlwaysMaxSize ? MF_GRAYED : MF_ENABLED);//MF_DISABLED
   ::EnableMenuItem(::GetSubMenu(m_hMenu, 2), ID_KEY_AREA_DECREMENT, m_Body.m_bAlwaysMaxSize ? MF_GRAYED : MF_ENABLED);//MF_DISABLED
   ::EnableMenuItem(::GetSubMenu(m_hMenu, 2), ID_KEY_AREA_MAX      , m_Body.m_bAlwaysMaxSize ? MF_GRAYED : MF_ENABLED);//MF_DISABLED
   ::EnableMenuItem(::GetSubMenu(m_hMenu, 2), ID_KEY_AREA_MIN      , m_Body.m_bAlwaysMaxSize ? MF_GRAYED : MF_ENABLED);//MF_DISABLED
}

void CFastMines2WinMain::Apply_ToTray() {
   CheckMenuItem(::GetSubMenu(m_hMenu, 2), IDM_OPTIONS_TO_TRAY,
      m_Body.m_bToTray ? MF_CHECKED : MF_UNCHECKED);
}

void CFastMines2WinMain::Apply_ShowToolbar() {
   CheckMenuItem(::GetSubMenu(m_hMenu, 2), IDM_OPTIONS_SHOW_TOOLBAR,
      m_Body.m_bShowToolbar ? MF_CHECKED : MF_UNCHECKED);
   OnMosaicChangeCounters(m_hWnd);
   if (m_Body.m_bAlwaysMaxSize) AreaMax();
   else if (m_Body.m_bShowToolbar) OnMosaicAdjustArea(m_hWnd);
   FORWARD_WM_SIZE(m_hWnd, SIZE_RESTORED, 0,0, SendMessage);
}

void CFastMines2WinMain::Apply_ShowMenu() {
   CheckMenuItem(::GetSubMenu(m_hMenu, 2), IDM_OPTIONS_SHOW_MENU,
      m_Body.m_bShowMenu ? MF_CHECKED : MF_UNCHECKED);
   SetMenu(m_hWnd, m_Body.m_bShowMenu ? m_hMenu : NULL);
   DrawMenuBar(m_hWnd);
   if (m_Body.m_bAlwaysMaxSize) AreaMax();
   else if (m_Body.m_bShowMenu) OnMosaicAdjustArea(m_hWnd);
}

void CFastMines2WinMain::Apply_ShowCaption() {
   CheckMenuItem(::GetSubMenu(m_hMenu, 2), IDM_OPTIONS_SHOW_CAPTION,
      m_Body.m_bShowCaption ? MF_CHECKED : MF_UNCHECKED);
   ::SetWindowLong(m_hWnd, GWL_STYLE, m_Body.m_bShowCaption ? WS_CAPTION_SHOW : WS_CAPTION_HIDE);
   ::ShowWindow(m_hWnd, SW_SHOW);
   DrawMenuBar(m_hWnd);
   if (m_Body.m_bAlwaysMaxSize) AreaMax();
   else if (m_Body.m_bShowCaption) {
      OnMosaicAdjustArea(m_hWnd);
   }
   FORWARD_WM_SIZE(m_hWnd, SIZE_RESTORED, 0,0, SendMessage);
}

void CFastMines2WinMain::Apply_UseAssistant() {
   CheckMenuItem(HSUBMENU_ASSISTANT, IDM_HELP_ASSISTANT_ONOFF,
      GetAssistantInfo().m_bUse ? MF_CHECKED : MF_UNCHECKED);
}

void CFastMines2WinMain::Apply_UseUnknown() {
   CheckMenuItem( ::GetSubMenu(m_hMenu, 2), IDM_OPTIONS_USE_UNKNOWN,
      m_Body.m_Mosaic.GetUseUnknown() ? MF_CHECKED : MF_UNCHECKED);
}

void CFastMines2WinMain::ApplySkin() {
 //SetCursor(LoadCursor(NULL, IDC_WAIT));

   /** /
   if (!CImage(m_Body.m_Skin.m_ImgMine   ).Valid()) m_Body.m_Skin.m_ImgMine    = CSkin().m_ImgMine   ;
   if (!CImage(m_Body.m_Skin.m_ImgFlag   ).Valid()) m_Body.m_Skin.m_ImgFlag    = CSkin().m_ImgFlag   ;
   if (!CImage(m_Body.m_Skin.m_ImgPause  ).Valid()) m_Body.m_Skin.m_ImgPause   = CSkin().m_ImgPause  ;
   if (!CImage(m_Body.m_Skin.m_ImgBckgrnd).Valid()) m_Body.m_Skin.m_ImgBckgrnd = CSkin().m_ImgBckgrnd;
   /**/

   m_Body.m_Mosaic.ApplySkin(m_Body.m_Skin);

   /** /
   if (!CImage(GetImageBtnNew  (0)).Valid()) m_Body.m_Skin.m_ImgBtnNew  [0] = CSkin().m_ImgBtnNew  [0];
   if (!CImage(GetImageBtnNew  (1)).Valid()) m_Body.m_Skin.m_ImgBtnNew  [1] = CSkin().m_ImgBtnNew  [1];
   if (!CImage(GetImageBtnNew  (2)).Valid()) m_Body.m_Skin.m_ImgBtnNew  [2] = CSkin().m_ImgBtnNew  [2];
   if (!CImage(GetImageBtnNew  (3)).Valid()) m_Body.m_Skin.m_ImgBtnNew  [3] = CSkin().m_ImgBtnNew  [3];
   if (!CImage(GetImageBtnPause(0)).Valid()) m_Body.m_Skin.m_ImgBtnPause[0] = CSkin().m_ImgBtnPause[0];
   if (!CImage(GetImageBtnPause(1)).Valid()) m_Body.m_Skin.m_ImgBtnPause[1] = CSkin().m_ImgBtnPause[1];
   if (!CImage(GetImageBtnPause(2)).Valid()) m_Body.m_Skin.m_ImgBtnPause[2] = CSkin().m_ImgBtnPause[2];
   if (!CImage(GetImageBtnPause(3)).Valid()) m_Body.m_Skin.m_ImgBtnPause[3] = CSkin().m_ImgBtnPause[3];
   /**/
   m_Toolbar.SetImageNew  (GetImageBtnNew  (0), CButtonImage::_nImageNormal);
   m_Toolbar.SetImagePause(GetImageBtnPause(0), CButtonImage::_nImageNormal);
   m_Toolbar.SetImagePause(GetImageBtnPause(1), CButtonImage::_nImagePushed);
   m_Toolbar.SetImagePause(GetImageBtnPause(2), CButtonImage::_nImageDisabled);
 //m_Toolbar.SetImagePause(GetImageBtnPause(3), CButtonImage::_nImageMouseOver);

   m_Toolbar.SetFont(m_Body.m_Skin.m_Font);

   ::InvalidateRect(m_hWnd, NULL, TRUE);
   ::InvalidateRect(m_Toolbar.m_hWnd, NULL, TRUE); // ???
   ::InvalidateRect(m_Body.m_Mosaic.m_hWnd, NULL, TRUE);
 //SetCursor(LoadCursor(NULL, IDC_ARROW));
}

void CFastMines2WinMain::SetPlayerName(LPCTSTR szCurrentPlayerName) {
   if(m_Body.m_Mosaic.GetPause())
      m_Body.m_Mosaic.SetPause(!m_Body.m_Mosaic.GetPause());
   if (!lstrcmp(m_Body.m_strPlayerName, szCurrentPlayerName)) return;
   m_Body.m_strPlayerName = szCurrentPlayerName;
   m_Body.m_Mosaic.GameNew();
}

const HICON CFastMines2WinMain::GetIconMosaic(EMosaic eMosaic) {
   const CImage &img = GetImageMosaic(eMosaic);
   if (img.GetImageType() == CImg::eImageICO)
      return img.GetHandleICO();
   else
      return NULL;
}

const CImage& CFastMines2WinMain::GetImageMosaic(EMosaic eMosaic) {
   static CImage ImgMosaic[eMosaicNil+1] = {
      CImage(NULL, _T("icon_MosaicTriangle1"  ), _T("ICO")),//RT_ICON),//),//_T("c:\\Project\\CPP\\_FastMines_\\FastMines2\\Resource\\MosaicTriangle1.ico"  )),//
      CImage(NULL, _T("icon_MosaicTriangle2"  ), _T("ICO")),//RT_ICON),//),//_T("c:\\Project\\CPP\\_FastMines_\\FastMines2\\Resource\\MosaicTriangle2.ico"  )),//
      CImage(NULL, _T("icon_MosaicTriangle3"  ), _T("ICO")),//RT_ICON),//),//_T("c:\\Project\\CPP\\_FastMines_\\FastMines2\\Resource\\MosaicTriangle3.ico"  )),//
      CImage(NULL, _T("icon_MosaicTriangle4"  ), _T("ICO")),//RT_ICON),//),//_T("c:\\Project\\CPP\\_FastMines_\\FastMines2\\Resource\\MosaicTriangle4.ico"  )),//
      CImage(NULL, _T("icon_MosaicSquare1"    ), _T("ICO")),//RT_ICON),//),//_T("c:\\Project\\CPP\\_FastMines_\\FastMines2\\Resource\\MosaicSquare1.ico"    )),//
      CImage(NULL, _T("icon_MosaicSquare2"    ), _T("ICO")),//RT_ICON),//),//_T("c:\\Project\\CPP\\_FastMines_\\FastMines2\\Resource\\MosaicSquare2.ico"    )),//
      CImage(NULL, _T("icon_MosaicParquet1"   ), _T("ICO")),//RT_ICON),//),//_T("c:\\Project\\CPP\\_FastMines_\\FastMines2\\Resource\\MosaicParquet1.ico"   )),//
      CImage(NULL, _T("icon_MosaicParquet2"   ), _T("ICO")),//RT_ICON),//),//_T("c:\\Project\\CPP\\_FastMines_\\FastMines2\\Resource\\MosaicParquet2.ico"   )),//
      CImage(NULL, _T("icon_MosaicTrapezoid1" ), _T("ICO")),//RT_ICON),//),//_T("c:\\Project\\CPP\\_FastMines_\\FastMines2\\Resource\\MosaicTrapezoid1.ico" )),//
      CImage(NULL, _T("icon_MosaicTrapezoid2" ), _T("ICO")),//RT_ICON),//),//_T("c:\\Project\\CPP\\_FastMines_\\FastMines2\\Resource\\MosaicTrapezoid2.ico" )),//
      CImage(NULL, _T("icon_MosaicTrapezoid3" ), _T("ICO")),//RT_ICON),//),//_T("c:\\Project\\CPP\\_FastMines_\\FastMines2\\Resource\\MosaicTrapezoid3.ico" )),//
      CImage(NULL, _T("icon_MosaicRhombus1"   ), _T("ICO")),//RT_ICON),//),//_T("c:\\Project\\CPP\\_FastMines_\\FastMines2\\Resource\\MosaicRhombus1.ico"   )),//
      CImage(NULL, _T("icon_MosaicQuadrangle1"), _T("ICO")),//RT_ICON),//),//_T("c:\\Project\\CPP\\_FastMines_\\FastMines2\\Resource\\MosaicQuadrangle1.ico")),//
      CImage(NULL, _T("icon_MosaicPentagonT24"), _T("ICO")),//RT_ICON),//),//_T("c:\\Project\\CPP\\_FastMines_\\FastMines2\\Resource\\MosaicPentagonT24.ico")),//
      CImage(NULL, _T("icon_MosaicPentagonT5" ), _T("ICO")),//RT_ICON),//),//_T("c:\\Project\\CPP\\_FastMines_\\FastMines2\\Resource\\MosaicPentagonT5.ico" )),//
      CImage(NULL, _T("icon_MosaicPentagonT10"), _T("ICO")),//RT_ICON),//),//_T("c:\\Project\\CPP\\_FastMines_\\FastMines2\\Resource\\MosaicPentagonT10.ico")),//
      CImage(NULL, _T("icon_MosaicHexagon1"   ), _T("ICO")),//RT_ICON),//),//_T("c:\\Project\\CPP\\_FastMines_\\FastMines2\\Resource\\MosaicHexagon1.ico"   )),//
      CImage(NULL, _T("icon_MosaicTrSq1"      ), _T("ICO")),//RT_ICON),//),//_T("c:\\Project\\CPP\\_FastMines_\\FastMines2\\Resource\\MosaicTrSq1.ico"      )),//
      CImage(NULL, _T("icon_MosaicTrSq2"      ), _T("ICO")),//RT_ICON),//),//_T("c:\\Project\\CPP\\_FastMines_\\FastMines2\\Resource\\MosaicTrSq2.ico"      )),//
      CImage(NULL, _T("icon_MosaicSqTrHex"    ), _T("ICO")),//RT_ICON),//),//_T("c:\\Project\\CPP\\_FastMines_\\FastMines2\\Resource\\MosaicSqTrHex.ico"    )),//
      CImage(NULL, _T("icon_MosaicNull"       ), _T("ICO")) //RT_ICON) //) //_T("c:\\Project\\CPP\\_FastMines_\\FastMines2\\Resource\\MosaicNull.ico"       )) //
   };
   return ImgMosaic[eMosaic];
}

void CFastMines2WinMain::DelOwnerDrawMenu() {
   for (LIST_MenuItem::const_iterator i=m_MenuItems.begin(); i!=m_MenuItems.end(); i++) {
      delete *i;
   }
}

void CFastMines2WinMain::SetOwnerDrawMenu(HMENU hMenu) {
   if (!hMenu) return;

   int iItems = ::GetMenuItemCount(hMenu);
   for (int i=0; i<iItems; i++) {
      UINT uID = ::GetMenuItemID(hMenu, i);
      BOOL fByPosition = ((uID==0) || (uID==-1));
      CMenuItem *pMenuItem = NULL;
      if (hMenu == ::GetSubMenu(::GetSubMenu(m_hMenu, 1), 0) ||
          hMenu == ::GetSubMenu(::GetSubMenu(m_hMenu, 1), 1) ||
          hMenu == ::GetSubMenu(::GetSubMenu(m_hMenu, 1), 2) ||
          hMenu == ::GetSubMenu(::GetSubMenu(m_hMenu, 1), 3) ||
          hMenu == ::GetSubMenu(::GetSubMenu(m_hMenu, 1), 4) )
      {
         pMenuItem = new CMenuItemIridescenceFM((hMenu!=m_hMenu), hMenu, fByPosition ? i : uID, fByPosition, fByPosition ? NULL : &GetImageMosaic((EMosaic)(uID-IDM_MOSAIC_TRIANGLE1)));
      } else {
         CImage *pImg = NULL;
         //if ((hMenu==m_hMenu) && (i==0)) pImg = new CImage(_T("?14#iconPROJECT")); // test
         pMenuItem = new CMenuItemFM((hMenu!=m_hMenu), hMenu, fByPosition ? i : uID, fByPosition, pImg);
         if (pImg) delete pImg;
      }
      if (pMenuItem)
         m_MenuItems.push_back(pMenuItem);

      SetOwnerDrawMenu(::GetSubMenu(hMenu, i));
   }
}

void CFastMines2WinMain::ReloadOwnerDrawMenu() {
   DelOwnerDrawMenu();
   if (bMainMenuOwnerDraw) {
      SetOwnerDrawMenu(m_hMenu);
      SetOwnerDrawMenu(::GetSystemMenu(m_hWnd, FALSE));
   }
}

void CFastMines2WinMain::GameSave() {
   TCHAR szFilter2[] = _T(" (*.fms)\0*.fms\0\0");
   TCHAR *szFilter = new TCHAR [CLang::GetString(IDS__FASTMINES_EXTENSIONS_FILE).GetLength() + (sizeof(szFilter2)>>(sizeof(TCHAR)-1))];
   lstrcpy(szFilter, CLang::GetString(IDS__FASTMINES_EXTENSIONS_FILE));
   memcpy(szFilter+lstrlen(szFilter), szFilter2, sizeof(szFilter2));
   CString strFileName = ::SelectFile(FALSE, m_hWnd, _T("newSave"), _T("fms"),
      szFilter,
      CLang::GetString(IDS__SAVE_FASTMINES_GAME),
      (LPCTSTR)::GetCurrentDirectory(),
      OFN_EXPLORER | OFN_OVERWRITEPROMPT |/*OFN_FILEMUSTEXIST | */OFN_HIDEREADONLY | OFN_LONGNAMES | OFN_PATHMUSTEXIST);
   delete [] szFilter; szFilter = NULL;
   if (!strFileName.IsEmpty()) {
      CFileGame fileGame;
      fileGame.SetFileName(strFileName);
      fileGame.SetData(GetMosaic(), GetSizeMosaic(), m_Body.m_Mosaic);
      fileGame.SaveFile();
   }
}

void CFastMines2WinMain::GameLoad(LPCTSTR szFileName) {
   CString strFileName;
   if (!szFileName) {
      TCHAR szFilter2[] = _T(" (*.fms)\0*.fms\0\0");
      TCHAR *szFilter = new TCHAR [CLang::GetString(IDS__FASTMINES_EXTENSIONS_FILE).GetLength() + (sizeof(szFilter2)>>(sizeof(TCHAR)-1))];
      lstrcpy(szFilter, CLang::GetString(IDS__FASTMINES_EXTENSIONS_FILE));
      memcpy(szFilter+lstrlen(szFilter), szFilter2, sizeof(szFilter2));
      strFileName = ::SelectFile(TRUE, m_hWnd, NULL, NULL,
         szFilter,
         CLang::GetString(IDS__LOAD_FASTMINES_GAME),
         (LPCTSTR)::GetCurrentDirectory());
      delete [] szFilter; szFilter = NULL;
   } else {
      strFileName = szFileName;
   }
   if (!strFileName.IsEmpty()) {
      CFileGame fileGame;
      fileGame.SetFileName(strFileName);
      BOOL bRes = fileGame.LoadFile();
      if (bRes) {
         m_Body.m_Mosaic.SetPause(false);
         m_Body.m_Mosaic.SetGame(fileGame.GetMosaic(), fileGame.GetSizeMosaic(), fileGame.GetMines(), fileGame.GetStorage());
         ::CheckMenuRadioItem(::GetSubMenu(m_hMenu, 0),
            IDM_GAME_BEGINNER, IDM_GAME_CUSTOM,
            IDM_GAME_BEGINNER + GetSkillLevel(),
            MF_BYCOMMAND);
      }
   }
}

void CFastMines2WinMain::Menu_ChangeLanguage() {
#define __SetMenuString(hMenu, id, fPos, str) \
      if (bMainMenuOwnerDraw) \
         CMenuItem::SetMenuString(hMenu, id, fPos, str); \
      else        ::SetMenuString(hMenu, id, fPos, str)

   // MenuBar
   __SetMenuString(m_hMenu, 0, TRUE, CLang::GetString(IDS__MENU_GAME   ));
   __SetMenuString(m_hMenu, 1, TRUE, CLang::GetString(IDS__MENU_MOSAIC ));
   __SetMenuString(m_hMenu, 2, TRUE, CLang::GetString(IDS__MENU_OPTIONS));
   __SetMenuString(m_hMenu, 3, TRUE, CLang::GetString(IDS__MENU_HELP   ));

   ::DrawMenuBar(m_hWnd);

   // menu Game
   HMENU hSubMenu = ::GetSubMenu(m_hMenu, 0);
   __SetMenuString(hSubMenu, IDM_GAME_NEW_GAME    , FALSE, (CLang::GetString(IDS__MENU_GAME__NEW_GAME    ) + _T("\tF2"    )));
   __SetMenuString(hSubMenu, IDM_GAME_BEGINNER    , FALSE, (CLang::GetString(IDS__MENU_GAME__BEGINNER    ) + _T("\t1"     )));
   __SetMenuString(hSubMenu, IDM_GAME_AMATEUR     , FALSE, (CLang::GetString(IDS__MENU_GAME__AMATEUR     ) + _T("\t2"     )));
   __SetMenuString(hSubMenu, IDM_GAME_PROFESSIONAL, FALSE, (CLang::GetString(IDS__MENU_GAME__PROFESSIONAL) + _T("\t3"     )));
   __SetMenuString(hSubMenu, IDM_GAME_CRAZY       , FALSE, (CLang::GetString(IDS__MENU_GAME__CRAZY       ) + _T("\t4"     )));
   __SetMenuString(hSubMenu, IDM_GAME_CUSTOM      , FALSE, (CLang::GetString(IDS__MENU_GAME__CUSTOM      ) + _T("\t5"     )));
   __SetMenuString(hSubMenu, IDM_GAME_CREATE      , FALSE, (CLang::GetString(IDS__MENU_GAME__CREATE      ) + _T("\tCtrl+C")));
   __SetMenuString(hSubMenu, IDM_GAME_SAVE        , FALSE, (CLang::GetString(IDS__MENU_GAME__SAVE        ) + _T("\tCtrl+S")));
   __SetMenuString(hSubMenu, IDM_GAME_LOAD        , FALSE, (CLang::GetString(IDS__MENU_GAME__LOAD        ) + _T("\tCtrl+O")));
   __SetMenuString(hSubMenu, IDM_GAME_PLAYERS     , FALSE, (CLang::GetString(IDS__MENU_GAME__PLAYERS     ) + _T("\t~"     )));
   __SetMenuString(hSubMenu, IDM_GAME_EXIT        , FALSE, (CLang::GetString(IDS__MENU_GAME__EXIT        ) + _T("\tAlt+F4")));


   // menu Mosaics
   hSubMenu = ::GetSubMenu(m_hMenu, 1);
   __SetMenuString(hSubMenu, 0, TRUE, (CLang::GetString(IDS__MENU_MOSAIC__TRIANGLES     ) + _T("\tNum3")));
   __SetMenuString(hSubMenu, 1, TRUE, (CLang::GetString(IDS__MENU_MOSAIC__QUADRILATERALS) + _T("\tNum4")));
   __SetMenuString(hSubMenu, 2, TRUE, (CLang::GetString(IDS__MENU_MOSAIC__PENTAGONS     ) + _T("\tNum5")));
   __SetMenuString(hSubMenu, 3, TRUE, (CLang::GetString(IDS__MENU_MOSAIC__HEXAGONS      ) + _T("\tNum6")));
   __SetMenuString(hSubMenu, 4, TRUE, (CLang::GetString(IDS__MENU_MOSAIC__OTHER         ) + _T("\tNum7")));

   hSubMenu = ::GetSubMenu(::GetSubMenu(m_hMenu, 1), 0);
   __SetMenuString(hSubMenu, IDM_MOSAIC_TRIANGLE1  , FALSE, (CLang::GetString(IDS__MOSAIC_NAME_00) + _T("\tNum30")));
   __SetMenuString(hSubMenu, IDM_MOSAIC_TRIANGLE2  , FALSE, (CLang::GetString(IDS__MOSAIC_NAME_01) + _T("\tNum31")));
   __SetMenuString(hSubMenu, IDM_MOSAIC_TRIANGLE3  , FALSE, (CLang::GetString(IDS__MOSAIC_NAME_02) + _T("\tNum32")));
   __SetMenuString(hSubMenu, IDM_MOSAIC_TRIANGLE4  , FALSE, (CLang::GetString(IDS__MOSAIC_NAME_03) + _T("\tNum33")));

   hSubMenu = ::GetSubMenu(::GetSubMenu(m_hMenu, 1), 1);
   __SetMenuString(hSubMenu, IDM_MOSAIC_SQUARE1    , FALSE, (CLang::GetString(IDS__MOSAIC_NAME_04) + _T("\tNum40")));
   __SetMenuString(hSubMenu, IDM_MOSAIC_SQUARE2    , FALSE, (CLang::GetString(IDS__MOSAIC_NAME_05) + _T("\tNum41")));
   __SetMenuString(hSubMenu, IDM_MOSAIC_PARQUET1   , FALSE, (CLang::GetString(IDS__MOSAIC_NAME_06) + _T("\tNum42")));
   __SetMenuString(hSubMenu, IDM_MOSAIC_PARQUET2   , FALSE, (CLang::GetString(IDS__MOSAIC_NAME_07) + _T("\tNum43")));
   __SetMenuString(hSubMenu, IDM_MOSAIC_TRAPEZOID1 , FALSE, (CLang::GetString(IDS__MOSAIC_NAME_08) + _T("\tNum44")));
   __SetMenuString(hSubMenu, IDM_MOSAIC_TRAPEZOID2 , FALSE, (CLang::GetString(IDS__MOSAIC_NAME_09) + _T("\tNum45")));
   __SetMenuString(hSubMenu, IDM_MOSAIC_TRAPEZOID3 , FALSE, (CLang::GetString(IDS__MOSAIC_NAME_10) + _T("\tNum46")));
   __SetMenuString(hSubMenu, IDM_MOSAIC_RHOMBUS1   , FALSE, (CLang::GetString(IDS__MOSAIC_NAME_11) + _T("\tNum47")));
   __SetMenuString(hSubMenu, IDM_MOSAIC_QUADRANGLE1, FALSE, (CLang::GetString(IDS__MOSAIC_NAME_12) + _T("\tNum48")));

   hSubMenu = ::GetSubMenu(::GetSubMenu(m_hMenu, 1), 2);
   __SetMenuString(hSubMenu, IDM_MOSAIC_PENTAGONT24, FALSE, (CLang::GetString(IDS__MOSAIC_NAME_13) + _T("\tNum50")));
   __SetMenuString(hSubMenu, IDM_MOSAIC_PENTAGONT5 , FALSE, (CLang::GetString(IDS__MOSAIC_NAME_14) + _T("\tNum51")));
   __SetMenuString(hSubMenu, IDM_MOSAIC_PENTAGONT10, FALSE, (CLang::GetString(IDS__MOSAIC_NAME_15) + _T("\tNum52")));

   hSubMenu = ::GetSubMenu(::GetSubMenu(m_hMenu, 1), 3);
   __SetMenuString(hSubMenu, IDM_MOSAIC_HEXAGON1   , FALSE, (CLang::GetString(IDS__MOSAIC_NAME_16) + _T("\tNum60")));

   hSubMenu = ::GetSubMenu(::GetSubMenu(m_hMenu, 1), 4);
   __SetMenuString(hSubMenu, IDM_MOSAIC_TRSQ1      , FALSE, (CLang::GetString(IDS__MOSAIC_NAME_17) + _T("\tNum70")));
   __SetMenuString(hSubMenu, IDM_MOSAIC_TRSQ2      , FALSE, (CLang::GetString(IDS__MOSAIC_NAME_18) + _T("\tNum71")));
   __SetMenuString(hSubMenu, IDM_MOSAIC_SQTRHEX    , FALSE, (CLang::GetString(IDS__MOSAIC_NAME_19) + _T("\tNum72")));

   // menu Options
   hSubMenu = ::GetSubMenu(m_hMenu, 2);
   __SetMenuString(hSubMenu, IDM_OPTIONS_ALWAYSMAXSIZE, FALSE, (CLang::GetString(IDS__MENU_OPTIONS__ALWAYSMAXSIZE ) + _T("")      ));
   __SetMenuString(hSubMenu, ID_KEY_AREA_MAX          , FALSE, (CLang::GetString(IDS__MENU_OPTIONS__AREA_MAX      ) + _T("\tNum*")));
   __SetMenuString(hSubMenu, ID_KEY_AREA_MIN          , FALSE, (CLang::GetString(IDS__MENU_OPTIONS__AREA_MIN      ) + _T("\tNum/")));
   __SetMenuString(hSubMenu, ID_KEY_AREA_INCREMENT    , FALSE, (CLang::GetString(IDS__MENU_OPTIONS__AREA_INCREMENT) + _T("\tNum+")));
   __SetMenuString(hSubMenu, ID_KEY_AREA_DECREMENT    , FALSE, (CLang::GetString(IDS__MENU_OPTIONS__AREA_DECREMENT) + _T("\tNum-")));
   __SetMenuString(hSubMenu, 6                        , TRUE , (CLang::GetString(IDS__MENU_OPTIONS__SKIN          ) + _T("")      ));
   __SetMenuString(hSubMenu, 8                        , TRUE , (CLang::GetString(IDS__MENU_OPTIONS__LANGUAGE      ) + _T("")      ));
   __SetMenuString(hSubMenu, IDM_OPTIONS_USE_UNKNOWN  , FALSE, (CLang::GetString(IDS__MENU_OPTIONS__USE_UNKNOWN   ) + _T("")      ));
   __SetMenuString(hSubMenu, IDM_OPTIONS_SHOW_TOOLBAR , FALSE, (CLang::GetString(IDS__MENU_OPTIONS__SHOW_TOOLBAR  ) + _T("\tF10") ));
   __SetMenuString(hSubMenu, IDM_OPTIONS_SHOW_MENU    , FALSE, (CLang::GetString(IDS__MENU_OPTIONS__SHOW_MENU     ) + _T("\tF11") ));
   __SetMenuString(hSubMenu, IDM_OPTIONS_SHOW_CAPTION , FALSE, (CLang::GetString(IDS__MENU_OPTIONS__SHOW_CAPTION  ) + _T("\tF12") ));
   __SetMenuString(hSubMenu, IDM_OPTIONS_TO_TRAY      , FALSE, (CLang::GetString(IDS__MENU_OPTIONS__TO_TRAY       ) + _T("")      ));

   hSubMenu = ::GetSubMenu(::GetSubMenu(m_hMenu, 2), 6);
   __SetMenuString(hSubMenu, 0                             , TRUE , (CLang::GetString(IDS__MENU_OPTIONS__SKIN__LOAD        ) + _T("")    ));
   __SetMenuString(HSUBMENU_SKIN, 0                        , TRUE , (CLang::GetString(IDS__MENU_OPTIONS__SKIN__LOAD_DEFAULT) + _T("")    ));
   __SetMenuString(hSubMenu, IDM_OPTIONS_SKIN_SAVE         , FALSE, (CLang::GetString(IDS__MENU_OPTIONS__SKIN__SAVE        ) + _T("")    ));
   __SetMenuString(hSubMenu, IDM_OPTIONS_SKIN_CHANGE       , FALSE, (CLang::GetString(IDS__MENU_OPTIONS__SKIN__CHANGE      ) + _T("\tF8")));
   __SetMenuString(hSubMenu, IDM_OPTIONS_SKIN_WEB_SKIN_PAGE, FALSE, (CLang::GetString(IDS__MENU_OPTIONS__SKIN__WEB_PAGE    ) + _T("")    ));

   hSubMenu = HSUBMENU_LANGUAGE;
   __SetMenuString(hSubMenu, 0, TRUE , (CLang::GetString(IDS__MENU_OPTIONS__LANGUAGE_ENGLISH) + _T("")));


   // menu Help
   hSubMenu = ::GetSubMenu(m_hMenu, 3);
   __SetMenuString(hSubMenu, 3                      , TRUE , (CLang::GetString(IDS__MENU_HELP__ASSISTANT ) + _T("")    ));
   __SetMenuString(hSubMenu, IDM_HELP_CHAMPIONS , FALSE, (CLang::GetString(IDS__MENU_HELP__CHAMPIONS ) + _T("\tF3")));
   __SetMenuString(hSubMenu, IDM_HELP_STATISTICS, FALSE, (CLang::GetString(IDS__MENU_HELP__STATISTICS) + _T("\tF4")));
   __SetMenuString(hSubMenu, IDM_HELP_ABOUT     , FALSE, (CLang::GetString(IDS__MENU_HELP__ABOUT     ) + _T("\tF1")));

   hSubMenu = HSUBMENU_ASSISTANT;
   __SetMenuString(hSubMenu, IDM_HELP_ASSISTANT_ONOFF  , FALSE, (CLang::GetString(IDS__MENU_HELP__ASSISTANT__ONOFF  ) + _T("\tA" )));
   __SetMenuString(hSubMenu, IDM_HELP_ASSISTANT_OPTIONS, FALSE, (CLang::GetString(IDS__MENU_HELP__ASSISTANT__OPTIONS) + _T("\tF5")));
   __SetMenuString(hSubMenu, IDM_HELP_ASSISTANT_SUGGEST, FALSE, (CLang::GetString(IDS__MENU_HELP__ASSISTANT__SUGGEST) + _T("\tH" )));

#undef __SetMenuString

   // change Caption
   OnMosaicChangeCounters(m_hWnd);
}

void CFastMines2WinMain::SetSkillLevelCustom(const SIZE& newSizeMosaic, int numberMines) {
   if ((newSizeMosaic == GetSizeMosaic()) &&
       (numberMines   == GetMines())) return;

   m_Body.m_Mosaic.SetSkillLevelCustom(newSizeMosaic, numberMines);
   ::CheckMenuRadioItem( ::GetSubMenu(m_hMenu, 0),
      IDM_GAME_BEGINNER, IDM_GAME_CUSTOM,
      IDM_GAME_BEGINNER + GetSkillLevel(),
      MF_BYCOMMAND);
}

bool CFastMines2WinMain::SetMosaic(EMosaic eMosaic) {
   if (eMosaic == eMosaicNil) return false;
   m_Body.m_Mosaic.SetMosaic(eMosaic);
   CheckMenuRadioItem_Mosaic(eMosaic);
   
   //   :
   // 1.    skillLevelCustom
   // 2.     . 
   //       skillLevel
   ::CheckMenuRadioItem(::GetSubMenu(m_hMenu, 0),
      IDM_GAME_BEGINNER, IDM_GAME_CUSTOM,
      IDM_GAME_BEGINNER + m_Body.m_Mosaic.GetSkillLevel(),
      MF_BYCOMMAND);
   return true;
}

HMENU CFastMines2WinMain::GetSkinSubMenu() const {
   return HSUBMENU_SKIN;
}

void CFastMines2WinMain::LoadAllFiles() {
   //m_Champions = CChampions();
   //m_Champions.LoadFile();
}
