////////////////////////////////////////////////////////////////////////////////
//                               FastMines project
//                                       Sergey Krivulya (KSerg, aka SeregaLBN)
// file name: "Image.h"
//
//   CImage -     
//
// Copyright (C) 2003-2007 Sergey Krivulya
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
////////////////////////////////////////////////////////////////////////////////

#ifndef __FILE__CIMAGE__
#define __FILE__CIMAGE__

#include "./Lib/CommonLib.h"

class CImageEasy {
private://protected:
   IPicture *m_pPicture;
protected:
   BOOL Release();
public:
   CImageEasy(LPCTSTR szFileName = NULL);
  ~CImageEasy() { Release(); }
   SIZE GetSizeImage() const;

   BOOL Draw(HDC hDC, const RECT&) const;
   BOOL Load(LPCTSTR szFileName);
   BOOL Load(const CString& strFileName) { return Load((LPCTSTR)strFileName); }
};

struct CImg {
   enum EPlace {
      // alignment
      ePlaceLeft      = 0x00,
      ePlaceCenter    = 0x01,
      ePlaceRight     = 0x02,

      ePlaceTop       = 0x00,
      ePlaceMiddle    = 0x04,
      ePlaceBottom    = 0x08,

      // placement
      ePlaceAsIs      = 0x00, //   
      ePlaceUniformly = 0x10, //  
      ePlaceStretch   = 0x20, //  
      ePlaceTile      = 0x30  // 
   };

   enum EImageType {
      eImageBMP,
      eImageICO,
      eImageWMF,
      eImageEMF,
      eImageOTH, // others image (instance of CImageEasy)
      eImageUnk
   };
};

struct CImageSerialize : public CImg {
   CString  m_strPath;
   BOOL     m_bTransparent; // image is transparent ? (only for .bmp image)
   COLORREF m_clrTransparent; // transparent color. If == CLR_INVALID then m_clrTransparent equals color of the pixel(0,0)
   BYTE     m_Place;
   CImageSerialize() :
      m_bTransparent  (TRUE),
      m_clrTransparent(CLR_INVALID),
      m_Place         (ePlaceStretch | ePlaceCenter | ePlaceMiddle) {}
   CImageSerialize(CString strPath, BOOL bTransparent = TRUE, COLORREF clrTransparent = CLR_INVALID, BOOL flagPlace = ePlaceStretch | ePlaceCenter | ePlaceMiddle) :
      m_strPath       (strPath),
      m_bTransparent  (bTransparent),
      m_clrTransparent(clrTransparent),
      m_Place         (flagPlace) {}
   const CImageSerialize& operator=(const CImageSerialize& image) {
      m_strPath        = image.m_strPath;
      m_bTransparent   = image.m_bTransparent;
      m_clrTransparent = image.m_clrTransparent;
      m_Place          = image.m_Place;
      return *this;
   }
   BOOL CImageSerialize::SerializeRead(HANDLE hFile) {
      DWORD dwNOBR = 0;
      return ::SerializeRead(hFile, m_strPath) &&
         (::ReadFile(hFile, &m_bTransparent  , sizeof(m_bTransparent  ), &dwNOBR, NULL) && (sizeof(m_bTransparent  ) == dwNOBR)) &&
         (::ReadFile(hFile, &m_clrTransparent, sizeof(m_clrTransparent), &dwNOBR, NULL) && (sizeof(m_clrTransparent) == dwNOBR)) &&
         (::ReadFile(hFile, &m_Place         , sizeof(m_Place         ), &dwNOBR, NULL) && (sizeof(m_Place         ) == dwNOBR)) &&
         ((m_Place>=ePlaceLeft) && (m_Place<=(ePlaceTile | ePlaceBottom | ePlaceRight)));
   }

   BOOL CImageSerialize::SerializeWrite(HANDLE hFile) const {
      DWORD dwNOBW = 0;
      return ::SerializeWrite(hFile, m_strPath) &&
         (::WriteFile(hFile, &m_bTransparent  , sizeof(m_bTransparent  ), &dwNOBW, NULL) && (sizeof(m_bTransparent  ) == dwNOBW)) &&
         (::WriteFile(hFile, &m_clrTransparent, sizeof(m_clrTransparent), &dwNOBW, NULL) && (sizeof(m_clrTransparent) == dwNOBW)) &&
         (::WriteFile(hFile, &m_Place         , sizeof(m_Place         ), &dwNOBW, NULL) && (sizeof(m_Place         ) == dwNOBW));
   }
};

class CImage: protected CImageSerialize, protected CImageEasy {
private:
   EImageType m_ImageType;
   HANDLE     m_hImage; // original image
   mutable SIZE m_Size; // image size

   mutable HBITMAP mm_hMask;     // . .   .
   mutable HDC     mm_hCDCMask;  //   . .   .
   mutable HDC     mm_hCDCImage; //   . .   .
   mutable HBITMAP mm_hMaskOld;
   mutable HBITMAP mm_hImageOld;
private:
   BOOL DeleteMutable() const;
   BOOL DeleteHandle();
   BOOL DrawType(HDC, const RECT&) const;
public:
   CImage(const CString& strFileName);
   CImage(LPCTSTR szFileName = NULL);
   CImage(LPCTSTR szModuleName, LPCTSTR szResourceName, LPCTSTR szResourceType);
   CImage(const CImageSerialize&);
  ~CImage() {
     DeleteHandle();
   }
   void Reset() {
      m_strPath.Empty();
      DeleteHandle();
   }
   SIZE GetSizeImage() const;

   BOOL Valid() const { return m_ImageType != eImageUnk; }
   EImageType   GetImageType() const { return m_ImageType; }
   HBITMAP      GetHandleBMP() const { DeleteMutable(); return (m_ImageType == eImageBMP) ? (HBITMAP     )m_hImage: NULL; }
   HICON        GetHandleICO() const {                  return (m_ImageType == eImageICO) ? (HICON       )m_hImage: NULL; }
   HMETAFILE    GetHandleWMF() const {                  return (m_ImageType == eImageWMF) ? (HMETAFILE   )m_hImage: NULL; }
   HENHMETAFILE GetHandleEMF() const {                  return (m_ImageType == eImageEMF) ? (HENHMETAFILE)m_hImage: NULL; }

   BOOL Draw(HDC hDC, const RECT&) const;
   BOOL DrawRotated           (HDC hDC, const RECTEX &rc, float angle, COLORREF clrBack        = CLR_INVALID /* Pixel(hDC, rc.left,rc.top) */) const;
   BOOL DrawRotatedTransparent(HDC hDC, const RECTEX &rc, float angle, COLORREF clrTransparent = CLR_INVALID /* Pixel(hDC, rc.left,rc.top) */) const;

   BOOL Load(const CString& strFileName) { return Load((LPCTSTR)strFileName); }
   BOOL Load(LPCTSTR szFileName);
   BOOL Load(LPCTSTR szModuleName, LPCTSTR szResourceName, LPCTSTR szResourceType);
   BOOL Load(const CImageSerialize&);
 //BOOL Load(HBITMAP, BOOL bTransparent);

   const CImage& operator =(const CImageSerialize &img) { Load(img); return *this; }
   const CImage& operator =(const CImage          &img) { Load(img); return *this; }
   operator CImageSerialize() const { return CImageSerialize(m_strPath, m_bTransparent, m_clrTransparent, m_Place); }

   BOOL     GetTransparent() const { return m_bTransparent; }
   BOOL     SetTransparent(BOOL bTransparent) { BOOL bTransparentOld = m_bTransparent; m_bTransparent = bTransparent; DeleteMutable(); return bTransparentOld; }
   COLORREF GetTransparentColor() const { return m_clrTransparent; }
   void     SetTransparentColor(COLORREF clrTransparent) { m_clrTransparent = clrTransparent; DeleteMutable(); }

   BYTE SetPlace(BYTE placeCombination) { BYTE placeOld = m_Place; m_Place = placeCombination; DeleteMutable(); return placeOld; }
   BYTE SetPlaceAlignmentH(BYTE placeH) { return SetPlace(GetPlacePlacement () | GetPlaceAlignmentV() | (placeH & 0x03)); }
   BYTE SetPlaceAlignmentV(BYTE placeV) { return SetPlace(GetPlacePlacement () | GetPlaceAlignmentH() | (placeV & 0x0C)); }
   BYTE SetPlaceAlignment (BYTE place ) { return SetPlace(GetPlacePlacement () |                        (place  & 0x0F)); }
   BYTE SetPlacePlacement (BYTE placeP) { return SetPlace(GetPlaceAlignmentV() | GetPlaceAlignmentH() | (placeP & 0x30)); }

   BYTE GetPlace()           const { return m_Place; }
   BYTE GetPlaceAlignmentH() const { return m_Place & 0x03; }
   BYTE GetPlaceAlignmentV() const { return m_Place & 0x0C; }
   BYTE GetPlaceAlignment () const { return m_Place & 0x0F; }
   BYTE GetPlacePlacement () const { return m_Place & 0x30; }

   class CResourceHelper { //        (   )
   private:
      CString m_strModuleName, m_strResourceType, m_strResourceName;
      int m_iResourceType, m_iResourceName;
   public:
      CResourceHelper(LPCTSTR szFileName);
      CResourceHelper(LPCTSTR szModuleName, LPCTSTR szResourceName, LPCTSTR szResourceType);
      BOOL Valid() const { return !m_strModuleName.IsEmpty() || m_iResourceType || m_iResourceName || !m_strResourceType.IsEmpty() || !m_strResourceName.IsEmpty(); }
      const CString& GetModuleName() const { return m_strModuleName; }
      LPCTSTR GetResourceType() const { return m_iResourceType ? MAKEINTRESOURCE(m_iResourceType) : (LPCTSTR)m_strResourceType; }
      LPCTSTR GetResourceName() const { return m_iResourceName ? MAKEINTRESOURCE(m_iResourceName) : (LPCTSTR)m_strResourceName; }
      CString GetResourceTypeAsString() const { return m_iResourceType ? ::Format(_T("%i"), m_iResourceType) : m_strResourceType; }
      CString GetResourceNameAsString() const { return m_iResourceName ? ::Format(_T("%i"), m_iResourceName) : m_strResourceName; }

      CString SaveToTempRandomFile(LPCTSTR szFilePrefix = NULL) const;
      CString MakePath() const { return m_strModuleName + _T('?')+GetResourceTypeAsString() + _T('#')+GetResourceNameAsString(); }
   };
};

#endif // __FILE__CIMAGE__
