////////////////////////////////////////////////////////////////////////////////
// File name: UnhandledException.cpp
// Author: Sergey Krivulya (Ce p) - KSerg (SeregaLBN)
// e-mail: Sergey_Krivulya@UkrPost.Net
// ICQ: 72099167
// WWW page: http://kserg77.narod.ru
// Date: 2007 01 24
//
// Description:     top level 
////////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "UnhandledException.h"
#include "EnumObj.h"

/*******************************************************************************************/
//
//          This module functions
//
/*******************************************************************************************/
LONG WINAPI TopLevelExceptionFilter(EXCEPTION_POINTERS*);

void CALLBACK ExceptionTextReport_Trace(LPCTSTR szReport) {
#ifdef LOG_PUT
   LOG_PUT(g_pLog, Put, LL_FATAL) _T("\r\nExceptionTextReport: %s"), szReport);
#else
   ::OutputDebugString(szReport);
   ::OutputDebugString(_T("\r\n"));
#endif
}

PF_ExceptionTextReport& ExceptionTextReport() {
   static PF_ExceptionTextReport func = ExceptionTextReport_Trace;
   return func;
}


//  top-level  
void SetUnhandledExceptionFilter_and_TextReport(IN LPTOP_LEVEL_EXCEPTION_FILTER lpTopLevelExceptionFilter, PF_ExceptionTextReport callback) {
   ::SetErrorMode(SEM_NOOPENFILEERRORBOX);
   ::SetUnhandledExceptionFilter(lpTopLevelExceptionFilter ? lpTopLevelExceptionFilter : TopLevelExceptionFilter);
   if (callback) {
      ExceptionTextReport() = callback;
   }
}

LONG WINAPI TopLevelExceptionFilter(EXCEPTION_POINTERS *pExceptionInfo) {
   CString strReport = _T("\r\nRaise top level exception!\r\n");

   MODULE_INFO::VECTOR listModules;
   SnapshotModules(listModules);

   if (pExceptionInfo) {
      EXCEPTION_RECORD *pRecord = pExceptionInfo->ExceptionRecord;
      if (pRecord) {
         CString strExceptionCode;
         CString strExceptionCodeD;
#if defined(__FILE__LOGGER__) && !defined(LOGGER_UNDEF)
         strExceptionCode  = CLogger::ExceptionCode(pRecord->ExceptionCode, FALSE);
         strExceptionCodeD = CLogger::ExceptionCode(pRecord->ExceptionCode, TRUE);
#else
         switch (pRecord->ExceptionCode) {
         #define CASE_EC(val, Description) case (val): strExceptionCode = _T(#val); strExceptionCodeD = (Description); break;
         CASE_EC(EXCEPTION_ACCESS_VIOLATION        , _T("The thread tried to read from or write to a virtual address for which it does not have the appropriate access."                                                                                                        ))
         CASE_EC(EXCEPTION_ARRAY_BOUNDS_EXCEEDED   , _T("The thread tried to access an array element that is out of bounds and the underlying hardware supports bounds checking."                                                                                               ))
         CASE_EC(EXCEPTION_BREAKPOINT              , _T("A breakpoint was encountered."                                                                                                                                                                                         ))
         CASE_EC(EXCEPTION_DATATYPE_MISALIGNMENT   , _T("The thread tried to read or write data that is misaligned on hardware that does not provide alignment. For example, 16-bit values must be aligned on 2-byte boundaries; 32-bit values on 4-byte boundaries, and so on."))
         CASE_EC(EXCEPTION_FLT_DENORMAL_OPERAND    , _T("One of the operands in a floating-point operation is denormal. A denormal value is one that is too small to represent as a standard floating-point value."                                                             ))
         CASE_EC(EXCEPTION_FLT_DIVIDE_BY_ZERO      , _T("The thread tried to divide a floating-point value by a floating-point divisor of zero."                                                                                                                                ))
         CASE_EC(EXCEPTION_FLT_INEXACT_RESULT      , _T("The result of a floating-point operation cannot be represented exactly as a decimal fraction."                                                                                                                         ))
         CASE_EC(EXCEPTION_FLT_INVALID_OPERATION   , _T("This exception represents any floating-point exception not included in this list."                                                                                                                                     ))
         CASE_EC(EXCEPTION_FLT_OVERFLOW            , _T("The exponent of a floating-point operation is greater than the magnitude allowed by the corresponding type."                                                                                                           ))
         CASE_EC(EXCEPTION_FLT_STACK_CHECK         , _T("The stack overflowed or underflowed as the result of a floating-point operation."                                                                                                                                      ))
         CASE_EC(EXCEPTION_FLT_UNDERFLOW           , _T("The exponent of a floating-point operation is less than the magnitude allowed by the corresponding type."                                                                                                              ))
         CASE_EC(EXCEPTION_ILLEGAL_INSTRUCTION     , _T("The thread tried to execute an invalid instruction."                                                                                                                                                                   ))
         CASE_EC(EXCEPTION_IN_PAGE_ERROR           , _T("The thread tried to access a page that was not present, and the system was unable to load the page. For example, this exception might occur if a network connection is lost while running a program over the network." ))
         CASE_EC(EXCEPTION_INT_DIVIDE_BY_ZERO      , _T("The thread tried to divide an integer value by an integer divisor of zero."                                                                                                                                            ))
         CASE_EC(EXCEPTION_INT_OVERFLOW            , _T("The result of an integer operation caused a carry out of the most significant bit of the result."                                                                                                                      ))
         CASE_EC(EXCEPTION_INVALID_DISPOSITION     , _T("An exception handler returned an invalid disposition to the exception dispatcher. Programmers using a high-level language such as C should never encounter this exception."                                            ))
         CASE_EC(EXCEPTION_NONCONTINUABLE_EXCEPTION, _T("The thread tried to continue execution after a noncontinuable exception occurred."                                                                                                                                     ))
         CASE_EC(EXCEPTION_PRIV_INSTRUCTION        , _T("The thread tried to execute an instruction whose operation is not allowed in the current machine mode."                                                                                                                ))
         CASE_EC(EXCEPTION_SINGLE_STEP             , _T("A trace trap or other single-instruction mechanism signaled that one instruction has been executed."                                                                                                                   ))
         CASE_EC(EXCEPTION_STACK_OVERFLOW          , _T("The thread used up its stack."                                                                                                                                                                                         ))
         #undef CASE_EC
         default:
            strExceptionCodeD = _T("Unknown exception");
            strExceptionCode.Format(_T("0x%08X"), pRecord->ExceptionCode);
         }
#endif
         strReport += ::Format(_T("ExceptionCode - %s (%s)\r\n"), (LPCTSTR)strExceptionCode, (LPCTSTR)strExceptionCodeD);
         strReport += ::Format(_T("ExceptionFlags - 0x%08X\r\n"), pRecord->ExceptionFlags);
         strReport += ::Format(_T("ExceptionRecord - 0x%08X\r\n"), pRecord->ExceptionRecord);
         strReport += ::Format(_T("ExceptionAddress - 0x%08X"), pRecord->ExceptionAddress);
         for (int i=0; i<listModules.size(); i++) {
            if ((pRecord->ExceptionAddress >= listModules[i].m_pModBaseAddr) &&
                (pRecord->ExceptionAddress <  listModules[i].m_pModBaseAddr+listModules[i].m_pModBaseSize))
            {
               strReport += ::Format(_T(" (module [%s])"), LPCTSTR(listModules[i].m_strModule));
               break;
            }
         }
         strReport += _T("\r\n");
         if (pRecord->ExceptionInformation != NULL) {
            if ((pRecord->ExceptionCode == EXCEPTION_ACCESS_VIOLATION) && (pRecord->NumberParameters >= 2)) {
               strReport += _T("ExceptionInformation: Type of operation that caused the access violation - ");
               strReport += (pRecord->ExceptionInformation[0] ? _T("the thread attempted to write to an inaccessible address") : _T("the thread attempted to read the inaccessible data"));
               strReport += ::Format(_T("\r\n                      The virtual address of the inaccessible data - 0x%08X\r\n"), pRecord->ExceptionInformation[1]);
            }
         }
      }

      PCONTEXT pContextRecord = pExceptionInfo->ContextRecord;
      if (pContextRecord) {
         strReport += _T("\r\nContext:\r\n");
         strReport += ::Format(_T("EDI:    0x%08X  ESI: 0x%08X  EAX:   0x%08X\r\n"), pContextRecord->Edi, pContextRecord->Esi, pContextRecord->Eax);
         strReport += ::Format(_T("EBX:    0x%08X  ECX: 0x%08X  EDX:   0x%08X\r\n"), pContextRecord->Ebx, pContextRecord->Ecx, pContextRecord->Edx);
         strReport += ::Format(_T("EIP:    0x%08X  EBP: 0x%08X  SegCs: 0x%08X\r\n"), pContextRecord->Eip, pContextRecord->Ebp, pContextRecord->SegCs);
         strReport += ::Format(_T("EFlags: 0x%08X  ESP: 0x%08X  SegSs: 0x%08X\r\n"), pContextRecord->EFlags, pContextRecord->Esp, pContextRecord->SegSs);
      }
      strReport += _T("\r\n");
   }

   strReport += CreateTextReport();

   ExceptionTextReport()(strReport);

   return EXCEPTION_EXECUTE_HANDLER;
}

CString CreateTextReport() {
   MODULE_INFO::VECTOR listModules;
   THREAD_INFO::VECTOR listThreads1;
   THREAD_INFO2::VECTOR listThreads2;
   SnapshotModules(listModules);
   SnapshotThreads(listThreads1);
   EnumThreads(listThreads2);

   CString strReport;
   
   COSVersion OS;
   strReport = _T("Windows platform ");
   switch (OS.Get().dwPlatformId) {
   case VER_PLATFORM_WIN32_NT     : strReport += _T("NT"); break;
   case VER_PLATFORM_WIN32_WINDOWS: strReport += _T("9x"); break;
   default                        : strReport += _T("unknown"); break;
   }
   strReport += ::Format(_T("; Version: %i.%i; Build number: %i; Platform Id: %i; CSDVersion-[%.*s]\r\n\r\n"), OS.Get().dwMajorVersion,  OS.Get().dwMinorVersion, OS.Get().dwBuildNumber, OS.Get().dwPlatformId, 128, OS.Get().szCSDVersion);


   if (listModules.size() > 0) {
      strReport += ::Format(_T("List modules (number %i):\r\n"), listModules.size());
      for (int i=0; i<listModules.size(); i++) {
         strReport += ::Format(_T(" - Address [0x%08X - 0x%08X],"), listModules[i].m_pModBaseAddr, listModules[i].m_pModBaseAddr+listModules[i].m_pModBaseSize-1);
         strReport += ::Format(_T(" Module [%s],"), LPCTSTR(listModules[i].m_strExePath));
         strReport += ::Format(_T(" Version [%s]\r\n"), LPCTSTR(GetFileVersion(listModules[i].m_strExePath)));
      }
      strReport += _T("\r\n");
   }
   if (listThreads1.size() > 0) {
      strReport += ::Format(_T("List threads (number %i):\r\n"), listThreads1.size());
      for (int i=0; i<listThreads1.size(); i++) {
         strReport += ::Format(_T(" - Thread ID 0x%08X, Usage %i, Base Priority %i, Delta Priority %i\r\n"), listThreads1[i].m_th32ThreadID, listThreads1[i].m_cntUsage, listThreads1[i].m_tpBasePri, listThreads1[i].m_tpDeltaPri);
      }
      strReport += _T("\r\n");
   }
   if (listThreads2.size() > 0) {
      strReport += ::Format(_T("List threads (number %i):\r\n"), listThreads2.size());
      for (int i=0; i<listThreads2.size(); i++) {
         CString strStartAdress;
         for (int j=0; j<listModules.size(); j++) {
            if ((listThreads2[i].m_pStartAdress >= listModules[j].m_pModBaseAddr) &&
                (listThreads2[i].m_pStartAdress <  listModules[j].m_pModBaseAddr+listModules[j].m_pModBaseSize))
            {
               strStartAdress.Format(_T(" (module [%s])"), LPCTSTR(listModules[j].m_strModule));
               break;
            }
         }
         SYSTEMTIME &st = listThreads2[i].m_stCreateTime;
         strReport += ::Format(_T(" - Thread ID 0x%08X,") \
                               _T(" Start adress 0x%08X%s,") \
                               _T(" Create time [%s],") \
                               _T(" Priority %i,") \
                               _T(" Base Priority %i\r\n"),
                               listThreads2[i].m_dwThreadID,
                               listThreads2[i].m_pStartAdress,
                               (LPCTSTR)strStartAdress,
#if defined(__FILE__LOGGER__) && !defined(LOGGER_UNDEF)
                               (LPCTSTR)CLogger::SystemTime(st, TRUE, TRUE),
#else
                               (LPCTSTR)::Format(_T("%04i.%02i.%02i %02i:%02i:%02i,%03i"),
                                                 int(st.wYear),
                                                 int(st.wMonth),
                                                 int(st.wDay),
                                                 int(st.wHour),
                                                 int(st.wMinute),
                                                 int(st.wSecond),
                                                 int(st.wMilliseconds)),
#endif
                               listThreads2[i].m_lPriority,
                               listThreads2[i].m_lBasePriority);
      }
      strReport += _T("\r\n");
   }
   return strReport;
}
