////////////////////////////////////////////////////////////////////////////////
//                               FastMines project
//                                       Sergey Krivulya (KSerg, aka SeregaLBN)
// file name: "Mosaic.h"
//
//
// Copyright (C) 2002-2007 Sergey Krivulya
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
////////////////////////////////////////////////////////////////////////////////

#ifndef __FILE__MOSAIC__
#define __FILE__MOSAIC__

#include <Windows.h>
#include <vector>

#include "./Interface.h"
#include "./Mosaic/Base.h"
#include "./StorageMines.h"
#include "./Assistant.h"

////////////////////////////////////////////////////////////////////////////////
//                            types & constants
////////////////////////////////////////////////////////////////////////////////
enum EMosaic {
   // Triangle
   eMosaicTriangle1,
   eMosaicTriangle2,
   eMosaicTriangle3,
   eMosaicTriangle4,
   // Quadrangle
   eMosaicSquare1,
   eMosaicSquare2,
   eMosaicParquet1,
   eMosaicParquet2,
   eMosaicTrapezoid1,
   eMosaicTrapezoid2,
   eMosaicTrapezoid3,
   eMosaicRhombus1,
   eMosaicQuadrangle1,
   // Pentagon
   eMosaicPentagonT24,
   eMosaicPentagonT5,
   eMosaicPentagonT10,
   // Hexagon
   eMosaicHexagon1,
   // Other
   eMosaicTrSq1,
   eMosaicTrSq2,
   eMosaicSqTrHex,
   //
   eMosaicNil  //      !!!
};

enum ESkillLevel {
   eSkillLevelBeginner,
   eSkillLevelAmateur,
   eSkillLevelProfessional,
   eSkillLevelCrazy,
   eSkillLevelCustom  //      !!!
};

struct CAssistantInfo {
   BOOL m_bUse;         // on/off m_Assistant
   struct {
      int m_iFirst;     //  ( )    (      )
      int m_iNext;      //  ( )   
   } m_Timeout;
   BOOL m_bAutoStart;   // autostart new game ?
   BOOL m_bStopWork;    //       ?
   BOOL m_bIgnorePause; // ignore Pause in game ?
   CAssistantInfo():
      m_bUse        (TRUE),
      m_bAutoStart  (TRUE),
      m_bStopWork   (FALSE),
      m_bIgnorePause(TRUE)
   {
      m_Timeout.m_iFirst = 30;
      m_Timeout.m_iNext  = 100;
   }
};

struct CSkinMosaic : public ISerialize {
public:
   static const LOGFONT& DefaultFont() {
      static LOGFONT f = {0,0,0,0, FW_BOLD,0,0,0, DEFAULT_CHARSET, OUT_DEFAULT_PRECIS,
                          CLIP_DEFAULT_PRECIS, DEFAULT_QUALITY, DEFAULT_PITCH | FF_DONTCARE,
                          _T("MS Sans Serif")};// _T("Times New Roman");//
      return f;
   }
   static CCell::CColorText DefaultTextColor() {
      static CCell::CColorText c =
         {    // BBGGRR
            {0x00000000, // _Nil       Black
             0x00800000, // _1         Navy
             0x00008000, // _2         Green
             0x000000FF, // _3         Red
             0x00000080, // _4         Maroon
             0x00FF0000, // _5         Blue
             0x00000000, // _6         Black
             0x00008080, // _7         Olive
             0x00FFFF00, // _8         Aqua
             0x00800000, // _9         Navy
             0x00008000, // _10        Green
             0x000000FF, // _11        Red
             0x00000080, // _12        Maroon
             0x00800000, // _13        Navy
             0x00008000, // _14        Green
             0x000000FF, // _15        Red
             0x00000080, // _16        Maroon
             0x00FF0000, // _17        Blue
             0x00000000, // _18        Black
             0x00008080, // _19        Olive
             0x00FFFF00, // _20        Aqua
             0x00800000, // _21        Navy
             0x00000000  // _Mine      Black
            },// BBGGRR
            {0x00808000, // _Unknown   Teal
             0x00000000, // _Clear     Black
             0x000000FF  // _Flag      Red
            }
         };
      return c;
   }
public:
   CImageSerialize   m_ImgMine, m_ImgFlag, m_ImgPause, m_ImgBckgrnd;
   LOGFONT           m_Font;
   CCell::CColorText m_ColorText;
   CCell::CBorder    m_Border;
private:
   COLORREF          m_colorBk;
public:
   CSkinMosaic():
      m_Font     (CSkinMosaic::DefaultFont()),
      m_ColorText(CSkinMosaic::DefaultTextColor()),
      m_colorBk  (CLR_INVALID)//::GetSysColor(COLOR_BTNFACE))
   {
      m_ImgPause.m_Place = CImg::ePlaceUniformly | CImg::ePlaceCenter | CImg::ePlaceMiddle;
   }
   COLORREF GetBkColor() const { return (m_colorBk==CLR_INVALID) ? ::GetSysColor(COLOR_BTNFACE) : m_colorBk; }
   void     SetBkColor(COLORREF clr) { m_colorBk = (clr==::GetSysColor(COLOR_BTNFACE)) ? CLR_INVALID : clr; }
   const CSkinMosaic& operator=(const CSkinMosaic& skin) {
      m_ImgMine    = CImage(skin.m_ImgMine   ).Valid() ? skin.m_ImgMine    : CSkinMosaic().m_ImgMine   ;
      m_ImgFlag    = CImage(skin.m_ImgFlag   ).Valid() ? skin.m_ImgFlag    : CSkinMosaic().m_ImgFlag   ;
      m_ImgPause   = CImage(skin.m_ImgPause  ).Valid() ? skin.m_ImgPause   : CSkinMosaic().m_ImgPause  ;
      m_ImgBckgrnd = CImage(skin.m_ImgBckgrnd).Valid() ? skin.m_ImgBckgrnd : CSkinMosaic().m_ImgBckgrnd;
      m_Font       = skin.m_Font;
      m_ColorText  = skin.m_ColorText;
      m_Border     = skin.m_Border;
      m_colorBk    = skin.m_colorBk;
      return *this;
   }
   BOOL SerializeRead(HANDLE hFile) {
      DWORD dwNOBR = 0;
      BOOL bUnicode;
      LOGFONTA lfA;
      LOGFONTW lfW;
      BOOL bRes = m_ImgMine   .SerializeRead(hFile) &&
                  m_ImgFlag   .SerializeRead(hFile) &&
                  m_ImgPause  .SerializeRead(hFile) &&
                  m_ImgBckgrnd.SerializeRead(hFile) &&
                  (::ReadFile(hFile, &bUnicode   , sizeof(bUnicode   ), &dwNOBR, NULL) && (sizeof(bUnicode   ) == dwNOBR)) &&
                 ( bUnicode ?
                  (::ReadFile(hFile, &lfW        , sizeof(lfW        ), &dwNOBR, NULL) && (sizeof(lfW        ) == dwNOBR)) :
                  (::ReadFile(hFile, &lfA        , sizeof(lfA        ), &dwNOBR, NULL) && (sizeof(lfA        ) == dwNOBR))
                 ) &&
                  (::ReadFile(hFile, &m_ColorText, sizeof(m_ColorText), &dwNOBR, NULL) && (sizeof(m_ColorText) == dwNOBR)) &&
                  (::ReadFile(hFile, &m_Border   , sizeof(m_Border   ), &dwNOBR, NULL) && (sizeof(m_Border   ) == dwNOBR)) &&
                  (::ReadFile(hFile, &m_colorBk  , sizeof(m_colorBk  ), &dwNOBR, NULL) && (sizeof(m_colorBk  ) == dwNOBR));
      if (bRes) {
         if (bUnicode) {
#ifdef _UNICODE
            m_Font = lfW;
#else
            m_Font.lfHeight          = lfW.lfHeight;
            m_Font.lfWidth           = lfW.lfWidth;
            m_Font.lfEscapement      = lfW.lfEscapement;
            m_Font.lfOrientation     = lfW.lfOrientation;
            m_Font.lfWeight          = lfW.lfWeight; 
            m_Font.lfItalic          = lfW.lfItalic;
            m_Font.lfUnderline       = lfW.lfUnderline;
            m_Font.lfStrikeOut       = lfW.lfStrikeOut;
            m_Font.lfCharSet         = lfW.lfCharSet;
            m_Font.lfOutPrecision    = lfW.lfOutPrecision;
            m_Font.lfClipPrecision   = lfW.lfClipPrecision;
            m_Font.lfQuality         = lfW.lfQuality;
            m_Font.lfPitchAndFamily  = lfW.lfPitchAndFamily;
            USES_CONVERSION;
            lstrcpyn((LPTSTR)m_Font.lfFaceName, W2CT(lfW.lfFaceName), LF_FACESIZE);
#endif
         } else {
#ifdef _UNICODE
            m_Font.lfHeight          = lfA.lfHeight;
            m_Font.lfWidth           = lfA.lfWidth;
            m_Font.lfEscapement      = lfA.lfEscapement;
            m_Font.lfOrientation     = lfA.lfOrientation;
            m_Font.lfWeight          = lfA.lfWeight; 
            m_Font.lfItalic          = lfA.lfItalic;
            m_Font.lfUnderline       = lfA.lfUnderline;
            m_Font.lfStrikeOut       = lfA.lfStrikeOut;
            m_Font.lfCharSet         = lfA.lfCharSet;
            m_Font.lfOutPrecision    = lfA.lfOutPrecision;
            m_Font.lfClipPrecision   = lfA.lfClipPrecision;
            m_Font.lfQuality         = lfA.lfQuality;
            m_Font.lfPitchAndFamily  = lfA.lfPitchAndFamily;
            USES_CONVERSION;
            lstrcpyn((LPTSTR)m_Font.lfFaceName, A2CT(lfA.lfFaceName), LF_FACESIZE);
#else
            m_Font = lfA;
#endif
         }
      }
      return bRes;
   }
   BOOL SerializeWrite(HANDLE hFile) const {
      DWORD dwNOBW = 0;
#ifdef _UNICODE
      BOOL bUnicode = TRUE;
#else
      BOOL bUnicode = FALSE;
#endif
      BOOL bRes = m_ImgMine   .SerializeWrite(hFile) &&
                  m_ImgFlag   .SerializeWrite(hFile) &&
                  m_ImgPause  .SerializeWrite(hFile) &&
                  m_ImgBckgrnd.SerializeWrite(hFile) &&
                  (::WriteFile(hFile, &bUnicode   , sizeof(bUnicode   ), &dwNOBW, NULL) && (sizeof(bUnicode   ) == dwNOBW)) &&
                  (::WriteFile(hFile, &m_Font     , sizeof(m_Font     ), &dwNOBW, NULL) && (sizeof(m_Font     ) == dwNOBW)) &&
                  (::WriteFile(hFile, &m_ColorText, sizeof(m_ColorText), &dwNOBW, NULL) && (sizeof(m_ColorText) == dwNOBW)) &&
                  (::WriteFile(hFile, &m_Border   , sizeof(m_Border   ), &dwNOBW, NULL) && (sizeof(m_Border   ) == dwNOBW)) &&
                  (::WriteFile(hFile, &m_colorBk  , sizeof(m_colorBk  ), &dwNOBW, NULL) && (sizeof(m_colorBk  ) == dwNOBW));
      return bRes;
   }
   BOOL IsDefaultFont() const { return (m_Font.lfItalic    == CSkinMosaic::DefaultFont().lfItalic   ) &&
                                       (m_Font.lfUnderline == CSkinMosaic::DefaultFont().lfUnderline) &&
                                       (lstrcmp(m_Font.lfFaceName, CSkinMosaic::DefaultFont().lfFaceName)==0); }
};

enum EGameStatus{ eGSReady      = 0,  //     -  ,      
                  eGSCreateGame = 1,  //       -    
                  eGSPlay       = 2,  // ,   
                  eGSEnd        = 4}; //   -  ,  

enum EPlayInfo{ ePIPlayerUnknown   = 0,  //    .   .
                ePIPlayerUser      = 1,  // /    --\_      , ..   ,  
                ePIPlayerAssistant = 2,  // /  --/   ,      
                ePIIgnorable       = 4}; //          -          

struct CGraphicContext: public CCell::CGraphicContext {
   HDC      m_hDCWnd,
            m_hDCPause;
   HBRUSH   m_hBrush;
   HFONT    m_hFont;
   LOGFONT  m_Font;
   COLORREF m_colorBk;
   SIZE     m_SizeBitmap;
   CImage   m_ImgPause,
            m_ImgBckgrnd;
   bool     m_isRefreshMosaic,
            m_isRefreshPause;
   CGraphicContext() :
      CCell::CGraphicContext(CSkinMosaic::DefaultTextColor()),
      m_hDCWnd         (NULL),
      m_hDCPause       (NULL),
      m_hBrush         (NULL),
      m_hFont          (NULL),
      m_Font           (CSkinMosaic::DefaultFont()),
      m_colorBk        (::GetSysColor(COLOR_BTNFACE)),
      m_isRefreshMosaic(true),
      m_isRefreshPause (true)
   {}
};

class CMosaic {
public:
   struct CSerialize {
      SIZE    m_SizeMosaic; //   ( )
      int     m_iArea;      // 
      EMosaic m_Mosaic;     //      
      int     m_iMines;     // -   
      bool    m_useUnknown;
      CAssistantInfo m_AssistantInfo;
      CSerialize():
         m_SizeMosaic(CMosaic::DefaultSize(eSkillLevelBeginner)),
         m_iArea     (2000),
         m_Mosaic    (eMosaicSquare1),
         m_iMines    (DefineNumberMines(eSkillLevelBeginner, eMosaicSquare1, CMosaic::DefaultSize(eSkillLevelBeginner))),
         m_useUnknown(true) {}
   };
   static SIZE DefaultSize(ESkillLevel e) { //  
      static SIZE s[eSkillLevelCustom] = {{10, 10},  // Beginner
                                          {20, 15},  // Amateur
                                          {30, 20},  // Professional
                                          {45, 25}}; // Crazy
      return s[e];
   }
   static int AreaMinimum() { return 230; }
   static float GetPercentMine (ESkillLevel, EMosaic); //         
   static int DefineNumberMines(ESkillLevel, EMosaic, const SIZE&);

   typedef std::vector<std::vector<CCell* > > MATRIX_Cell;

public:
   HWND m_hWnd;
private:
   static LPCTSTR GetWindowClass() {
      const static TCHAR szWndClass[] = _T("ClassWndFastMinesMosaic");
      return szWndClass;
   }

   MATRIX_Cell m_Mosaic;

   CGraphicContext m_GContext;

   CSerialize m_SerializeData; //   
   COORD m_CoordDown;          //       (    )
   bool  m_bPause;
                             //           GameNew()      GameBegin()     GameEnd()      GameNew()
                             //    time      |               |               |             |
                             //  -------->   | eGSCreateGame |               |             |
   EGameStatus m_GameStatus; //              |  or eGSReady  |    eGSPlay    |   eGSEnd    |
                             //   - \------ 1 -----/ \----- 2 -----/ \---- 3 ----/
                             //  1 - :
                             //   -       - 
                             //          (game status - gsCreateGame)
                             //   -      (  ),
                             //           (game status -  Ready   )
                             //  2 -   ,     (game status - Play)
                             //  3 -   /     (game status - End)
                             //
                             // PS:   gsReady   -  !      
                             //         ,       . 
                             //         ,         .

   BYTE m_PlayInfo; // 1. K  ?  / ?    2. Play from file or created game?

   int m_iCountOpen;    //     
   int m_iCountFlag;    //     
   int m_iCountUnknown;
   int m_iCountClick;
   int m_iCountTimer;   //  

   int m_iOldMines;     // -      .     ,      .
   CStorageMines m_RepositoryMines; //  load'a -    

   friend CAssistant;
   CAssistant m_Assistant;
private:
   void MosaicDestroy(/*const SIZE&*/);

   void GameBegin(COORD);
   void GameEnd(bool);
   void VerifyFlag();

   COORD WinToArray(int, int) const; //      m_Mosaic'a
   int (*GetSizeInscribedSquare)(int area, int borderWidth); //     
   void SetDCFont();

   static LRESULT CALLBACK WndProc(HWND, UINT, WPARAM, LPARAM);
   void OnPaint        (HWND);                       // WM_PAINT
   void OnLButtonDown  (HWND, BOOL, int, int, UINT); // WM_LBUTTONDOWN & WM_LBUTTONDBLCLK
   void OnLButtonDown_F(const COORD &coordLDown);
   void OnLButtonUp    (HWND, int , int, UINT);      // WM_LBUTTONUP
   void OnLButtonUp_F  (const COORD &coordLUp, bool bFromAssistant);
   void OnRButtonDown  (HWND, BOOL, int, int, UINT); // WM_RBUTTONDOWN & WM_RBUTTONDBLCLK
   void OnRButtonDown_F(const COORD &coordRDown, bool bFromAssistant, CCell::EClose setCloseAssistant);
   void OnSize         (HWND, UINT, int, int);       // WM_SIZE
   void OnTimer        (HWND, UINT);                 // WM_TIMER

   void SetClasureToMosaic();

   void SetFont         (const LOGFONT&);
   void SetCellColorText(const CCell::CColorText&);
   void SetBorder       (const CCell::CBorder&);
   void SetBrush        (COLORREF colorSolidBrush);

   void MosaicCreate();
   void ResetAssistant();
public:
   void GameNew(bool bFromAssistant = false);
   void GameCreate(); //    -    

   ESkillLevel GetSkillLevel() const; //   
   void        SetSkillLevel(ESkillLevel); //    
   void        SetSkillLevelCustom(const SIZE& newSizeMosaic, int numberMines); //    ,      
   void        SetGame(EMosaic mosaic, const SIZE& newSizeMosaic, int numberMines, const CStorageMines &pStorageCoordMines); //          (      " ")
   void        SetMosaic(EMosaic); //   
   EMosaic     GetMosaic() const; //   
   const CCell* GetCell(int x, int y      ) const {return m_Mosaic[x][y];          } //    
   const CCell* GetCell(const COORD &coord) const {return GetCell(coord.X,coord.Y);}

   int    GetMines()      const {return m_SerializeData.m_iMines;} //  
   UINT   GetMaxMines(const SIZE &size) const;                     //  -     
   UINT   GetMaxMines()   const {return GetMaxMines(GetSize());}   //  -       
   int    GetArea()       const {return m_SerializeData.m_iArea;}  //  
   void   SetArea(int newArea); //    
   SIZE (*GetSizeWindow) (const SIZE& sizeField, int area);            //   
   SIZE   GetSize()       const {return m_SerializeData.m_SizeMosaic;} //   
   int    GetNeighborNumber() const { //      
#ifdef _DEBUG
      _ASSERTE(m_Mosaic.size() > 0);
      _ASSERTE(m_Mosaic[0].size() > 0);
      _ASSERTE(m_Mosaic[0][0]);
#endif // _DEBUG
      return m_Mosaic[0][0]->GetNeighborNumber();
   }

   void ApplySkin(const CSkinMosaic& newSkin);

   void SetPause(bool);
   bool GetPause() const { return m_bPause; }
   EGameStatus GetGameStatus() const { return m_GameStatus; }
   bool GetPlayerIsUser     () const { return !!(m_PlayInfo & ePIPlayerUser ); }
   bool GetPlayerIsAssistant() const { return !!(m_PlayInfo & ePIPlayerAssistant);}
   bool GetGameIsIgnorable  () const { return !!(m_PlayInfo & ePIIgnorable  ); }
   int  GetCountMines() const {return m_SerializeData.m_iMines-m_iCountFlag;}
   int  GetCountTimer() const {return m_iCountTimer;}
   int  GetCountClick() const {return m_iCountClick;}
   int  GetCountOpen()  const {return m_iCountOpen;}
   bool GetVictory()    const {return !GetCountMines();} //    gameStatus == gsEnd;

   void SetUseUnknown(bool newUseUnknown) {m_SerializeData.m_useUnknown = newUseUnknown;}
   bool GetUseUnknown() const      {return m_SerializeData.m_useUnknown;}

   CMosaic();
  ~CMosaic();
   BOOL Create(HWND hWndParent);
   BOOL SerializeRead (HANDLE hFile);
   BOOL SerializeWrite(HANDLE hFile) const;
   void                  SetAssistantInfo(  const CAssistantInfo &newAssistantInfo);
   const CAssistantInfo& GetAssistantInfo() const { return m_SerializeData.m_AssistantInfo; }

   void AssistantSuggest(); //    -         
   CMosaic& operator=(const CMosaic &Mosaic) {
      MosaicDestroy(/*m_SerializeData.m_SizeMosaic*/);
      m_SerializeData = Mosaic.m_SerializeData;
      MosaicCreate();
      return *this;
   }

   void SetImageMine   (CImage&);
   void SetImageFlag   (CImage&);
   void SetImagePause  (CImage&);
   void SetImageBckgrnd(CImage&);

   operator CStorageMines() const;
};

#endif // __FILE__MOSAIC__
