////////////////////////////////////////////////////////////////////////////////
//                               FastMines project
//                                       Sergey Krivulya (KSerg, aka SeregaLBN)
// file name: "PentagonT24.cpp"
//
//   CPentagonT24 -  5- ,  2  4
// Copyright (C) 2002-2007 Sergey Krivulya
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
////////////////////////////////////////////////////////////////////////////////

#include "StdAfx.h"
#include "PentagonT24.h"

////////////////////////////////////////////////////////////////////////////////
//                               static function
////////////////////////////////////////////////////////////////////////////////

SIZE CPentagonT24::GetSizeInPixel(const SIZE &sizeField, int iArea) {
   m_a() = sqrt((float)iArea); //   
   m_b() = m_a()*6/11;
   SIZE result = {toLONG(m_b() + sizeField.cx*m_a()),
                  toLONG(m_b() + sizeField.cy*m_a())};
   return result;
}

int CPentagonT24::SizeInscribedSquare(int iArea, int iBorderWidth) {
   m_sq() = sqrt((float)iArea)*8/11-(iBorderWidth+iBorderWidth/SIN135a())/SQRT2();
   return toINT(m_sq());
}

////////////////////////////////////////////////////////////////////////////////
//                               virtual function
////////////////////////////////////////////////////////////////////////////////

CPentagonT24::CPentagonT24(const COORD &Coord, const SIZE &sizeField, int iArea, const CGraphicContext &gContext)
   : CCell(Coord, sizeField, iArea, gContext,
           7, 5,
           ((Coord.Y&1)<<1) + (Coord.X&1) // 0..3
          )
{
   SetPoint(iArea);
   //   
   switch (m_iDirection) {
   case 0:  m_pNeighbor[0].X = m_Coord.X-1; m_pNeighbor[0].Y = m_Coord.Y-1;
            m_pNeighbor[1].X = m_Coord.X  ; m_pNeighbor[1].Y = m_Coord.Y-1;
            m_pNeighbor[2].X = m_Coord.X-1; m_pNeighbor[2].Y = m_Coord.Y  ;
            m_pNeighbor[3].X = m_Coord.X+1; m_pNeighbor[3].Y = m_Coord.Y  ;
            m_pNeighbor[4].X = m_Coord.X-1; m_pNeighbor[4].Y = m_Coord.Y+1;
            m_pNeighbor[5].X = m_Coord.X  ; m_pNeighbor[5].Y = m_Coord.Y+1;
            m_pNeighbor[6].X = m_Coord.X+1; m_pNeighbor[6].Y = m_Coord.Y+1; break;
   case 1:  m_pNeighbor[0].X = m_Coord.X-1; m_pNeighbor[0].Y = m_Coord.Y-1;
            m_pNeighbor[1].X = m_Coord.X  ; m_pNeighbor[1].Y = m_Coord.Y-1;
            m_pNeighbor[2].X = m_Coord.X+1; m_pNeighbor[2].Y = m_Coord.Y-1;
            m_pNeighbor[3].X = m_Coord.X-1; m_pNeighbor[3].Y = m_Coord.Y  ;
            m_pNeighbor[4].X = m_Coord.X+1; m_pNeighbor[4].Y = m_Coord.Y  ;
            m_pNeighbor[5].X = m_Coord.X-1; m_pNeighbor[5].Y = m_Coord.Y+1;
            m_pNeighbor[6].X = m_Coord.X  ; m_pNeighbor[6].Y = m_Coord.Y+1; break;
   case 2:  m_pNeighbor[0].X = m_Coord.X  ; m_pNeighbor[0].Y = m_Coord.Y-1;
            m_pNeighbor[1].X = m_Coord.X+1; m_pNeighbor[1].Y = m_Coord.Y-1;
            m_pNeighbor[2].X = m_Coord.X-1; m_pNeighbor[2].Y = m_Coord.Y  ;
            m_pNeighbor[3].X = m_Coord.X+1; m_pNeighbor[3].Y = m_Coord.Y  ;
            m_pNeighbor[4].X = m_Coord.X-1; m_pNeighbor[4].Y = m_Coord.Y+1;
            m_pNeighbor[5].X = m_Coord.X  ; m_pNeighbor[5].Y = m_Coord.Y+1;
            m_pNeighbor[6].X = m_Coord.X+1; m_pNeighbor[6].Y = m_Coord.Y+1; break;
   case 3:  m_pNeighbor[0].X = m_Coord.X-1; m_pNeighbor[0].Y = m_Coord.Y-1;
            m_pNeighbor[1].X = m_Coord.X  ; m_pNeighbor[1].Y = m_Coord.Y-1;
            m_pNeighbor[2].X = m_Coord.X+1; m_pNeighbor[2].Y = m_Coord.Y-1;
            m_pNeighbor[3].X = m_Coord.X-1; m_pNeighbor[3].Y = m_Coord.Y  ;
            m_pNeighbor[4].X = m_Coord.X+1; m_pNeighbor[4].Y = m_Coord.Y  ;
            m_pNeighbor[5].X = m_Coord.X  ; m_pNeighbor[5].Y = m_Coord.Y+1;
            m_pNeighbor[6].X = m_Coord.X+1; m_pNeighbor[6].Y = m_Coord.Y+1; break;
   }
   VerifyNeighbor(sizeField);
}

void CPentagonT24::SetPoint(int iArea) {
   if (m_Coord.X==0 && m_Coord.Y==0) {
      m_a() = sqrt((float)iArea); //   
      m_b() = m_a()*6/11;
      m_c() = m_b()/2;
      m_sq() = m_a()*8/11-(m_GContext.m_Border.m_iWidth+m_GContext.m_Border.m_iWidth/SIN135a())/SQRT2();
   }

   //    
   float oX = m_a()*((m_Coord.X>>1)<<1); // offset X
   float oY = m_a()*((m_Coord.Y>>1)<<1); // offset Y
   switch (m_iDirection) {
   case 0:
      m_pRegion[0].x = toLONG(oX +           m_a());   m_pRegion[0].y = toLONG(oY + m_b()          );
      m_pRegion[1].x = toLONG(oX + m_c()          );   m_pRegion[1].y = toLONG(oY + m_c()          );
      m_pRegion[2].x = toLONG(oX                  );   m_pRegion[2].y = toLONG(oY +           m_a());
      m_pRegion[4].x = toLONG(oX + m_c() +   m_a());   m_pRegion[4].y = toLONG(oY + m_c() +   m_a());
      m_pRegion[3].x = toLONG(oX + m_b()          );   m_pRegion[3].y = toLONG(oY + m_b() +   m_a());
      break;
   case 1:
      m_pRegion[3].x = toLONG(oX + m_c() +   m_a());   m_pRegion[3].y = toLONG(oY + m_c() +   m_a());
      m_pRegion[2].x = toLONG(oX +           m_a());   m_pRegion[2].y = toLONG(oY + m_b()          );
      m_pRegion[1].x = toLONG(oX + m_b() +   m_a());   m_pRegion[1].y = toLONG(oY                  );
      m_pRegion[0].x = toLONG(oX + m_c() + 2*m_a());   m_pRegion[0].y = toLONG(oY + m_c()          );
      m_pRegion[4].x = toLONG(oX +         2*m_a());   m_pRegion[4].y = toLONG(oY +           m_a());
      break;
   case 2:
      m_pRegion[0].x = toLONG(oX + m_c()          );   m_pRegion[0].y = toLONG(oY + m_c() + 2*m_a());
      m_pRegion[1].x = toLONG(oX +           m_a());   m_pRegion[1].y = toLONG(oY + m_b() + 2*m_a());
      m_pRegion[2].x = toLONG(oX + m_b() +   m_a());   m_pRegion[2].y = toLONG(oY +         2*m_a());
      m_pRegion[3].x = toLONG(oX + m_c() +   m_a());   m_pRegion[3].y = toLONG(oY + m_c() +   m_a());
      m_pRegion[4].x = toLONG(oX + m_b()          );   m_pRegion[4].y = toLONG(oY + m_b() +   m_a());
      break;
   case 3:
      m_pRegion[0].x = toLONG(oX + m_b() +   m_a());   m_pRegion[0].y = toLONG(oY +         2*m_a());
      m_pRegion[1].x = toLONG(oX + m_c() + 2*m_a());   m_pRegion[1].y = toLONG(oY + m_c() + 2*m_a());
      m_pRegion[2].x = toLONG(oX + m_b() + 2*m_a());   m_pRegion[2].y = toLONG(oY + m_b() +   m_a());
      m_pRegion[3].x = toLONG(oX +         2*m_a());   m_pRegion[3].y = toLONG(oY +           m_a());
      m_pRegion[4].x = toLONG(oX + m_c() +   m_a());   m_pRegion[4].y = toLONG(oY + m_c() +   m_a());
      break;
   }

   //       -     /
   float w2 = m_GContext.m_Border.m_iWidth/SQRT2();
   switch (m_iDirection) {
   case 0:  m_Square.left   = toLONG(m_pRegion[1].x+w2);  m_Square.right  = toLONG(m_pRegion[1].x+w2 + m_sq());
            m_Square.bottom = toLONG(m_pRegion[4].y-w2);  m_Square.top    = toLONG(m_pRegion[4].y-w2 - m_sq()); break;
   case 1:  m_Square.left   = toLONG(m_pRegion[3].x+w2);  m_Square.right  = toLONG(m_pRegion[3].x+w2 + m_sq());
            m_Square.top    = toLONG(m_pRegion[0].y+w2);  m_Square.bottom = toLONG(m_pRegion[0].y+w2 + m_sq()); break;
   case 2:  m_Square.right  = toLONG(m_pRegion[3].x-w2);  m_Square.left   = toLONG(m_pRegion[3].x-w2 - m_sq());
            m_Square.bottom = toLONG(m_pRegion[0].y-w2);  m_Square.top    = toLONG(m_pRegion[0].y-w2 - m_sq()); break;
   case 3:  m_Square.right  = toLONG(m_pRegion[1].x-w2);  m_Square.left   = toLONG(m_pRegion[1].x-w2 - m_sq());
            m_Square.top    = toLONG(m_pRegion[4].y+w2);  m_Square.bottom = toLONG(m_pRegion[4].y+w2 + m_sq()); break;
   }
   CCell::SetPoint(iArea);
}

void CPentagonT24::Paint() const {
   CCell::Paint();

   switch (m_iDirection) {
   case 0: case 1:   SelectObject(m_GContext.m_hDCTmp, m_bDown ? m_GContext.m_hPenShadow : m_GContext.m_hPenLight ); break;
   case 2: case 3:   SelectObject(m_GContext.m_hDCTmp, m_bDown ? m_GContext.m_hPenLight  : m_GContext.m_hPenShadow); break;
   }
   MoveToEx(m_GContext.m_hDCTmp, m_pRegion[0].x, m_pRegion[0].y, NULL);
   LineTo  (m_GContext.m_hDCTmp, m_pRegion[1].x, m_pRegion[1].y);
   LineTo  (m_GContext.m_hDCTmp, m_pRegion[2].x, m_pRegion[2].y);
   LineTo  (m_GContext.m_hDCTmp, m_pRegion[3].x, m_pRegion[3].y);
   switch (m_iDirection) {
   case 0: case 1:   SelectObject(m_GContext.m_hDCTmp, m_bDown ? m_GContext.m_hPenLight  : m_GContext.m_hPenShadow); break;
   case 2: case 3:   SelectObject(m_GContext.m_hDCTmp, m_bDown ? m_GContext.m_hPenShadow : m_GContext.m_hPenLight ); break;
   }
   LineTo  (m_GContext.m_hDCTmp, m_pRegion[4].x, m_pRegion[4].y);
   LineTo  (m_GContext.m_hDCTmp, m_pRegion[0].x, m_pRegion[0].y);

   ::BitBltMasked(
      m_GContext.m_hDCDst, GetOuterRect(),
      m_GContext.m_hDCTmp, GetOuterRect().pointLT(),
      m_pRegion, 5, GetOuterRect().pointLT());
}
