////////////////////////////////////////////////////////////////////////////////
//                               FastMines project
//                                       Sergey Krivulya (KSerg, aka SeregaLBN)
// file name: "SqTrHex.cpp"
//
//   CSqTrHex - 6Square 4Triangle 2Hexagon
// Copyright (C) 2002-2007 Sergey Krivulya
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
////////////////////////////////////////////////////////////////////////////////

#include "StdAfx.h"
#include "SqTrHex.h"

////////////////////////////////////////////////////////////////////////////////
//                               static function
////////////////////////////////////////////////////////////////////////////////

SIZE CSqTrHex::GetSizeInPixel(const SIZE &sizeField, int iArea) {
   m_a() = sqrt(iArea/(0.5f+1/SQRT3()));
   m_h() = m_a()*SQRT3()/2;

   SIZE result = {toLONG(m_a()/2+m_h() + m_a()/2*((sizeField.cx+2)/3) +
                                         m_h() * ((sizeField.cx+1)/3) +
                                (m_a()/2+m_h())* ((sizeField.cx+0)/3)),
                  toLONG(m_a()/2       + m_h() * ((sizeField.cy+1)/2)+
                         m_a()*3/2*              ((sizeField.cy+0)/2))};
   return result;
}

int CSqTrHex::SizeInscribedSquare(int iArea, int iBorderWidth) {
   m_a() = sqrt(iArea/(0.5f+1/SQRT3()));
   m_sq() = (m_a()*SQRT3()-iBorderWidth*6)/(SQRT3()+2);
   return toINT(m_sq());
}

////////////////////////////////////////////////////////////////////////////////
//                               virtual function
////////////////////////////////////////////////////////////////////////////////

CSqTrHex::CSqTrHex(const COORD &Coord, const SIZE &sizeField, int iArea, const CGraphicContext &gContext)
   : CCell(Coord, sizeField, iArea, gContext,
           12, 6,
           (Coord.Y&3)*3+(Coord.X%3) // 0..11
          )
{
   SetPoint(iArea);
   //   
   switch (m_iDirection) {
   case  0:
      m_pNeighbor[ 0].X = m_Coord.X-1;   m_pNeighbor[ 0].Y = m_Coord.Y-1;
      m_pNeighbor[ 1].X = m_Coord.X+1;   m_pNeighbor[ 1].Y = m_Coord.Y  ;
      m_pNeighbor[ 2].X = m_Coord.X-2;   m_pNeighbor[ 2].Y = m_Coord.Y+1;
      m_pNeighbor[ 3].X = m_Coord.X-1;   m_pNeighbor[ 3].Y = m_Coord.Y+1;
      m_pNeighbor[ 4].X = m_Coord.X  ;   m_pNeighbor[ 4].Y = m_Coord.Y+1;
      m_pNeighbor[ 5].X = m_Coord.X+1;   m_pNeighbor[ 5].Y = m_Coord.Y+1;
      m_pNeighbor[ 6] =
      m_pNeighbor[ 7] =
      m_pNeighbor[ 8] =
      m_pNeighbor[ 9] =
      m_pNeighbor[10] =
      m_pNeighbor[11] = INCORRECT_COORD;
      break;
   case  1:
      m_pNeighbor[ 0].X = m_Coord.X-2;   m_pNeighbor[ 0].Y = m_Coord.Y-1;
      m_pNeighbor[ 1].X = m_Coord.X  ;   m_pNeighbor[ 1].Y = m_Coord.Y-1;
      m_pNeighbor[ 2].X = m_Coord.X-1;   m_pNeighbor[ 2].Y = m_Coord.Y  ;
      m_pNeighbor[ 3].X = m_Coord.X+1;   m_pNeighbor[ 3].Y = m_Coord.Y  ;
      m_pNeighbor[ 4].X = m_Coord.X-2;   m_pNeighbor[ 4].Y = m_Coord.Y+1;
      m_pNeighbor[ 5].X = m_Coord.X-1;   m_pNeighbor[ 5].Y = m_Coord.Y+1;
      m_pNeighbor[ 6].X = m_Coord.X  ;   m_pNeighbor[ 6].Y = m_Coord.Y+1;
      m_pNeighbor[ 7].X = m_Coord.X+1;   m_pNeighbor[ 7].Y = m_Coord.Y+1;
      m_pNeighbor[ 8] =
      m_pNeighbor[ 9] =
      m_pNeighbor[10] =
      m_pNeighbor[11] = INCORRECT_COORD;
      break;
   case  2:
      m_pNeighbor[ 0].X = m_Coord.X-3;   m_pNeighbor[ 0].Y = m_Coord.Y-1;
      m_pNeighbor[ 1].X = m_Coord.X-1;   m_pNeighbor[ 1].Y = m_Coord.Y-1;
      m_pNeighbor[ 2].X = m_Coord.X  ;   m_pNeighbor[ 2].Y = m_Coord.Y-1;
      m_pNeighbor[ 3].X = m_Coord.X-1;   m_pNeighbor[ 3].Y = m_Coord.Y  ;
      m_pNeighbor[ 4].X = m_Coord.X-1;   m_pNeighbor[ 4].Y = m_Coord.Y+1;
      m_pNeighbor[ 5].X = m_Coord.X  ;   m_pNeighbor[ 5].Y = m_Coord.Y+1;
      m_pNeighbor[ 6] =
      m_pNeighbor[ 7] =
      m_pNeighbor[ 8] =
      m_pNeighbor[ 9] =
      m_pNeighbor[10] =
      m_pNeighbor[11] = INCORRECT_COORD;
      break;
   case  3:
      m_pNeighbor[ 0].X = m_Coord.X  ;   m_pNeighbor[ 0].Y = m_Coord.Y-1;
      m_pNeighbor[ 1].X = m_Coord.X+1;   m_pNeighbor[ 1].Y = m_Coord.Y-1;
      m_pNeighbor[ 2].X = m_Coord.X-2;   m_pNeighbor[ 2].Y = m_Coord.Y  ;
      m_pNeighbor[ 3].X = m_Coord.X-1;   m_pNeighbor[ 3].Y = m_Coord.Y  ;
      m_pNeighbor[ 4].X = m_Coord.X+1;   m_pNeighbor[ 4].Y = m_Coord.Y  ;
      m_pNeighbor[ 5].X = m_Coord.X-1;   m_pNeighbor[ 5].Y = m_Coord.Y+1;
      m_pNeighbor[ 6].X = m_Coord.X  ;   m_pNeighbor[ 6].Y = m_Coord.Y+1;
      m_pNeighbor[ 7].X = m_Coord.X  ;   m_pNeighbor[ 7].Y = m_Coord.Y+2;
      m_pNeighbor[ 8] =
      m_pNeighbor[ 9] =
      m_pNeighbor[10] =
      m_pNeighbor[11] = INCORRECT_COORD;
      break;
   case  4:
      m_pNeighbor[ 0].X = m_Coord.X-1;   m_pNeighbor[ 0].Y = m_Coord.Y-1;
      m_pNeighbor[ 1].X = m_Coord.X  ;   m_pNeighbor[ 1].Y = m_Coord.Y-1;
      m_pNeighbor[ 2].X = m_Coord.X+1;   m_pNeighbor[ 2].Y = m_Coord.Y-1;
      m_pNeighbor[ 3].X = m_Coord.X+2;   m_pNeighbor[ 3].Y = m_Coord.Y-1;
      m_pNeighbor[ 4].X = m_Coord.X-1;   m_pNeighbor[ 4].Y = m_Coord.Y  ;
      m_pNeighbor[ 5].X = m_Coord.X+1;   m_pNeighbor[ 5].Y = m_Coord.Y  ;
      m_pNeighbor[ 6].X = m_Coord.X+2;   m_pNeighbor[ 6].Y = m_Coord.Y  ;
      m_pNeighbor[ 7].X = m_Coord.X-1;   m_pNeighbor[ 7].Y = m_Coord.Y+1;
      m_pNeighbor[ 8].X = m_Coord.X  ;   m_pNeighbor[ 8].Y = m_Coord.Y+1;
      m_pNeighbor[ 9].X = m_Coord.X+1;   m_pNeighbor[ 9].Y = m_Coord.Y+1;
      m_pNeighbor[10].X = m_Coord.X+2;   m_pNeighbor[10].Y = m_Coord.Y+1;
      m_pNeighbor[11].X = m_Coord.X-1;   m_pNeighbor[11].Y = m_Coord.Y+2;
      break;
   case  5:
      m_pNeighbor[ 0].X = m_Coord.X-1;   m_pNeighbor[ 0].Y = m_Coord.Y-2;
      m_pNeighbor[ 1].X = m_Coord.X  ;   m_pNeighbor[ 1].Y = m_Coord.Y-2;
      m_pNeighbor[ 2].X = m_Coord.X-1;   m_pNeighbor[ 2].Y = m_Coord.Y-1;
      m_pNeighbor[ 3].X = m_Coord.X  ;   m_pNeighbor[ 3].Y = m_Coord.Y-1;
      m_pNeighbor[ 4].X = m_Coord.X+1;   m_pNeighbor[ 4].Y = m_Coord.Y-1;
      m_pNeighbor[ 5].X = m_Coord.X+2;   m_pNeighbor[ 5].Y = m_Coord.Y-1;
      m_pNeighbor[ 6].X = m_Coord.X-1;   m_pNeighbor[ 6].Y = m_Coord.Y  ;
      m_pNeighbor[ 7].X = m_Coord.X+1;   m_pNeighbor[ 7].Y = m_Coord.Y  ;
      m_pNeighbor[ 8] =
      m_pNeighbor[ 9] =
      m_pNeighbor[10] =
      m_pNeighbor[11] = INCORRECT_COORD;
      break;
   case  6:
      m_pNeighbor[ 0].X = m_Coord.X-2;   m_pNeighbor[ 0].Y = m_Coord.Y-1;
      m_pNeighbor[ 1].X = m_Coord.X  ;   m_pNeighbor[ 1].Y = m_Coord.Y-1;
      m_pNeighbor[ 2].X = m_Coord.X+1;   m_pNeighbor[ 2].Y = m_Coord.Y-1;
      m_pNeighbor[ 3].X = m_Coord.X-1;   m_pNeighbor[ 3].Y = m_Coord.Y  ;
      m_pNeighbor[ 4].X = m_Coord.X-1;   m_pNeighbor[ 4].Y = m_Coord.Y+1;
      m_pNeighbor[ 5].X = m_Coord.X  ;   m_pNeighbor[ 5].Y = m_Coord.Y+1;
      m_pNeighbor[ 6] =
      m_pNeighbor[ 7] =
      m_pNeighbor[ 8] =
      m_pNeighbor[ 9] =
      m_pNeighbor[10] =
      m_pNeighbor[11] = INCORRECT_COORD;
      break;
   case  7:
      m_pNeighbor[ 0].X = m_Coord.X  ;   m_pNeighbor[ 0].Y = m_Coord.Y-1;
      m_pNeighbor[ 1].X = m_Coord.X+1;   m_pNeighbor[ 1].Y = m_Coord.Y  ;
      m_pNeighbor[ 2].X = m_Coord.X-2;   m_pNeighbor[ 2].Y = m_Coord.Y+1;
      m_pNeighbor[ 3].X = m_Coord.X-1;   m_pNeighbor[ 3].Y = m_Coord.Y+1;
      m_pNeighbor[ 4].X = m_Coord.X  ;   m_pNeighbor[ 4].Y = m_Coord.Y+1;
      m_pNeighbor[ 5].X = m_Coord.X+1;   m_pNeighbor[ 5].Y = m_Coord.Y+1;
      m_pNeighbor[ 6] =
      m_pNeighbor[ 7] =
      m_pNeighbor[ 8] =
      m_pNeighbor[ 9] =
      m_pNeighbor[10] =
      m_pNeighbor[11] = INCORRECT_COORD;
      break;
   case  8:
      m_pNeighbor[ 0].X = m_Coord.X-1;   m_pNeighbor[ 0].Y = m_Coord.Y-1;
      m_pNeighbor[ 1].X = m_Coord.X+1;   m_pNeighbor[ 1].Y = m_Coord.Y-1;
      m_pNeighbor[ 2].X = m_Coord.X-1;   m_pNeighbor[ 2].Y = m_Coord.Y  ;
      m_pNeighbor[ 3].X = m_Coord.X+1;   m_pNeighbor[ 3].Y = m_Coord.Y  ;
      m_pNeighbor[ 4].X = m_Coord.X-2;   m_pNeighbor[ 4].Y = m_Coord.Y+1;
      m_pNeighbor[ 5].X = m_Coord.X-1;   m_pNeighbor[ 5].Y = m_Coord.Y+1;
      m_pNeighbor[ 6].X = m_Coord.X  ;   m_pNeighbor[ 6].Y = m_Coord.Y+1;
      m_pNeighbor[ 7].X = m_Coord.X+1;   m_pNeighbor[ 7].Y = m_Coord.Y+1;
      m_pNeighbor[ 8] =
      m_pNeighbor[ 9] =
      m_pNeighbor[10] =
      m_pNeighbor[11] = INCORRECT_COORD;
      break;
   case  9:
      m_pNeighbor[ 0].X = m_Coord.X  ;   m_pNeighbor[ 0].Y = m_Coord.Y-2;
      m_pNeighbor[ 1].X = m_Coord.X+1;   m_pNeighbor[ 1].Y = m_Coord.Y-2;
      m_pNeighbor[ 2].X = m_Coord.X-1;   m_pNeighbor[ 2].Y = m_Coord.Y-1;
      m_pNeighbor[ 3].X = m_Coord.X  ;   m_pNeighbor[ 3].Y = m_Coord.Y-1;
      m_pNeighbor[ 4].X = m_Coord.X+1;   m_pNeighbor[ 4].Y = m_Coord.Y-1;
      m_pNeighbor[ 5].X = m_Coord.X+2;   m_pNeighbor[ 5].Y = m_Coord.Y-1;
      m_pNeighbor[ 6].X = m_Coord.X-1;   m_pNeighbor[ 6].Y = m_Coord.Y  ;
      m_pNeighbor[ 7].X = m_Coord.X+1;   m_pNeighbor[ 7].Y = m_Coord.Y  ;
      m_pNeighbor[ 8] =
      m_pNeighbor[ 9] =
      m_pNeighbor[10] =
      m_pNeighbor[11] = INCORRECT_COORD;
      break;
   case 10:
      m_pNeighbor[ 0].X = m_Coord.X  ;   m_pNeighbor[ 0].Y = m_Coord.Y-1;
      m_pNeighbor[ 1].X = m_Coord.X+1;   m_pNeighbor[ 1].Y = m_Coord.Y-1;
      m_pNeighbor[ 2].X = m_Coord.X-2;   m_pNeighbor[ 2].Y = m_Coord.Y  ;
      m_pNeighbor[ 3].X = m_Coord.X-1;   m_pNeighbor[ 3].Y = m_Coord.Y  ;
      m_pNeighbor[ 4].X = m_Coord.X+1;   m_pNeighbor[ 4].Y = m_Coord.Y  ;
      m_pNeighbor[ 5].X = m_Coord.X  ;   m_pNeighbor[ 5].Y = m_Coord.Y+1;
      m_pNeighbor[ 6].X = m_Coord.X+1;   m_pNeighbor[ 6].Y = m_Coord.Y+1;
      m_pNeighbor[ 7].X = m_Coord.X+1;   m_pNeighbor[ 7].Y = m_Coord.Y+2;
      m_pNeighbor[ 8] =
      m_pNeighbor[ 9] =
      m_pNeighbor[10] =
      m_pNeighbor[11] = INCORRECT_COORD;
      break;
   case 11:
      m_pNeighbor[ 0].X = m_Coord.X-1;   m_pNeighbor[ 0].Y = m_Coord.Y-1;
      m_pNeighbor[ 1].X = m_Coord.X  ;   m_pNeighbor[ 1].Y = m_Coord.Y-1;
      m_pNeighbor[ 2].X = m_Coord.X+1;   m_pNeighbor[ 2].Y = m_Coord.Y-1;
      m_pNeighbor[ 3].X = m_Coord.X+2;   m_pNeighbor[ 3].Y = m_Coord.Y-1;
      m_pNeighbor[ 4].X = m_Coord.X-1;   m_pNeighbor[ 4].Y = m_Coord.Y  ;
      m_pNeighbor[ 5].X = m_Coord.X+1;   m_pNeighbor[ 5].Y = m_Coord.Y  ;
      m_pNeighbor[ 6].X = m_Coord.X+2;   m_pNeighbor[ 6].Y = m_Coord.Y  ;
      m_pNeighbor[ 7].X = m_Coord.X  ;   m_pNeighbor[ 7].Y = m_Coord.Y+1;
      m_pNeighbor[ 8].X = m_Coord.X+1;   m_pNeighbor[ 8].Y = m_Coord.Y+1;
      m_pNeighbor[ 9].X = m_Coord.X+2;   m_pNeighbor[ 9].Y = m_Coord.Y+1;
      m_pNeighbor[10].X = m_Coord.X+3;   m_pNeighbor[10].Y = m_Coord.Y+1;
      m_pNeighbor[11].X = m_Coord.X  ;   m_pNeighbor[11].Y = m_Coord.Y+2;
      break;
   }
   VerifyNeighbor(sizeField);
}

bool CSqTrHex::PointInRegion(const POINT &point) const { //      
   switch (m_iDirection) {
   case  0:
   case  2:
   case  6:
   case  7: return PointInPolygon(point, m_pRegion, 3);
   case  1:
   case  3:
   case  5:
   case  8:
   case  9:
   case 10: return PointInPolygon(point, m_pRegion, 4);
   case  4:
   case 11: return PointInPolygon(point, m_pRegion, 6);
   }
   return false;
}

void CSqTrHex::SetPoint(int iArea) {
   if (m_Coord.X==0 && m_Coord.Y==0) {
      m_a() = sqrt(iArea/(0.5f+1/SQRT3()));
      m_h() = m_a()*SQRT3()/2;
      m_sq() = (m_a()*SQRT3()-m_GContext.m_Border.m_iWidth*6)/(SQRT3()+2);
   }

   //    
   float b = m_a()/2;
   float oX = (m_h()*2+m_a()  )*(m_Coord.X/3) + m_a()+m_h();   // offset X
   float oY = (m_h()*2+m_a()*3)*(m_Coord.Y/4) + m_a()*2+m_h(); // offset Y
   switch (m_iDirection) {
   case 0:
      m_pRegion[0].x = toLONG(oX - b-m_h()      ); m_pRegion[0].y = toLONG(oY - m_a()-b-m_h()    );
      m_pRegion[1].x = toLONG(oX - m_h()        ); m_pRegion[1].y = toLONG(oY - m_a()-b          );
      m_pRegion[2].x = toLONG(oX - m_h()-m_a()  ); m_pRegion[2].y = toLONG(oY - m_a()-b          );
      break;
   case 1:
      m_pRegion[0].x = toLONG(oX - b            ); m_pRegion[0].y = toLONG(oY - m_a()-m_a()-m_h());
      m_pRegion[1].x = toLONG(oX - b-m_h()      ); m_pRegion[1].y = toLONG(oY - m_a()-b-m_h()    );
      m_pRegion[2].x = toLONG(oX - m_h()        ); m_pRegion[2].y = toLONG(oY - m_a()-b          );
      m_pRegion[3].x = toLONG(oX                ); m_pRegion[3].y = toLONG(oY - m_a()-m_a()      );
      break;
   case 2:
      m_pRegion[0].x = toLONG(oX + b            ); m_pRegion[0].y = toLONG(oY - m_a()-m_a()-m_h());
      m_pRegion[1].x = toLONG(oX - b            ); m_pRegion[1].y = toLONG(oY - m_a()-m_a()-m_h());
      m_pRegion[2].x = toLONG(oX                ); m_pRegion[2].y = toLONG(oY - m_a()-m_a()      );
      break;
   case 3:
      m_pRegion[0].x = toLONG(oX - m_h()        ); m_pRegion[0].y = toLONG(oY - m_a()-b          );
      m_pRegion[1].x = toLONG(oX - m_a()-m_h()  ); m_pRegion[1].y = toLONG(oY - m_a()-b          );
      m_pRegion[2].x = toLONG(oX - m_a()-m_h()  ); m_pRegion[2].y = toLONG(oY - b                );
      m_pRegion[3].x = toLONG(oX - m_h()        ); m_pRegion[3].y = toLONG(oY - b                );
      break;
   case 4:
      m_pRegion[0].x = toLONG(oX                ); m_pRegion[0].y = toLONG(oY - m_a()-m_a()      );
      m_pRegion[1].x = toLONG(oX - m_h()        ); m_pRegion[1].y = toLONG(oY - m_a()-b          );
      m_pRegion[2].x = toLONG(oX - m_h()        ); m_pRegion[2].y = toLONG(oY - b                );
      m_pRegion[3].x = toLONG(oX                ); m_pRegion[3].y = toLONG(oY                    );
      m_pRegion[4].x = toLONG(oX + m_h()        ); m_pRegion[4].y = toLONG(oY - b                );
      m_pRegion[5].x = toLONG(oX + m_h()        ); m_pRegion[5].y = toLONG(oY - m_a()-b          );
      break;
   case 5:
      m_pRegion[0].x = toLONG(oX + b            ); m_pRegion[0].y = toLONG(oY - m_a()-m_a()-m_h());
      m_pRegion[1].x = toLONG(oX                ); m_pRegion[1].y = toLONG(oY - m_a()-m_a()      );
      m_pRegion[2].x = toLONG(oX + m_h()        ); m_pRegion[2].y = toLONG(oY - m_a()-b          );
      m_pRegion[3].x = toLONG(oX + b+m_h()      ); m_pRegion[3].y = toLONG(oY - m_a()-b-m_h()    );
      break;
   case 6:
      m_pRegion[0].x = toLONG(oX - m_h()        ); m_pRegion[0].y = toLONG(oY - b                );
      m_pRegion[1].x = toLONG(oX - m_a()-m_h()  ); m_pRegion[1].y = toLONG(oY - b                );
      m_pRegion[2].x = toLONG(oX - b-m_h()      ); m_pRegion[2].y = toLONG(oY - b+m_h()          );
      break;
   case 7:
      m_pRegion[0].x = toLONG(oX                ); m_pRegion[0].y = toLONG(oY                    );
      m_pRegion[1].x = toLONG(oX + b            ); m_pRegion[1].y = toLONG(oY + m_h()            );
      m_pRegion[2].x = toLONG(oX - b            ); m_pRegion[2].y = toLONG(oY + m_h()            );
      break;
   case 8:
      m_pRegion[0].x = toLONG(oX + m_h()        ); m_pRegion[0].y = toLONG(oY - b                );
      m_pRegion[1].x = toLONG(oX                ); m_pRegion[1].y = toLONG(oY                    );
      m_pRegion[2].x = toLONG(oX + b            ); m_pRegion[2].y = toLONG(oY + m_h()            );
      m_pRegion[3].x = toLONG(oX + m_h()+b      ); m_pRegion[3].y = toLONG(oY - b+m_h()          );
      break;
   case 9:
      m_pRegion[0].x = toLONG(oX - m_h()        ); m_pRegion[0].y = toLONG(oY - b                );
      m_pRegion[1].x = toLONG(oX - b-m_h()      ); m_pRegion[1].y = toLONG(oY - b+m_h()          );
      m_pRegion[2].x = toLONG(oX - b            ); m_pRegion[2].y = toLONG(oY + m_h()            );
      m_pRegion[3].x = toLONG(oX                ); m_pRegion[3].y = toLONG(oY                    );
      break;
   case 10:
      m_pRegion[0].x = toLONG(oX + b            ); m_pRegion[0].y = toLONG(oY + m_h()            );
      m_pRegion[1].x = toLONG(oX - b            ); m_pRegion[1].y = toLONG(oY + m_h()            );
      m_pRegion[2].x = toLONG(oX - b            ); m_pRegion[2].y = toLONG(oY + m_a()+m_h()      );
      m_pRegion[3].x = toLONG(oX + b            ); m_pRegion[3].y = toLONG(oY + m_a()+m_h()      );
      break;
   case 11:
      m_pRegion[0].x = toLONG(oX + b+m_h()      ); m_pRegion[0].y = toLONG(oY + m_h()-b          );
      m_pRegion[1].x = toLONG(oX + b            ); m_pRegion[1].y = toLONG(oY + m_h()            );
      m_pRegion[2].x = toLONG(oX + b            ); m_pRegion[2].y = toLONG(oY + m_a()+m_h()      );
      m_pRegion[3].x = toLONG(oX + b+m_h()      ); m_pRegion[3].y = toLONG(oY + m_a()+b+m_h()    );
      m_pRegion[4].x = toLONG(oX + b+m_h()+m_h()); m_pRegion[4].y = toLONG(oY + m_a()+m_h()      );
      m_pRegion[5].x = toLONG(oX + b+m_h()+m_h()); m_pRegion[5].y = toLONG(oY + m_h()            );
      break;
   }

   //       -     /
   {
      POINTFLOAT center; //     
      switch (m_iDirection) {
      case  0:  center.x = oX -  b-m_h();    center.y = oY - m_a()-b-m_GContext.m_Border.m_iWidth-m_sq()/2;            break;
      case  1:  center.x = oX - (b+m_h())/2; center.y = oY - m_a()-b-(b+m_h())/2;                                      break;
      case  2:  center.x = oX;               center.y = oY - m_a()-m_a()-m_h()+m_GContext.m_Border.m_iWidth+m_sq()/2;  break;
      case  3:  center.x = oX -  b-m_h();    center.y = oY - m_a();                                                    break;
      case  4:  center.x = oX;               center.y = oY - m_a();                                                    break;
      case  5:  center.x = oX + (b+m_h())/2; center.y = oY - m_a()-b-(b+m_h())/2;                                      break;
      case  6:  center.x = oX -  b-m_h();    center.y = oY - b  +m_GContext.m_Border.m_iWidth+m_sq()/2;                break;
      case  7:  center.x = oX;               center.y = oY + m_h()-m_GContext.m_Border.m_iWidth-m_sq()/2;              break;
      case  8:  center.x = oX + (b+m_h())/2; center.y = oY - b+(b+m_h())/2;                                            break;
      case  9:  center.x = oX - (b+m_h())/2; center.y = oY - b+(b+m_h())/2;                                            break;
      case 10:  center.x = oX;               center.y = oY + b+m_h();                                                  break;
      case 11:  center.x = oX +  b+m_h();    center.y = oY + b+m_h();                                                  break;
      }
      m_Square.left   = toLONG(center.x - m_sq()/2);
      m_Square.top    = toLONG(center.y - m_sq()/2);
      m_Square.right  = toLONG(m_Square.left + m_sq());
      m_Square.bottom = toLONG(m_Square.top  + m_sq());
   }
   CCell::SetPoint(iArea);
}

void CSqTrHex::Paint() const {
   CCell::Paint();

   switch (m_iDirection) {
   case 0: case  7:
      SelectObject(m_GContext.m_hDCTmp, m_bDown ? m_GContext.m_hPenLight  : m_GContext.m_hPenShadow);
      MoveToEx(m_GContext.m_hDCTmp, m_pRegion[0].x, m_pRegion[0].y, NULL);
      LineTo  (m_GContext.m_hDCTmp, m_pRegion[1].x, m_pRegion[1].y);
      break;
   case 2: case  6:
      SelectObject(m_GContext.m_hDCTmp, m_bDown ? m_GContext.m_hPenShadow : m_GContext.m_hPenLight );
      MoveToEx(m_GContext.m_hDCTmp, m_pRegion[0].x, m_pRegion[0].y, NULL);
      LineTo  (m_GContext.m_hDCTmp, m_pRegion[1].x, m_pRegion[1].y);
      break;
   case 1: case  8:
   case 3: case 10:
   case 4: case 11:
   case 5: case  9:
      SelectObject(m_GContext.m_hDCTmp, m_bDown ? m_GContext.m_hPenShadow : m_GContext.m_hPenLight );
      MoveToEx(m_GContext.m_hDCTmp, m_pRegion[1].x, m_pRegion[1].y, NULL);
      break;
   }
   LineTo  (m_GContext.m_hDCTmp, m_pRegion[2].x, m_pRegion[2].y);
   switch (m_iDirection) {
   case 4: case 11:
      LineTo  (m_GContext.m_hDCTmp, m_pRegion[3].x, m_pRegion[3].y);  break;
   }
   switch (m_iDirection) {
   case 0: case  7:
      SelectObject(m_GContext.m_hDCTmp, m_bDown ? m_GContext.m_hPenShadow : m_GContext.m_hPenLight ); break;
   case 1: case  8:
   case 3: case 10:
   case 4: case 11:
   case 5: case  9:
   case 2: case  6:
      SelectObject(m_GContext.m_hDCTmp, m_bDown ? m_GContext.m_hPenLight  : m_GContext.m_hPenShadow); break;
   }
   switch (m_iDirection) {
   case 0: case  7:
   case 2: case  6:
      LineTo  (m_GContext.m_hDCTmp, m_pRegion[0].x, m_pRegion[0].y);
      ::BitBltMasked(
         m_GContext.m_hDCDst, GetOuterRect(),
         m_GContext.m_hDCTmp, GetOuterRect().pointLT(),
         m_pRegion, 3, GetOuterRect().pointLT());
      break;
   case 1: case  8:
   case 3: case 10:
   case 5: case  9:
      LineTo  (m_GContext.m_hDCTmp, m_pRegion[3].x, m_pRegion[3].y);
      LineTo  (m_GContext.m_hDCTmp, m_pRegion[0].x, m_pRegion[0].y);
      SelectObject(m_GContext.m_hDCTmp, m_bDown ? m_GContext.m_hPenShadow : m_GContext.m_hPenLight );
      LineTo  (m_GContext.m_hDCTmp, m_pRegion[1].x, m_pRegion[1].y);
      ::BitBltMasked(
         m_GContext.m_hDCDst, GetOuterRect(),
         m_GContext.m_hDCTmp, GetOuterRect().pointLT(),
         m_pRegion, 4, GetOuterRect().pointLT());
      break;
   case 4: case 11:
      LineTo  (m_GContext.m_hDCTmp, m_pRegion[4].x, m_pRegion[4].y);
      LineTo  (m_GContext.m_hDCTmp, m_pRegion[5].x, m_pRegion[5].y);
      LineTo  (m_GContext.m_hDCTmp, m_pRegion[0].x, m_pRegion[0].y);
      SelectObject(m_GContext.m_hDCTmp, m_bDown ? m_GContext.m_hPenShadow : m_GContext.m_hPenLight );
      LineTo  (m_GContext.m_hDCTmp, m_pRegion[1].x, m_pRegion[1].y);
      ::BitBltMasked(
         m_GContext.m_hDCDst, GetOuterRect(),
         m_GContext.m_hDCTmp, GetOuterRect().pointLT(),
         m_pRegion, 6, GetOuterRect().pointLT());
      break;
   }
}
