////////////////////////////////////////////////////////////////////////////////
//                               FastMines project
//                                       Sergey Krivulya (KSerg, aka SeregaLBN)
// file name: "Square2.cpp"
//
//   CSquare2 -  (  )
// Copyright (C) 2002-2007 Sergey Krivulya
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
////////////////////////////////////////////////////////////////////////////////

#include "StdAfx.h"
#include "Square2.h"

////////////////////////////////////////////////////////////////////////////////
//                               static function
////////////////////////////////////////////////////////////////////////////////

SIZE CSquare2::GetSizeInPixel(const SIZE &sizeField, int iArea) {
   m_a() = sqrt((float)iArea); //   
   SIZE result = {toLONG(sizeField.cx * m_a() + m_a()/2),
                  toLONG(sizeField.cy * m_a()          )};
   return result;
}

int CSquare2::SizeInscribedSquare(int iArea, int iBorderWidth) {
   m_sq() = sqrt((float)iArea)-2*iBorderWidth;
   return toINT(m_sq());
}

////////////////////////////////////////////////////////////////////////////////
//                               virtual function
////////////////////////////////////////////////////////////////////////////////

CSquare2::CSquare2(const COORD &Coord, const SIZE &sizeField, int iArea, const CGraphicContext &gContext)
   : CCell(Coord, sizeField, iArea, gContext,
           6, 4,
           Coord.Y&1 // 0..1
          )
{
   SetPoint(iArea);
   //   
   m_pNeighbor[0].X = m_Coord.X-(m_iDirection?1:0); m_pNeighbor[0].Y = m_Coord.Y-1;
   m_pNeighbor[1].X = m_Coord.X+(m_iDirection?0:1); m_pNeighbor[1].Y = m_Coord.Y-1;
   m_pNeighbor[2].X = m_Coord.X-1;                  m_pNeighbor[2].Y = m_Coord.Y;
   m_pNeighbor[3].X = m_Coord.X+1;                  m_pNeighbor[3].Y = m_Coord.Y;
   m_pNeighbor[4].X = m_Coord.X-(m_iDirection?1:0); m_pNeighbor[4].Y = m_Coord.Y+1;
   m_pNeighbor[5].X = m_Coord.X+(m_iDirection?0:1); m_pNeighbor[5].Y = m_Coord.Y+1;
   VerifyNeighbor(sizeField);
}

bool CSquare2::PointInRegion(const POINT &point) const {
   if ((point.x < m_pRegion[1].x) || (point.x >= m_pRegion[0].x) ||
       (point.y < m_pRegion[0].y) || (point.y >= m_pRegion[2].y))
      return false;
   return true;
}

void CSquare2::SetPoint(int iArea) {
   if (m_Coord.X==0 && m_Coord.Y==0) {
      m_a()  = sqrt((float)iArea); //   
      m_sq() = m_a()-2*m_GContext.m_Border.m_iWidth;      //  ,   
   }

   //    
   m_pRegion[1].x = m_pRegion[2].x = toLONG(m_a()*(m_Coord.X+0) + (m_iDirection ? 0 : m_a()/2));
   m_pRegion[0].x = m_pRegion[3].x = toLONG(m_a()*(m_Coord.X+1) + (m_iDirection ? 0 : m_a()/2));

   m_pRegion[0].y = m_pRegion[1].y = toLONG(m_a()*(m_Coord.Y+0));
   m_pRegion[2].y = m_pRegion[3].y = toLONG(m_a()*(m_Coord.Y+1));

   //       -     /
   m_Square.left   = toLONG(m_pRegion[1].x + m_GContext.m_Border.m_iWidth);
   m_Square.top    = toLONG(m_pRegion[1].y + m_GContext.m_Border.m_iWidth);
   m_Square.right  = toLONG(m_Square.left + m_sq());
   m_Square.bottom = toLONG(m_Square.top  + m_sq());

   CCell::SetPoint(iArea);
}

void CSquare2::Paint() const {
   CCell::Paint();

   SelectObject(m_GContext.m_hDCTmp, m_bDown ? m_GContext.m_hPenShadow : m_GContext.m_hPenLight );
   MoveToEx(m_GContext.m_hDCTmp, m_pRegion[1].x, m_pRegion[1].y, NULL);
   LineTo  (m_GContext.m_hDCTmp, m_pRegion[2].x, m_pRegion[2].y);
   SelectObject(m_GContext.m_hDCTmp, m_bDown ? m_GContext.m_hPenLight  : m_GContext.m_hPenShadow);
   LineTo  (m_GContext.m_hDCTmp, m_pRegion[3].x, m_pRegion[3].y);
   LineTo  (m_GContext.m_hDCTmp, m_pRegion[0].x, m_pRegion[0].y);
   SelectObject(m_GContext.m_hDCTmp, m_bDown ? m_GContext.m_hPenShadow : m_GContext.m_hPenLight );
   LineTo  (m_GContext.m_hDCTmp, m_pRegion[1].x, m_pRegion[1].y);

   ::BitBltMasked(
      m_GContext.m_hDCDst, GetOuterRect(),
      m_GContext.m_hDCTmp, GetOuterRect().pointLT(),
      m_pRegion, 4, GetOuterRect().pointLT());
}
