////////////////////////////////////////////////////////////////////////////////
//                               FastMines project
//                                       Sergey Krivulya (KSerg, aka SeregaLBN)
// file name: "MenuItem.cpp"
// Copyright (C) 2006-2007 Sergey Krivulya
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
////////////////////////////////////////////////////////////////////////////////

#include "StdAfx.h"
#include "MenuItem.h"

#define OFFSET_CHECK_X 0
#define OFFSET_CHECK_Y 0
#define OFFSET_TEXT_X  6
#define OFFSET_TAB     6 //       ' '

CMenuItem::CMenuItem(
   BOOL bPopup,
   HMENU hMenu,
   UINT uItem,
   BOOL fByPosition,
   const CImage *pImg
) :
   m_dwSignature(CMenuItem::GetSignature()),
   m_bPopup     (bPopup),
   m_uWidthCheck(0),
   m_uArrow     (0),
   m_hMenu      (hMenu),
   m_uItem      (uItem),
   m_fByPosition(fByPosition)
{
   if (pImg) {
      m_Img = *pImg;
   }

   Attach();

   /** /
   MENUITEMINFO mii = {sizeof(MENUITEMINFO), MIIM_BITMAP};
   BOOL bRes = ::GetMenuItemInfo(hMenu, uItem, fByPosition, &mii);
   if (bRes) {
      if (mii.hbmpItem &&
         (mii.hbmpItem >= HBMMENU_SYSTEM) &&
         (mii.hbmpItem <= HBMMENU_POPUP_MINIMIZE))
      {
         mii.hbmpItem = HBMMENU_CALLBACK; // A bitmap that is drawn by the window that owns the menu. The application must process the WM_MEASUREITEM and WM_DRAWITEM messages.
         bRes = ::SetMenuItemInfo(hMenu, uItem, fByPosition, &mii);
      }
   }
   /**/
}

/* static */ BOOL CMenuItem::CheckSign(CMenuItem *This) {
   if (This == NULL) return FALSE;
#ifdef _DEBUG
   BOOL bRes = (This->m_dwSignature == CMenuItem::GetSignature());
   if (!bRes) alert(_T("Invalid CMenuItem signature!"));
   return bRes;
#else
   return (This->m_dwSignature == CMenuItem::GetSignature());
#endif
}

/* static */ CMenuItem* CMenuItem::GetInstance(HMENU hMenu, UINT uItem, BOOL bByPosition) {
   CMenuItem *This = NULL;
   MENUITEMINFO mii = {sizeof(MENUITEMINFO), MIIM_TYPE | MIIM_DATA};
   BOOL bRes = ::GetMenuItemInfo(hMenu, uItem, bByPosition, &mii);
   if (bRes) {
      if (mii.fType & MFT_OWNERDRAW) {
         This = (CMenuItem *)mii.dwItemData;
         if (!CMenuItem::CheckSign(This)) This = NULL;
      }
   }
   return This;
}

/* static */ BOOL CMenuItem::SetMenuString(HMENU hMenu, UINT uItem, BOOL bByPosition, LPCTSTR szText) {
   MENUITEMINFO mii = {sizeof(MENUITEMINFO), MIIM_TYPE | MIIM_DATA};
   BOOL bRes = ::GetMenuItemInfo(hMenu, uItem, bByPosition, &mii);
   if (bRes) {
      if (mii.fType & MFT_OWNERDRAW) {
         CMenuItem *This = (CMenuItem*)mii.dwItemData;
         if (CMenuItem::CheckSign(This)) {
            bRes = ::SetMenuString(hMenu, uItem, bByPosition, szText);
            bRes &= This->Attach();
         } else {
            bRes = FALSE;         
         }
      } else {
         bRes = ::SetMenuString(hMenu, uItem, bByPosition, szText);
      #ifdef _DEBUG
         alert(_T("    ?      OWNERDRAW...\r\nMenu text - ")+CString(szText));
      #endif
      }
   }
   return bRes;
}


//       OWNERDRAW
BOOL CMenuItem::Attach() {
   MENUITEMINFO mii = {sizeof(MENUITEMINFO), MIIM_TYPE | MIIM_DATA, MFT_OWNERDRAW, 0,0,0,0,0, (ULONG_PTR)this, 0,0};
   BOOL bRes = ::SetMenuItemInfo(m_hMenu, m_uItem, m_fByPosition, &mii);
   return bRes;
}
//   OWNERDRAW   
BOOL CMenuItem::Detach() {
   BOOL bRes = FALSE;
   if (::IsMenu(m_hMenu)) {
      //bRes = ::SetMenuString(m_hMenu, m_uItem, m_fByPosition, GetText());
      MENUITEMINFO mii = {sizeof(MENUITEMINFO), MIIM_TYPE, MFT_STRING, 0,0,0,0,0, NULL, 0,0};
      bRes = ::SetMenuItemInfo(m_hMenu, m_uItem, m_fByPosition, &mii);
   }
   return bRes;
}

// WM_MEASUREITEM
/* static */ void CMenuItem::OnMeasureItem(MEASUREITEMSTRUCT *lpMeasureItem) {
   if (!lpMeasureItem) return;
   CMenuItem *This = (CMenuItem*)lpMeasureItem->itemData;
   if (CMenuItem::CheckSign(This))
      This->OnMeasureItem(*lpMeasureItem);
}
void CMenuItem::OnMeasureItem(MEASUREITEMSTRUCT &mi) {
   if (mi.CtlType != ODT_MENU) // error
      return;

   NONCLIENTMETRICS ncm = {sizeof(ncm)};
   ::SystemParametersInfo(SPI_GETNONCLIENTMETRICS, ncm.cbSize, (PVOID)&ncm, 0);

   m_uWidthCheck = (!m_bPopup && !m_Img.Valid()) ? 0 : ncm.iMenuWidth+2*OFFSET_CHECK_X;
   m_uArrow      =  !m_bPopup ? 0 : ncm.iMenuWidth;

   bool isSeparator = ((m_fByPosition ? ::GetMenuItemID(m_hMenu, m_uItem) : m_uItem) == 0) && GetText().IsEmpty();

   if (isSeparator) {
      mi.itemWidth  = 0;
      mi.itemHeight = 4;
   } else {
      HDC hCDC = ::CreateCompatibleDC(NULL);

      LOGFONT font = ncm.lfMenuFont;
      BOOL bFont = GetLogFont(font);
      if (bFont) font.lfHeight = ncm.lfMenuFont.lfHeight;
      PLOGFONT pFont = bFont ? &font : &ncm.lfMenuFont;
      HFONT   hFontNew = ::CreateFontIndirect(pFont);
      HGDIOBJ hFontOld = ::SelectObject(hCDC, hFontNew);

      SIZEEX sizeText;
      CString strText = GetText();
      ::GetTextExtentPoint32(hCDC, strText, strText.GetLength(), &sizeText);

      UINT uHeightItem = max(ncm.iMenuHeight, /*abs*/(pFont->lfHeight))+2*OFFSET_CHECK_Y;

      mi.itemWidth  = m_uWidthCheck + sizeText.cx + ((strText.Find(_T('\t'))==-1) ? 0 : OFFSET_TAB) + m_uArrow;
      mi.itemHeight = uHeightItem; //::GetSystemMetrics(SM_CYMENU)//2+16+2//::GetSystemMetrics(SM_CYMENUCHECK)+5//**/abs(fontData.lfHeight)*/

      ::SelectObject(hCDC, hFontOld);
      ::DeleteObject(hFontNew);
      ::DeleteDC(hCDC);
   }

   //LOG_PUT(g_pLog, Put, LL_DEBUG) _T("  OnMeasureItem: m_uWidthCheck=%i, m_uArrow=%i, mi.itemWidth=%i, mi.itemHeight=%i, text=[%s]"), m_uWidthCheck, m_uArrow, mi.itemWidth, mi.itemHeight, (LPCTSTR)GetText());
}

// WM_DRAWITEM
/* static */ void CMenuItem::OnDrawItem(const DRAWITEMSTRUCT *lpDrawItem) {
   if (!lpDrawItem) return;
   CMenuItem *This = (CMenuItem*)lpDrawItem->itemData;
   if (CMenuItem::CheckSign(This))
      This->OnDrawItem(*lpDrawItem);
}
void CMenuItem::OnDrawItem(const DRAWITEMSTRUCT &di) {
   if (di.CtlType != ODT_MENU) // error
      return;

   NONCLIENTMETRICS ncm = {sizeof(ncm)};
   ::SystemParametersInfo(SPI_GETNONCLIENTMETRICS, ncm.cbSize, (PVOID)&ncm, 0);

   if (m_uWidthCheck == 0) { // verify
      m_uWidthCheck = (!m_bPopup && !m_Img.Valid()) ? 0 : ncm.iMenuWidth+2*OFFSET_CHECK_X;
   }

   //LOG_PUT(g_pLog, Put, LL_DEBUG) _T("  OnDrawItem: di.itemID=%i  di.itemAction=[%s] di.itemState=[%s] "), di.itemID, (LPCTSTR)CLogger::OwnerDrawItemAction(di.itemAction), (LPCTSTR)CLogger::OwnerDrawItemState(di.itemState));

   // rcItem -  item'a
   RECTEX rcItem(di.rcItem);

   HDC     hCDC    = ::CreateCompatibleDC(di.hDC);
   HBITMAP hBmp    = ::CreateCompatibleBitmap(di.hDC, rcItem.width(), rcItem.height());
   HGDIOBJ hBMPOld = ::SelectObject(hCDC, hBmp);

   //       .
   //          ODS_CHECKED.

   // rcBtn -  
   RECTEX rcBtn(+OFFSET_CHECK_X,
                +OFFSET_CHECK_Y,
                -OFFSET_CHECK_X+m_uWidthCheck,
                -OFFSET_CHECK_Y+rcItem.height());

   // rcTxt -       
   RECTEX rcTxt(m_uWidthCheck+OFFSET_TEXT_X,
                0,
                rcItem.width()-m_uArrow,
                rcItem.height());

   // rcFill -  
   RECTEX rcFill(rcItem.size());

   bool isCheck  = !!(di.itemState & ODS_CHECKED); //    ?
   bool isBullet = false; //      "Bullet'" ?
   if (isCheck) {
      //    ,    -  "Check mark'"   "Bullet'" ?
      MENUITEMINFO mii = {sizeof(MENUITEMINFO), MIIM_TYPE};
      BOOL bRes = ::GetMenuItemInfo((HMENU)di.hwndItem, di.itemID, FALSE, &mii);
      isBullet = !!(mii.fType & MFT_RADIOCHECK);
   }
   bool isSelect    = !!(di.itemState & ODS_SELECTED);
   bool isGrayed    = !!(di.itemState & ODS_GRAYED);
 //bool isArrow     = ((m_fByPosition ? ::GetMenuItemID(m_hMenu, m_uItem) : m_uItem) == -1); // Submenu arrow
   bool isSeparator = ((m_fByPosition ? ::GetMenuItemID(m_hMenu, m_uItem) : m_uItem) == 0) && GetText().IsEmpty();


   // fill menu item
   if (!DrawBkgrnd(di, hCDC, rcFill, isSelect, isGrayed, isSeparator)) {
      if (isSelect && !isSeparator) {
         ::SelectObject(hCDC, ::GetSysColorBrush(COLOR_HIGHLIGHT));
         ::PatBlt(hCDC, rcFill.left, rcFill.top, rcFill.width(), rcFill.height(), PATCOPY);
      } else {
         ::SelectObject(hCDC, ::GetSysColorBrush(COLOR_MENU));
         ::PatBlt(hCDC, rcFill.left, rcFill.top, rcFill.width(), rcFill.height(), PATCOPY);
      }
      if (m_Img.Valid()) {
         ::SelectObject(hCDC, ::GetSysColorBrush(COLOR_MENU));
         ::PatBlt(hCDC, rcBtn.left, rcBtn.top, rcBtn.width(), rcBtn.height(), PATCOPY);
      }
   }

   if (isSeparator) {
      // draw Separator
      HPEN    hPenNew = ::CreatePen(PS_SOLID, 1, ::GetSysColor(COLOR_GRAYTEXT));
      HGDIOBJ hPenOld = ::SelectObject(hCDC, hPenNew);
      ::MoveToEx(hCDC, /*m_uWidthCheck+*/1, 1, NULL);
      ::LineTo  (hCDC, rcItem.right-1, 1);
      ::SelectObject(hCDC, ::GetStockObject(WHITE_PEN));
      ::MoveToEx(hCDC, /*m_uWidthCheck+*/1, 2, NULL);
      ::LineTo  (hCDC, rcItem.right-1, 2);
      ::SelectObject(hCDC, hPenOld);
      ::DeleteObject(hPenNew);
   } else {
      // draw image & check
      if (m_Img.Valid()) {
         // image
         if (isSelect) {
            if (isCheck)
               rcBtn.move(+1,+1);
            else
               rcBtn.move(-1,-1);
         }
         m_Img.Draw(hCDC, rcBtn);
         // button
         if (/*isSelect || */isCheck) {
            ::SelectObject(hCDC, ::GetStockObject(isCheck ? BLACK_PEN : WHITE_PEN));
            ::MoveToEx(hCDC, rcBtn.left   , rcBtn.bottom-1, NULL);
            ::LineTo  (hCDC, rcBtn.left   , rcBtn.top     );
            ::LineTo  (hCDC, rcBtn.right-1, rcBtn.top     );
            ::SelectObject(hCDC, ::GetStockObject(isCheck ? WHITE_PEN : BLACK_PEN));
            ::LineTo  (hCDC, rcBtn.right-1, rcBtn.bottom-1);
            ::LineTo  (hCDC, rcBtn.left   , rcBtn.bottom-1);
         }
      } else {
         HBITMAP hBmp = NULL;
         MENUITEMINFO mii = {sizeof(MENUITEMINFO), MIIM_BITMAP};
         BOOL bRes2 = ::GetMenuItemInfo(m_hMenu, m_uItem, m_fByPosition, &mii);
         //LOG_PUT(g_pLog, Put, LL_DEBUG) _T("  OnDrawItem: bRes2=%i, mii.hbmpItem=0x%08X, text=[%s]"), bRes2, mii.hbmpItem, (LPCTSTR)GetText());
         WORD iOEMRes = 0; // OEM Resource Ordinal Numbers
         if (bRes2 && mii.hbmpItem) {
            hBmp = mii.hbmpItem;
            switch ((LONG_PTR)hBmp) {
            case HBMMENU_POPUP_CLOSE   : iOEMRes = OBM_CLOSE; break; //    ?    #define OEMRESOURCE   
            case HBMMENU_POPUP_RESTORE : iOEMRes = isSelect ? OBM_RESTORED : OBM_RESTORE; break;
            case HBMMENU_POPUP_MINIMIZE: iOEMRes = isSelect ? OBM_REDUCED  : OBM_REDUCE ; break;
            case HBMMENU_POPUP_MAXIMIZE: iOEMRes = isSelect ? OBM_ZOOMD    : OBM_ZOOM   ; break;
            }
            if (iOEMRes) {
               hBmp = ::LoadBitmap(NULL, MAKEINTRESOURCE(iOEMRes));
            }
            // valid bitmap?
            BITMAP bmp = {0,0,0,0,0,0,NULL};
            int iRes = ::GetObject(hBmp, sizeof(BITMAP), &bmp);
            if (iRes != sizeof(BITMAP))
               hBmp = NULL;
         }

         if (hBmp) {
            if (isSelect) {
               if (isCheck)
                  rcBtn.move(+1,+1);
               else
                  rcBtn.move(+1,+1);
            }
            if (iOEMRes != OBM_CLOSE) {
               ::StretchBltMasked(hCDC, rcBtn, hBmp, RECTEX(SizeBitmap(hBmp)), ::GetSysColor(COLOR_BTNFACE));
            } else {
               HDC hCDCTmp = ::CreateCompatibleDC(di.hDC);
               HBITMAP hBmpTmp = ::CreateCompatibleBitmap(di.hDC, rcBtn.width(), rcBtn.height());
               HGDIOBJ hBmpOld = ::SelectObject(hCDCTmp, hBmpTmp);
               ::DrawFrameControl(hCDCTmp, &rcBtn, DFC_CAPTION, DFCS_CAPTIONCLOSE | (isSelect ? DFCS_FLAT : 0));
               ::SelectObject(hCDCTmp, hBmpOld);
               ::StretchBltMasked(hCDC, rcBtn, hBmpTmp, rcBtn, ::GetSysColor(COLOR_BTNFACE));
               ::DeleteObject(hBmpTmp);
               ::DeleteDC(hCDCTmp);
            }
         } else {
            if (isCheck && !!m_uWidthCheck) {
               /** /
               ::DrawFrameControl( //    
                  hCDC,
                  &rcBtn,
                  DFC_MENU,
                  isBullet ? DFCS_MENUBULLET : DFCS_MENUCHECK
               );/**/
               if (isBullet) {
                  ::SelectObject(hCDC, ::GetStockObject(isSelect ? WHITE_BRUSH : BLACK_BRUSH));
                  ::SelectObject(hCDC, ::GetStockObject(NULL_PEN));
                  POINT p = rcBtn.center();
                  ::Ellipse(hCDC, p.x-4, p.y-4, p.x+4, p.y+4);
               } else {
                  //::SelectObject(hCDC, ::GetStockObject(isSelect ? WHITE_PEN : BLACK_PEN));
                  POINTEX p = rcBtn.center();
                  HPEN    hPenNew = ::CreatePen(PS_SOLID, 2, isSelect ? 0xFFFFFF : 0x000000);
                  HGDIOBJ hPenOld = ::SelectObject(hCDC, hPenNew);
                  p += POINTEX(-5, 0); ::MoveToEx(hCDC, p.x, p.y, NULL);
                  p += POINTEX( 3, 4); ::LineTo  (hCDC, p.x, p.y);
                  p += POINTEX( 6,-6); ::LineTo  (hCDC, p.x, p.y);
                  ::SelectObject(hCDC, hPenOld);
                  ::DeleteObject(hPenNew);
               }
            }
         }
      }

      CString strText = GetText();
      // draw text
      if (!strText.IsEmpty()) {
         LOGFONT font;
         BOOL bFont = GetLogFont(font);
         if (bFont) font.lfHeight = ncm.lfMenuFont.lfHeight;
         PLOGFONT pFont = bFont ? &font : &ncm.lfMenuFont;
         HFONT   hFontNew = ::CreateFontIndirect(pFont);
         HGDIOBJ hFontOld = ::SelectObject(hCDC, hFontNew);

         CString strTab;
         int iPos = strText.Find(_T('\t'));
         if (iPos >= 0) {
            strTab  = strText.Mid(iPos+1);
            strText = strText.Left(iPos);
         }

         ::SetBkMode(hCDC, TRANSPARENT);
         if (isGrayed) {
            if (isSelect){
               ::SetTextColor(hCDC, ::GetSysColor(COLOR_GRAYTEXT));
               ::DrawText(hCDC, strText, strText.GetLength(), &rcTxt, DT_LEFT | DT_VCENTER | DT_SINGLELINE | DT_EXPANDTABS);
               if (iPos >= 0) {
                  ::DrawText(hCDC, strTab, strTab.GetLength(), &rcTxt, DT_RIGHT | DT_VCENTER | DT_SINGLELINE | DT_EXPANDTABS);
               }
            } else {
               ::SetTextColor(hCDC, 0xFFFFFF);
               ::DrawText(hCDC, strText, strText.GetLength(), &rcTxt, DT_LEFT | DT_VCENTER | DT_SINGLELINE | DT_EXPANDTABS);
               if (iPos >= 0) {
                  ::DrawText(hCDC, strTab, strTab.GetLength(), &rcTxt, DT_RIGHT | DT_VCENTER | DT_SINGLELINE | DT_EXPANDTABS);
               }
               
               rcTxt.move(-1,-1);
               ::SetTextColor(hCDC, ::GetSysColor(COLOR_GRAYTEXT));
               ::DrawText(hCDC, strText, strText.GetLength(), &rcTxt, DT_LEFT | DT_VCENTER | DT_SINGLELINE | DT_EXPANDTABS);
               if (iPos >= 0) {
                  ::DrawText(hCDC, strTab, strTab.GetLength(), &rcTxt, DT_RIGHT | DT_VCENTER | DT_SINGLELINE | DT_EXPANDTABS);
               }
            }
         } else {
            ::SetTextColor(hCDC, ::GetSysColor(isSelect ? COLOR_HIGHLIGHTTEXT : COLOR_MENUTEXT));
            ::DrawText(hCDC, strText, strText.GetLength(), &rcTxt, DT_LEFT | DT_VCENTER | DT_SINGLELINE | DT_EXPANDTABS);
            if (iPos >= 0) {
               ::DrawText(hCDC, strTab, strTab.GetLength(), &rcTxt, DT_RIGHT | DT_VCENTER | DT_SINGLELINE | DT_EXPANDTABS);
            }
         }
         // erasing
         ::SelectObject(hCDC, hFontOld);
         ::DeleteObject(hFontNew);
      }

   }

   ::BitBlt(di.hDC, di.rcItem, hCDC, SIZEEX(), SRCCOPY);

   // erasing
   ::SelectObject(hCDC, hBMPOld);
   ::DeleteDC(hCDC);
   ::DeleteObject(hBmp);

   PostDraw(di, hCDC, rcFill, isSelect, isGrayed, isSeparator);

   //HBITMAP hBMP__ = ::GetMenuItemBitmap(m_hMenu, m_uItem, m_fByPosition);
   //SIZEEX s = ::SizeBitmap(hBMP__);
   //LOG_PUT(g_pLog, Put, LL_DEBUG) _T("  OnDrawItem: m_uWidthCheck=%i, m_uArrow=%i, di.rcItem=[%i, %i, w%i, h%i] tex=[%s] HBITMAP=0x%08X size=[%i, %i]"), m_uWidthCheck, m_uArrow, di.rcItem.left, di.rcItem.top, di.rcItem.right-di.rcItem.left, di.rcItem.bottom-di.rcItem.top, (LPCTSTR)GetText(), hBMP__, s.cx, s.cy);

   return;

   /** /
   static const DWORD dwROP=0x00A000C9L;
   static const WORD  bmpMASK[] = {0x55, 0xaa, 0x55, 0xaa, 0x55, 0xaa, 0x55, 0xaa};

   static HBITMAP hBmp   = ::CreateBitmap(8, 8, 1, 1, bmpMASK);
   static HBRUSH  hBR    = ::CreatePatternBrush(hBmp);
   HGDIOBJ oldhBR = ::SelectObject(di.hDC, hBR);

   COLORREF oldclrText = ::SetTextColor(di.hDC, RGB(190, 190, 190));
   COLORREF oldclrBkgr = SetBkColor(di.hDC, RGB(190, 190, 190));

   if (di.rcItem.top==0)
      ::PatBlt(di.hDC, di.rcItem.left, di.rcItem.top, RECTEX(di.rcItem).width(), RECTEX(di.rcItem).height(), dwROP);
   /**/
}
