////////////////////////////////////////////////////////////////////////////////
//                               FastMines project
//                                       Sergey Krivulya (KSerg, aka SeregaLBN)
// file name: "MenuItem.h"
// Copyright (C) 2006-2007 Sergey Krivulya
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
////////////////////////////////////////////////////////////////////////////////

#ifndef __FILE__CMENUITEM__
#define __FILE__CMENUITEM__

#include "../Image.h"

/** /
 :
1.    .
2.  CMenuItem,    
3.     (- SetText  SetMenuString()).
       WM_MEASUREITEM.
4.   WM_MEASUREITEM  WM_DRAWITEM 
    OnMeasureItem   OnDrawItem .
/**/

class CMenuItem {
private:
   const DWORD m_dwSignature;
   static DWORD GetSignature() { return 0xB6C85BEC; }
   //   widthCheck           text            arrow
   //  <--------><-------------------------><----->
   //   ____________________...____________________
   //  |         |                 |        |     | /|\
   //  |  Image  |                          | |\  |  |
   //  |   or    |     szText      | szAcel | |:\ |  | heightItem
   //  |  check  |                          | |:/ |  |
   //  | (radio) |                 |        | |/  |  |
   //  |_________|__________..._____________|_____| \|/

protected:
   HMENU   m_hMenu;
   UINT    m_uItem;
   BOOL    m_fByPosition;

   BOOL    m_bPopup;
   UINT    m_uWidthCheck,
           m_uArrow;
public:
   CImage  m_Img;

private:
   void OnMeasureItem(MEASUREITEMSTRUCT&); // WM_MEASUREITEM
   void OnDrawItem(const DRAWITEMSTRUCT&); // WM_DRAWITEM

public:
   CMenuItem(BOOL bPopup, HMENU hMenu, UINT uItem, BOOL fByPosition, const CImage *pImg = NULL);
  ~CMenuItem() { Detach(); }

   CString GetText() const         { return GetMenuString(m_hMenu, m_uItem, m_fByPosition); }
   BOOL    SetText(LPCTSTR szText) { return SetMenuString(m_hMenu, m_uItem, m_fByPosition, szText); }

   static CMenuItem* GetInstance(HMENU hMenu, UINT uItem, BOOL bByPosition);

   static BOOL SetMenuString(HMENU hMenu, UINT uItem, BOOL bByPosition, LPCTSTR szText);
   static void OnMeasureItem(MEASUREITEMSTRUCT*);        // WM_MEASUREITEM
   static void OnDrawItem(const DRAWITEMSTRUCT*);        // WM_DRAWITEM

   BOOL Detach(); //   OWNERDRAW   
private:
   BOOL Attach(); //       OWNERDRAW, ..       CMenuItem
protected:
   static BOOL CheckSign(CMenuItem *This);

protected:
   virtual BOOL GetLogFont(OUT LOGFONT&) const { return FALSE; }
   virtual BOOL DrawBkgrnd(const DRAWITEMSTRUCT&, HDC, const RECTEX&, bool isSelect, bool isGrayed, bool isSeparator) const { return FALSE; }
   virtual void PostDraw  (const DRAWITEMSTRUCT&, HDC, const RECTEX&, bool isSelect, bool isGrayed, bool isSeparator) const {}
};

#endif // __FILE__CMENUITEM__
