////////////////////////////////////////////////////////////////////////////////
//                               FastMines project
//                                       Sergey Krivulya (KSerg, aka SeregaLBN)
// file name: "MenuItemFM.cpp"
// ,      
// Copyright (C) 2006-2007 Sergey Krivulya
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
////////////////////////////////////////////////////////////////////////////////

#include "StdAfx.h"
#include "MenuItemFM.h"
#include "../FastMines2.h"

extern CFastMines2WinMain& GetFM2WinMain();

extern HBITMAP GetMenuItemBitmap(HMENU hMenu, UINT uItem, BOOL fByPosition);

CMenuItemFM::CMenuItemFM(BOOL bPopup, HMENU hMenu, UINT uItem, BOOL fByPosition, const CImage *pImg) :
   CMenuItemEx(bPopup, hMenu, uItem, fByPosition, pImg)
{
   /** /
   MENUITEMINFO mii = {sizeof(MENUITEMINFO), MIIM_BITMAP};
   BOOL bRes = ::GetMenuItemInfo(hMenu, uItem, fByPosition, &mii);
   if (bRes) {
      HBITMAP hBMP = mii.hbmpItem;
      if ((UINT)hBMP == 0x8) {
         m_Img = CSkin().m_ImgBtnNew[3]; // 
      }
   }
   /**/
}

BOOL CMenuItemFM::GetLogFont(OUT LOGFONT &logFont) const {
   if (GetFM2WinMain().GetSkin().IsDefaultFont()) return false;
   logFont = GetFM2WinMain().GetSkin().m_Font;
   return TRUE;
}

COLORREF CMenuItemFM::GetMenuColor() const {
   return GetFM2WinMain().GetSkin().m_bToAll ? GetFM2WinMain().GetSkin().GetBkColor() : ::GetSysColor(COLOR_BTNFACE);
}

BOOL CMenuItemIridescenceFM::GetLogFont(OUT LOGFONT &logFont) const {
   if (GetFM2WinMain().GetSkin().IsDefaultFont()) return false;
   logFont = GetFM2WinMain().GetSkin().m_Font;
   return TRUE;
}

COLORREF CMenuItemIridescenceFM::GetMenuColor() const {
   return GetFM2WinMain().GetSkin().m_bToAll ? GetFM2WinMain().GetSkin().GetBkColor() : ::GetSysColor(COLOR_BTNFACE);
}

////////////////////////////////////////////////////////////////////////////////


HRGN CMenuNCDrawFM::CreateRegionMenuUpdate(HWND hWnd) {
   HMENU hMenu = ::GetMenu(hWnd);
   if (!hMenu) return NULL;
   // 1.    
   int iItems = ::GetMenuItemCount(hMenu);
   RECTEX rcMenuBar; //       
   HRGN hRgnMenu = NULL; //   ,      
   RECTEX rcWin = ::GetWindowRect(hWnd);
   for (int i=0; i<iItems; i++) {
      RECTEX rcItem;
      ::GetMenuItemRect(hWnd, hMenu, i, &rcItem);
      if (i==0) {
         rcMenuBar.pointLT(rcItem.pointLT());                        //       
      } else
      if (i==iItems-1) {
         rcMenuBar.bottom = rcItem.bottom;                           //      
         rcMenuBar.right  = rcWin.right-(rcMenuBar.left-rcWin.left); //     
      }

      HRGN hRgnItem = ::CreateRectRgnIndirect(&rcItem);
      if (hRgnMenu == NULL) {
         hRgnMenu = hRgnItem;
      } else {
         int iRes = ::CombineRgn(hRgnMenu, hRgnMenu, hRgnItem, RGN_OR);
         ::DeleteObject(hRgnItem); hRgnItem = NULL;
      }
   }
   HRGN hRgnMenuBar = ::CreateRectRgnIndirect(&rcMenuBar); //    
   // 2.        -        
   int iRes = ::CombineRgn(hRgnMenu, hRgnMenuBar, hRgnMenu, RGN_DIFF); // 
   ::DeleteObject(hRgnMenuBar); hRgnMenuBar = NULL;

   return hRgnMenu;
}

/* static */ BOOL CMenuNCDrawFM::OnNCPaint(HWND hWnd, HRGN hRgn, bool bCallDefWinProc) {
   if (!::IsWindow(hWnd)) return FALSE;
   BOOL bHandlet = FALSE;
   COLORREF clr = GetFM2WinMain().GetSkin().m_bToAll ? GetFM2WinMain().GetSkin().GetBkColor() : ::GetSysColor(COLOR_BTNFACE); //     
   if ((clr!=::GetSysColor(COLOR_BTNFACE)) && hRgn && ::GetMenu(hWnd) && ::IsWindowVisible(hWnd)) {
      HWND hWnd4DC = hWnd;//NULL;//::GetDesktopWindow();//
      RECT rcWin = ::GetWindowRect(hWnd);

      // 1.      
      HRGN hRgnMenu  = CreateRegionMenuUpdate(hWnd); //    
      HRGN hRgnDraw  = ::CreateRectRgnIndirect(&RECTEX());   //  (  hRgnMenu),     
      HRGN hRgnOther = ::CreateRectRgnIndirect(&RECTEX());   //    ,     
      if ((WPARAM)hRgn == 1) { // When wParam is 1, the entire window frame needs to be updated.
         hRgn = ::CreateRectRgnIndirect(&rcWin);
         int iRes1 = ::CombineRgn(hRgnDraw , hRgnMenu,     NULL, RGN_COPY);
         int iRes2 = ::CombineRgn(hRgnOther, hRgn    , hRgnMenu, RGN_DIFF);
         ::DeleteObject(hRgn); hRgn = (HRGN)1;
      } else {
         int iRes1 = ::CombineRgn(hRgnDraw , hRgn, hRgnMenu, RGN_AND);
         int iRes2 = ::CombineRgn(hRgnOther, hRgn, hRgnMenu, RGN_DIFF);
      }

      // 2. 
      HDC hDC = ::GetWindowDC(hWnd4DC);//::GetDCEx(hWnd4DC, hRgnDraw, DCX_WINDOW|DCX_INTERSECTRGN);//

      /**/
      {
         HRGN hRgnMenuOffset = ::CreateRectRgnIndirect(&RECTEX());
         int iRes1 = ::CombineRgn(hRgnMenuOffset, hRgnMenu, NULL, RGN_COPY);
         int iRes2 = ::OffsetRgn(hRgnMenuOffset, -rcWin.left, -rcWin.top);
         int iRes3 = ::SelectClipRgn(hDC, hRgnMenuOffset);
         //LOG_PUT(g_pLog, Put, LL_DEBUG) _T("  OnNCPaint: iRes1=%i iRes2=%i iRes3=%i  "), iRes1, iRes2, iRes3);
         ::DeleteObject(hRgnMenuOffset);
      }/**/
      if (hDC)
      {
         RECTEX rc;
         int iRes2 = ::GetRgnBox(hRgnDraw, &rc);
         HDC hCDC = ::CreateCompatibleDC(hDC);
         HBITMAP hBmp = ::CreateCompatibleBitmap(hDC, rc.width(), rc.height());
         HGDIOBJ hBmpOld = ::SelectObject(hCDC, hBmp);
         HBRUSH  hBrush = ::CreateSolidBrush(clr);
         HGDIOBJ hBrushOld = ::SelectObject(hCDC, hBrush);

         bHandlet = TRUE; // 

         ::PatBlt(hCDC, 0,0,rc.width(),rc.height(), PATCOPY);                          //    
         ::BitBlt(hDC, RECTEX(rc).move(-rcWin.left, -rcWin.top), hCDC, SRCCOPY);       //    
         if (bCallDefWinProc) ::DefWindowProc(hWnd, WM_NCPAINT, (WPARAM)hRgnOther, 0); //    

         ::SelectObject(hCDC, hBrushOld);
         ::SelectObject(hCDC, hBmpOld);
         ::DeleteObject(hBrush);
         ::DeleteObject(hBmp);
      }
      ::DeleteObject(hRgnMenu ); hRgnMenu  = NULL;
      ::DeleteObject(hRgnDraw ); hRgnDraw  = NULL;
      ::DeleteObject(hRgnOther); hRgnOther = NULL;
      ::ReleaseDC(hWnd4DC, hDC); hDC = NULL;
   }
   return bHandlet;
}

/* static */ BOOL CMenuNCDrawFM::OnNCActivate(HWND hWnd, WPARAM wParam) {
   if (!::IsWindow(hWnd)) return FALSE;
   LRESULT lResult = ::DefWindowProc(hWnd, WM_NCACTIVATE, wParam, 0);
   //LOG_PUT(g_pLog, Put, LL_DEBUG) _T("  OnNCActivate: msg=%s wParam=%i return=%i"), (LPCTSTR)CLogger::WindowMessage(uMsg), wParam, lResult);
   CMenuNCDrawFM::OnNCPaint(hWnd, (HRGN)1, false);
   //HRGN hRgnMenu = CreateRegionMenuUpdate(hWnd);
   //::RedrawWindow(m_hWnd, NULL, hRgnMenu, RDW_FRAME | RDW_UPDATENOW | RDW_ALLCHILDREN);
   //::DeleteObject(hRgnMenu); hRgnMenu = NULL;
   return lResult;
}
