////////////////////////////////////////////////////////////////////////////////
//                               FastMines project
//                                       Sergey Krivulya (KSerg, aka SeregaLBN)
// file name: "TabItem.h"
// Copyright (C) 2006-2007 Sergey Krivulya
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
////////////////////////////////////////////////////////////////////////////////

#ifndef __FILE__CTABITEM__
#define __FILE__CTABITEM__

#include "../Image.h"
#include "../DynLoadLib.h"

class CTabItem: protected CWinXPTheme {
private:
   const DWORD m_dwSignature;
   static DWORD GetSignature() { return 0x2995D26D; }

protected:
   HWND   m_hWndTab;
   int    m_iItem; // Index of the tab
   CImage m_Img;

private:
   void OnMeasureItem(MEASUREITEMSTRUCT&); // WM_MEASUREITEM
   void OnDrawItem(const DRAWITEMSTRUCT&); // WM_DRAWITEM

public:
   CTabItem();
   CTabItem(HWND hWndTab, int iItem, const CImage *pImg = NULL);
  ~CTabItem() { Detach(); }

   static CTabItem* GetInstance(HWND hWndTab, int iItem);

   static void OnMeasureItem(HWND hWndTab, MEASUREITEMSTRUCT*); // WM_MEASUREITEM
   static void OnDrawItem(HWND hWndTab, const DRAWITEMSTRUCT*); // WM_DRAWITEM

   BOOL Detach();
   BOOL Attach(HWND hWndTab, int iItem, const CImage *pImg = NULL);
private:
   static BOOL CheckSign(CTabItem *This);

protected:
   virtual BOOL DrawBkgrnd(const DRAWITEMSTRUCT&, HDC, const RECTEX&, HTHEME) const { return FALSE; }
   virtual BOOL Draw      (const DRAWITEMSTRUCT&, HDC, const RECTEX&        ) const { return FALSE; }
};

#endif // __FILE__CTABITEM__
