////////////////////////////////////////////////////////////////////////////////
//                               FastMines project
//                                       Sergey Krivulya (KSerg, aka SeregaLBN)
// file name: "Players.h" -    
// Copyright (C) 2003-2007 Sergey Krivulya
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
////////////////////////////////////////////////////////////////////////////////

#ifndef __FILE__PLAYERS__
#define __FILE__PLAYERS__

#if !defined(__AFX_H__) && !defined(_WTL_USE_CSTRING)
   #include "./Lib/CStringKS.h"
#endif
#include "./SerializeImpl.h"
#include "./Mosaic.h"
#include "./Lang.h"

namespace nsPlayers {
   struct CSubRecord : public ISerialize {
      DWORD
         m_dwGameNumber, //   
         m_dwGameWin,    //   
         m_dwOpenField,  //     -     
         m_dwPlayTime,   //    -         
         m_dwClickCount; //    -       
      CSubRecord():
         m_dwGameNumber(0),
         m_dwGameWin   (0),
         m_dwOpenField (0),
         m_dwPlayTime  (0),
         m_dwClickCount(0) {};
      BOOL SerializeRead(HANDLE hFile) {
         DWORD dwNOBR = 0;
         return ::ReadFile(hFile, this, sizeof(*this), &dwNOBR, NULL) && (sizeof(*this) == dwNOBR);
      }
      BOOL SerializeWrite(HANDLE hFile) const {
         DWORD dwNOBW = 0;
         return ::WriteFile(hFile, this, sizeof(*this), &dwNOBW, NULL) && (sizeof(*this) == dwNOBW);
      }
   };

   struct CRecord : public ISerialize {
      CString    m_strPlayerName;
      CString    m_strPassword;
      CString    m_strFileAvatar;
      CSubRecord m_Record[eMosaicNil][eSkillLevelCustom];
      bool operator==(const CRecord &r) {
         return (m_strPlayerName == r.m_strPlayerName) &&
                (m_strPassword   == r.m_strPassword  ) &&
                (m_strFileAvatar == r.m_strFileAvatar);
      }
      BOOL SerializeRead(HANDLE hFile) {
         CString strPasswordB64;
         BOOL bRes = ::SerializeRead(hFile, m_strPlayerName) &&
                     ::SerializeRead(hFile, strPasswordB64 ) &&
                     ::SerializeRead(hFile, m_strFileAvatar);
         if (bRes) {
            if (strPasswordB64.IsEmpty()) { //    !
               m_strPassword.Empty();
            } else {
               TCHAR szPassword[32]; ZeroMemory(szPassword, sizeof(szPassword));
               DWORD dwSize = sizeof(szPassword)-sizeof(TCHAR);
               bRes = ::Base64_Decode(strPasswordB64, strPasswordB64.GetLength(), (BYTE*)szPassword, dwSize);
               if (bRes && (::GetLastError()==NO_ERROR)) {
                  m_strPassword = szPassword;
               }
            }
         }
         for (int i=0; bRes && i<eMosaicNil; i++)
            for (int j=0; bRes && j<eSkillLevelCustom; j++)
               bRes = m_Record[i][j].SerializeRead(hFile);
         return bRes;
      }
      BOOL SerializeWrite(HANDLE hFile) const {
         CString strPasswordB64; //    !
         BOOL bRes = 
            (m_strPassword.IsEmpty() ? TRUE :
            !(strPasswordB64=::Base64_Code(m_strPassword)).IsEmpty()) &&
            ::SerializeWrite(hFile, m_strPlayerName) &&
            ::SerializeWrite(hFile, strPasswordB64) &&
            ::SerializeWrite(hFile, m_strFileAvatar);
         for (int i=0; bRes && i<eMosaicNil; i++)
            for (int j=0; bRes && j<eSkillLevelCustom; j++)
               bRes = m_Record[i][j].SerializeWrite(hFile);
         return bRes;
      }
   };

   struct CBody: ISerialize { // serialize body
      std::vector<CRecord> m_Players;
      BOOL SerializeRead(HANDLE hFile) {
         BOOL bRes = TRUE;
         do {
            DWORD dwSizeToEOF = ::GetFileSize(hFile, NULL) - ::SetFilePointer(hFile, 0, NULL, FILE_CURRENT);
            if (dwSizeToEOF == 0) break;
            CRecord rec;
            bRes = rec.SerializeRead(hFile);
            if (bRes) m_Players.push_back(rec);
         } while(bRes);
         return bRes;
      }
      BOOL SerializeWrite(HANDLE hFile) const {
         BOOL bRes = TRUE;
         for (std::vector<CRecord>::const_iterator I=m_Players.begin(); bRes && (I!=m_Players.end()); I++) {
            bRes &= (*I).SerializeWrite(hFile);
         }
         return bRes;
      }
   };
} // namespace nsPlayers

// CPlayers -    +   
class CPlayers : public CCommonSerializeEx<IFileHeaderImpl<0xA53A4E79>, nsPlayers::CBody,
                                           IDS__PLAYERSandSTATISTIC__ERROR_CREATE,
                                           IDS__PLAYERSandSTATISTIC__ERROR_OPEN,
                                           IDS__PLAYERSandSTATISTIC__ERROR_READ,
                                           IDS__PLAYERSandSTATISTIC__ERROR_WRITE,
                                           IDS__PLAYERSandSTATISTIC__ERROR_VERSION>
{
private:
   CString GetFileName() const { return _T("Mines.stc"); }
   BOOL Convert(HANDLE hFile);
   BOOL IgnoreFileNotFound() const { return TRUE; }

public:
   CPlayers();
  ~CPlayers();

   bool IndexCheck(int index) const { return (index >= 0) && (index < NumberPlayers()); }

public:
   LPCTSTR GetPlayerName(int index)                    const { if (!IndexCheck(index)) return NULL ; return m_Body.m_Players[index].m_strPlayerName; }
   LPCTSTR GetPassword  (int index)                    const { if (!IndexCheck(index)) return NULL ; return m_Body.m_Players[index].m_strPassword; }
   bool    SetPassword  (int index, LPCTSTR szNewPassword)   { if (!IndexCheck(index)) return false;        m_Body.m_Players[index].m_strPassword   = szNewPassword  ; return true; }
   LPCTSTR GetAvatar    (int index)                    const { if (!IndexCheck(index)) return false; return m_Body.m_Players[index].m_strFileAvatar; }
   bool    SetAvatar    (int index, LPCTSTR szFileAvatar)    { if (!IndexCheck(index)) return false;        m_Body.m_Players[index].m_strFileAvatar = szFileAvatar   ; return true; }
   bool    RenamePlayer (int index, LPCTSTR szNewPlayerName) { if (!IndexCheck(index)) return false;        m_Body.m_Players[index].m_strPlayerName = szNewPlayerName; return true; }
   bool    RemovePlayer (int index);

   bool   PlayerExist (LPCTSTR szPlayerName) const { return FindName(szPlayerName) != -1; }
   size_t NumberPlayers() const { return m_Body.m_Players.size(); };
   size_t AddNewRecord(LPCTSTR szPlayerName, LPCTSTR szPassword);
   int    FindName    (LPCTSTR szPlayerName) const;
   bool   ModifyData  (LPCTSTR szPlayerName, const EMosaic, const ESkillLevel, bool bVictory, DWORD dwOpenField, DWORD dwPlayTime, DWORD dwClickCount);

   bool    RenamePlayer (LPCTSTR szPlayerName, LPCTSTR szNewPlayerName)    { return RenamePlayer(FindName(szPlayerName), szNewPlayerName); }
   bool    RemovePlayer (LPCTSTR szPlayerName)                             { return RemovePlayer(FindName(szPlayerName)); }
   LPCTSTR GetPassword  (LPCTSTR szPlayerName)                       const { return GetPassword (FindName(szPlayerName)); }
   bool    SetPassword  (LPCTSTR szPlayerName, LPCTSTR szNewPassword)      { return SetPassword (FindName(szPlayerName), szNewPassword); }
   LPCTSTR GetAvatar    (LPCTSTR szPlayerName)                       const { return GetAvatar   (FindName(szPlayerName)); }
   bool    SetAvatar    (LPCTSTR szPlayerName, LPCTSTR szFileAvatar)       { return SetAvatar   (FindName(szPlayerName), szFileAvatar); }

   DWORD GetInfo_GameNumber(int index, EMosaic mosaic, ESkillLevel skillLevel) const { if (!IndexCheck(index)) return 0; return m_Body.m_Players[index].m_Record[mosaic][skillLevel].m_dwGameNumber; }
   DWORD GetInfo_GameWin   (int index, EMosaic mosaic, ESkillLevel skillLevel) const { if (!IndexCheck(index)) return 0; return m_Body.m_Players[index].m_Record[mosaic][skillLevel].m_dwGameWin   ; }
   DWORD GetInfo_OpenField (int index, EMosaic mosaic, ESkillLevel skillLevel) const { if (!IndexCheck(index)) return 0; return m_Body.m_Players[index].m_Record[mosaic][skillLevel].m_dwOpenField ; }
   DWORD GetInfo_PlayTime  (int index, EMosaic mosaic, ESkillLevel skillLevel) const { if (!IndexCheck(index)) return 0; return m_Body.m_Players[index].m_Record[mosaic][skillLevel].m_dwPlayTime  ; }
   DWORD GetInfo_ClickCount(int index, EMosaic mosaic, ESkillLevel skillLevel) const { if (!IndexCheck(index)) return 0; return m_Body.m_Players[index].m_Record[mosaic][skillLevel].m_dwClickCount; }
};

#endif // __FILE__PLAYERS__
