// Implementations interfaces for FastMines project
// Copyright (C) 2004-2007 Sergey Krivulya
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#pragma once

#include "./Interface.h"
#include "./Lib/CommonLib.h"
#include "./Lang.h"

template <DWORD TSignature>
struct IFileHeaderImpl: public IFileHeader {
   DWORD   m_dwSignature;
   CString m_strVersion;
   static DWORD GetSignature() { return TSignature; }
   IFileHeaderImpl() :
      m_dwSignature(GetSignature()),
      m_strVersion(_T("FastMines Project - version ")+::GetFileVersion(::GetModuleFileName())) {}
   BOOL SerializeRead(HANDLE hFile) {
      DWORD dwNOBR = 0;
      BOOL bRes = (::ReadFile(hFile, &m_dwSignature, sizeof(m_dwSignature), &dwNOBR, NULL) && (sizeof(m_dwSignature) == dwNOBR) && (m_dwSignature == this->GetSignature())) &&
                   ::SerializeRead(hFile, m_strVersion);
      return bRes;
   }
   BOOL SerializeWrite(HANDLE hFile) const {
      DWORD dwNOBW = 0;
      BOOL bRes = (::WriteFile(hFile, &m_dwSignature, sizeof(m_dwSignature), &dwNOBW, NULL) && (sizeof(m_dwSignature) == dwNOBW)) &&
                   ::SerializeWrite(hFile, m_strVersion);
      return bRes;
   }
};

template <class CFileHeader, class CFileBody, int TID_ERROR_READ,
                                              int TID_ERROR_WRITE,
                                              int TID_ERROR_VERSION>
class CCommonSerialize : public ISerialize, public IOldVersion {
protected:
   CFileBody m_Body;
public:
   BOOL SerializeRead(HANDLE hFile) {
      int iIDString = 0;
      BOOL bRes = CFileHeader().SerializeRead(hFile);
      if (!bRes) {
         bRes = Convert(hFile); // old ver. -> new ver.
         if (!bRes) {
            iIDString = TID_ERROR_VERSION;
         }
      } else {
         bRes &= m_Body.SerializeRead(hFile);
         if (!bRes) {
            iIDString = TID_ERROR_READ;
         }
      }
      if (!bRes) {
         ::MessageBox(::GetActiveWindow(), CLang::GetString(iIDString), CLang::GetString(IDS__ERROR), MB_ICONERROR | MB_OK);
      }
      return bRes;
   }
   BOOL SerializeWrite(HANDLE hFile) const {
      BOOL bRes = CFileHeader().SerializeWrite(hFile) &&
                  m_Body.SerializeWrite(hFile);
      if (!bRes) {
         ::MessageBox(::GetActiveWindow(), CLang::GetString(TID_ERROR_WRITE), CLang::GetString(IDS__ERROR), MB_ICONSTOP | MB_OK);
      }
      return bRes;
   }
};

template <class CFileHeader, class CFileBody, int TID_ERROR_CREATE,
                                              int TID_ERROR_OPEN,
                                              int TID_ERROR_READ,
                                              int TID_ERROR_WRITE,
                                              int TID_ERROR_VERSION>
class CCommonSerializeEx : public CCommonSerialize<CFileHeader, CFileBody,
                                                   TID_ERROR_READ,
                                                   TID_ERROR_WRITE,
                                                   TID_ERROR_VERSION>, public IFile
{
public:
   BOOL LoadFile() {
      CString strFileName = this->GetFileName();
      if ((strFileName.Find(_T('\\')) != -1) || (strFileName.Find(_T('/')) != -1)) {
         //  
      } else {
         //   
         strFileName = ::GetModuleDir(::GetModuleHandle(NULL)) + strFileName;
      }
      HANDLE hFile = ::CreateFile(
         strFileName,
         GENERIC_READ,
         FILE_SHARE_READ,
         NULL,
         OPEN_EXISTING,
         FILE_ATTRIBUTE_NORMAL,
         NULL
      );
      BOOL bRes = (hFile != INVALID_HANDLE_VALUE);
      if (!bRes) {
         DWORD dwErrCode = ::GetLastError();
         bRes = ((ERROR_FILE_NOT_FOUND == dwErrCode) && IgnoreFileNotFound());
         if (!bRes) {
            ::MessageBox(::GetActiveWindow(), CLang::GetString(TID_ERROR_OPEN)+_T("\r\n\r\n")+strFileName+_T("\r\n")+::ErrorCode2Str(dwErrCode), CLang::GetString(IDS__ERROR), MB_ICONSTOP | MB_OK);
         }
      } else {
         bRes = SerializeRead(hFile);
         ::CloseHandle(hFile); hFile = NULL;
      }
      return bRes;
   }
   BOOL SaveFile() const {
      CString strFileName = this->GetFileName();
      if ((strFileName.Find(_T('\\')) != -1) || (strFileName.Find(_T('/')) != -1)) {
         //  
      } else {
         //   
         strFileName = ::GetModuleDir(::GetModuleHandle(NULL)) + strFileName;
      }
      HANDLE hFile = ::CreateFile(
         strFileName,
         GENERIC_WRITE,
         0,
         NULL,
         CREATE_ALWAYS,
         FILE_ATTRIBUTE_NORMAL,
         NULL
      );
      BOOL bRes = (hFile != INVALID_HANDLE_VALUE);
      if (!bRes)  {
         DWORD dwErrCode = ::GetLastError();
         ::MessageBox(::GetActiveWindow(), CLang::GetString(TID_ERROR_CREATE)+_T("\r\n\r\n")+strFileName+_T("\r\n")+::ErrorCode2Str(dwErrCode), CLang::GetString(IDS__ERROR), MB_ICONSTOP | MB_OK);
      } else {
         bRes = SerializeWrite(hFile);
         ::CloseHandle(hFile); hFile = NULL;
      }
      return bRes;
   }
};
